/*
 * callbacks.c -- gRustibus callbacks 
 *
 * gRustibus (c) 2000-2001 Kjetil Thuen <kjetilt@users.sourceforge.net>
 * MAME frontend for the GNOME desktop environment.
 *
 * gRustibus is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * This code is indented with GNU indent using k&r style with 8 character 
 * indents as suggested by the GNOME project. Please keep it that way.
 * ($ indent -kr -ts8 -i8 *.[ch])
 *
 * Last modified: 10 May 2001 <kjetilt@users.sourceforge.net>
 *
 */
#include <sys/wait.h>

#include "grustibus.h"
#include "fileio.h"
#include "mameio.h"
#include "gui.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include <ctype.h>


gint pid;
guint timeoutid, selected_row = -1, available_row = -1;
gboolean may_steal_focus = TRUE;
gboolean gamelist_focus = TRUE;
gboolean selection_is_record = TRUE;



gboolean
on_main_window_key_press_event(GtkWidget * widget,
                               GdkEventKey * event, gpointer user_data)
{
        /* Because we only want to catch keys that are pressed inside gRustibus
         * not, say, the release of a return pressed to quit another window
         */
        key_pressed = TRUE;
        return FALSE;
}

gboolean
on_main_window_key_release_event(GtkWidget * widget,
                                 GdkEventKey * event, gpointer user_data)
{

        /* Give focus to the gamelist when a letter or arrow is pressed */
        if ((key_pressed && may_steal_focus) &&
            ((event->keyval == GDK_BackSpace) ||
             (event->keyval == GDK_Up) ||
             (event->keyval == GDK_Down) ||
             (event->keyval == GDK_space) ||
             (((event->keyval) > 0x02f) && ((event->keyval) < 0x07b)))) {
                gtk_widget_grab_focus(GTK_WIDGET(appwin.list));
                return FALSE;
        }
        if (key_pressed
            && (event->keyval != GDK_Return
                && event->keyval != GDK_KP_Enter))
                key_pressed = FALSE;
        return TRUE;
}


/* Menu Callbacks */

void on_start_game_activate(GtkMenuItem * menuitem, gpointer user_data)
{
        key_pressed = FALSE;
        if (visible_games > 0) {
                start_game(general_prefs.current_row);
        }
}

void on_playback_game_activate(GtkMenuItem * menuitem, gpointer user_data)
{
        key_pressed = FALSE;
        if (visible_games > 0 && xmame_version_ok) {
                selection_is_record = FALSE;
                gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION
                                                       (fileselection));
                gtk_window_set_title(GTK_WINDOW(fileselection),
                                     _("gRustibus: Playback game"));
                gtk_widget_show(fileselection);
        }
}

void on_record_game_activate(GtkMenuItem * menuitem, gpointer user_data)
{
        RomEntry *romdata;
        gchar *game;
        gchar *defaultfilename = g_malloc(200);

        key_pressed = FALSE;
        if (visible_games > 0 && xmame_version_ok) {
                romdata =
                    gtk_clist_get_row_data(appwin.list,
                                           general_prefs.current_row);
                if (romdata) {
                        game = romdata->romname;
                        selection_is_record = TRUE;
                        g_snprintf(defaultfilename, 200,
                                   "%s/%s-%s-%i.%s.%i.inp",
                                   general_prefs.playback_dir,
                                   getenv("USER"), game,
                                   general_prefs.xmame_major,
                                   general_prefs.xmame_minor,
                                   general_prefs.xmame_release);
                        gtk_file_selection_set_filename(GTK_FILE_SELECTION
                                                        (fileselection),
                                                        defaultfilename);
                        gtk_file_selection_show_fileop_buttons
                            (GTK_FILE_SELECTION(fileselection));
                        gtk_window_set_title(GTK_WINDOW(fileselection),
                                             _("gRustibus: Record game"));
                        gtk_widget_show(fileselection);
                }
        }
        g_free(defaultfilename);
}

/* Callbakcs for the fileselection dialog */
void
on_fileselection_ok_button1_clicked(GtkButton * button, gpointer user_data)
{
        RomEntry *romdata;
        gchar *file;
        gchar *game;

        key_pressed = FALSE;
        romdata =
            gtk_clist_get_row_data(appwin.list, general_prefs.current_row);

        if (romdata) {
                game = romdata->romname;
                file =
                    gtk_file_selection_get_filename(GTK_FILE_SELECTION
                                                    (fileselection));
                gtk_widget_hide(fileselection);
                while (gtk_events_pending())
                        gtk_main_iteration();

                if (selection_is_record) {
                        record_game(file, game, romdata);
                } else {
                        playback_game(file, game, romdata);
                }
        }
}

void
 on_fileselection_cancel_button1_clicked
    (GtkButton * button, gpointer user_data) {
        gtk_widget_hide(fileselection);
}

void
on_select_random_game1_activate(GtkMenuItem * menuitem, gpointer user_data)
{
        key_pressed = FALSE;
        select_random_game();
}

void
on_rebuild_gamelist_activate(GtkMenuItem * menuitem, gpointer user_data)
{
        key_pressed = FALSE;
        build_new_game_list();
}

void on_exit_activate(GtkMenuItem * menuitem, gpointer user_data)
{
        key_pressed = FALSE;
        quit();
}

void
on_show_correct_roms_activate(GtkMenuItem * menuitem, gpointer user_data)
{
        key_pressed = FALSE;
        general_prefs.ListFilter.correct =
            GTK_CHECK_MENU_ITEM(menuitem)->active;
        rebuild_game_list();
}

void
on_show_incorect_roms_activate(GtkMenuItem * menuitem, gpointer user_data)
{
        key_pressed = FALSE;
        general_prefs.ListFilter.incorrect =
            GTK_CHECK_MENU_ITEM(menuitem)->active;
        rebuild_game_list();
}

void
on_show_missing_roms_activate(GtkMenuItem * menuitem, gpointer user_data)
{
        key_pressed = FALSE;
        general_prefs.ListFilter.missing =
            GTK_CHECK_MENU_ITEM(menuitem)->active;
        rebuild_game_list();
}

void
on_show_maywork_roms_activate(GtkMenuItem * menuitem, gpointer user_data)
{
        key_pressed = FALSE;
        general_prefs.ListFilter.may_work =
            GTK_CHECK_MENU_ITEM(menuitem)->active;
        rebuild_game_list();
}

void on_show_clones_activate(GtkMenuItem * menuitem, gpointer user_data)
{
        key_pressed = FALSE;
        general_prefs.ListFilter.clones =
            GTK_CHECK_MENU_ITEM(menuitem)->active;
        rebuild_game_list();
}

void
on_show_favourites_activate(GtkMenuItem * menuitem, gpointer user_data)
{
        key_pressed = FALSE;
        general_prefs.ListFilter.favourites =
            GTK_CHECK_MENU_ITEM(menuitem)->active;
        rebuild_game_list();
}

void
on_advanced_filters_activate(GtkMenuItem * menuitem, gpointer user_data)
{
        key_pressed = FALSE;
        general_prefs.ListFilter.advanced_filters_active =
            GTK_CHECK_MENU_ITEM(menuitem)->active;
        rebuild_game_list();
}

void
on_configure_advanced_filter_activate(GtkMenuItem * menuitem,
                                      gpointer user_data)
{
        key_pressed = FALSE;
        filter_dialog_reset();
        gtk_widget_show(filter_dialog);
}

void
on_screenshot_menuitem_activate(GtkMenuItem * menuitem, gpointer user_data)
{
        key_pressed = FALSE;
        if (general_prefs.notebook_page != SCREENSHOT_PAGE)
                gtk_notebook_set_page(appwin.notebook, SCREENSHOT_PAGE);
}

void
on_game_settings_menuitem_activate(GtkMenuItem * menuitem,
                                   gpointer user_data)
{
        key_pressed = FALSE;
        if (general_prefs.notebook_page != SETTINGS_PAGE)
                gtk_notebook_set_page(appwin.notebook, SETTINGS_PAGE);
}

void
on_game_info_menuitem_activate(GtkMenuItem * menuitem, gpointer user_data)
{
        key_pressed = FALSE;
        if (general_prefs.notebook_page != INFO_PAGE)
                gtk_notebook_set_page(appwin.notebook, INFO_PAGE);
}

void on_flyer_menuitem_activate(GtkMenuItem * menuitem, gpointer user_data)
{
        key_pressed = FALSE;
        if (general_prefs.notebook_page != FLYER_PAGE)
                gtk_notebook_set_page(appwin.notebook, FLYER_PAGE);
}

void
on_cabinet_photo_menuitem_activate(GtkMenuItem * menuitem,
                                   gpointer user_data)
{
        key_pressed = FALSE;
        if (general_prefs.notebook_page != CABINET_PAGE)
                gtk_notebook_set_page(appwin.notebook, CABINET_PAGE);
}

void on_preferences_activate(GtkMenuItem * menuitem, gpointer user_data)
{
        key_pressed = FALSE;
        prefs_reset(prefs_window);
        // FIXME: Must check if the window is already shown first.
        gtk_widget_show(prefs_window);
}

void
on_save_preferences_activate(GtkMenuItem * menuitem, gpointer user_data)
{
        key_pressed = FALSE;
        save_prefs();
}

void
on_save_window_state_activate(GtkMenuItem * menuitem, gpointer user_data)
{
        key_pressed = FALSE;
        save_window_prefs();
}

void
 on_save_window_state_automatically_on_quit1_activate
    (GtkMenuItem * menuitem, gpointer user_data) {
        key_pressed = FALSE;
        general_prefs.save_win_on_quit =
            GTK_CHECK_MENU_ITEM(menuitem)->active;
}

void
on_download_selected_rom_activate(GtkMenuItem * menuitem,
                                  gpointer user_data)
{
        RomEntry *romdata;
        gchar **buttons;

        key_pressed = FALSE;

        if (general_prefs.rom_url && strcmp (general_prefs.rom_url, "")) {
                buttons = g_new (gchar*, 3);
                buttons[0] = GNOME_STOCK_BUTTON_YES;
                buttons[1] = GNOME_STOCK_BUTTON_NO;
                buttons[2] = NULL;
        
                romdata = gtk_clist_get_row_data(appwin.list, general_prefs.current_row);
                if (romdata->status == CORRECT) {
                        if (!grustibus_question
                             (buttons,
                             _("This game is already working\nDo you want to download it anyway?")))
                        {
                                download_selected_rom(romdata);
                        }
                } else {
                        download_selected_rom(romdata);
                }
        }
        else {
                grustibus_warning (_("You have not specified an URL from which to download roms."));
        }
}

void
 on_download_selected_screenshot1_activate(GtkMenuItem * menuitem, 
                                           gpointer user_data) {
        RomEntry *romdata;
        key_pressed = FALSE;

        if (general_prefs.screenshot_url && strcmp (general_prefs.screenshot_url, "")) {
                romdata = gtk_clist_get_row_data(appwin.list, general_prefs.current_row);
                download_file (romdata->romname, general_prefs.screenshot_dir, general_prefs.screenshot_url);
        }
        else {
                grustibus_warning (_("You have not specified an URL from which to download screenshots."));
        }
}

void
on_download_selected_flyer_activate(GtkMenuItem * menuitem,
                                    gpointer user_data)
{
        RomEntry *romdata;
        key_pressed = FALSE;


        if (general_prefs.flyer_url && strcmp (general_prefs.flyer_url, "")) {
                romdata = gtk_clist_get_row_data(appwin.list, general_prefs.current_row);
                download_file (romdata->romname, general_prefs.flyer_dir, general_prefs.flyer_url);
        }
        else {
                grustibus_warning (_("You have not specified an URL from which to download flyers."));
        }
}

void
on_download_selected_cabinet_picture_activate
                                        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
        RomEntry *romdata;
        key_pressed = FALSE;

        if (general_prefs.cabinet_url && strcmp (general_prefs.cabinet_url, "")) {
                romdata = gtk_clist_get_row_data(appwin.list, general_prefs.current_row);
                download_file (romdata->romname, general_prefs.cabinet_dir, general_prefs.cabinet_url);
        }
        else {
                grustibus_warning (_("You have not specified an URL from which to download cabinet pictures."));
        }
}

void on_visit_mamedk_activate(GtkMenuItem * menuitem, gpointer user_data)
{
        key_pressed = FALSE;
        gnome_url_show("http://www.mame.dk");
}

void on_about_activate(GtkMenuItem * menuitem, gpointer user_data)
{
        GtkWidget *about_window;
        key_pressed = FALSE;
        about_window = create_about_window();
        gtk_widget_show(about_window);
}

void on_about_ok_clicked(GtkButton * button, gpointer user_data)
{
        key_pressed = FALSE;
        gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}

void
on_visit_grustibus_homepage_activate(GtkMenuItem * menuitem,
                                     gpointer user_data)
{
        key_pressed = FALSE;
        gnome_url_show("http://grustibus.sourceforge.net");
}

/* Buttonbar callbacks */

gboolean
on_start_game_button_release_event(GtkWidget * widget,
                                   GdkEventButton * event,
                                   gpointer user_data)
{
        if (visible_games > 0) {
             start_game(general_prefs.current_row);
        }
        return FALSE;
}

gboolean
on_preferences_button_release_event(GtkWidget * widget,
                                    GdkEventButton * event,
                                    gpointer user_data)
{
        if (!prefs_window) {
                prefs_window = create_prefs_window();
                prefs_startup(prefs_window);
        }
        gtk_widget_show(prefs_window);
        return FALSE;
}

gboolean
on_random_button_button_release_event(GtkWidget * widget,
                                      GdkEventButton * event,
                                      gpointer user_data)
{
        select_random_game();
        return FALSE;
}

gboolean
on_quit_button_release_event(GtkWidget * widget,
                             GdkEventButton * event, gpointer user_data)
{
        quit();
        return FALSE;
}


/* Listview callbacks */
void
on_game_clist_select_row(GtkCList * clist,
                         gint row,
                         gint column, GdkEventButton * event,
                         gpointer user_data)
{
        if (event && event->type == GDK_2BUTTON_PRESS) {
                lookup_free();
                gnome_appbar_pop(appwin.msgbar);
                start_game(row);

        } else if (event && event->type == GDK_BUTTON_PRESS) {
                if (row != general_prefs.current_row) {
                        lookup_free();
                        gnome_appbar_pop(appwin.msgbar);
                        rom_selected(clist, row);
                }
        } else {
                general_prefs.current_row = row;
                appwin.list->focus_row = general_prefs.current_row;

                /* Call rom_selected after a timeout */
                if (timeoutid)
                        gtk_timeout_remove(timeoutid);
                timeoutid =
                    gtk_timeout_add(SELECT_TIMEOUT,
                                    (GtkFunction) catch_alarm, NULL);
        }
}

gboolean catch_alarm()
{
        rom_selected(appwin.list, general_prefs.current_row);
        gtk_timeout_remove(timeoutid);
        return FALSE;
}

gboolean
on_game_clist_key_release_event(GtkWidget * widget,
                                GdkEventKey * event, gpointer user_data)
{
        gchar *search_char;
        gchar *search_string;
        /* Filter out modifiers */
        if (key_pressed && (visible_games > 0) && gamelist_focus && (!event->state || (event->state & GDK_SHIFT_MASK) ||        // Shift 
                                                                     (event->state & GDK_LOCK_MASK) ||  // Caps lock 
                                                                     (event->state & GDK_MOD2_MASK) ||  // Num lock 
                                                                     (event->state & GDK_MOD5_MASK))) { // Scroll lock 

                switch (event->keyval) {

                case GDK_KP_Enter:
                        lookup_free();
                        gnome_appbar_pop(appwin.msgbar);
                        start_game(general_prefs.current_row);
                        break;
                case GDK_Return:
                        lookup_free();
                        gnome_appbar_pop(appwin.msgbar);
                        start_game(general_prefs.current_row);
                        break;
                case GDK_Page_Up:
                        lookup_free();
                        gtk_widget_grab_focus(GTK_WIDGET(appwin.list));
                        gnome_appbar_pop(appwin.msgbar);
                        break;
                case GDK_Page_Down:
                        lookup_free();
                        gtk_widget_grab_focus(GTK_WIDGET(appwin.list));
                        gnome_appbar_pop(appwin.msgbar);
                        break;
                case GDK_Up:
                        lookup_free();
                        gtk_widget_grab_focus(GTK_WIDGET(appwin.list));
                        gnome_appbar_pop(appwin.msgbar);
                        break;
                case GDK_Down:
                        lookup_free();
                        gtk_widget_grab_focus(GTK_WIDGET(appwin.list));
                        gnome_appbar_pop(appwin.msgbar);
                        break;
                case GDK_BackSpace:
                        if (strlen(romlookup.search_string) > 0) {
                                lookup_pop();

                                select_game(romlookup.pos);
                                search_string =
                                    g_strconcat(_("Finding <"),
                                                romlookup.search_string,
                                                "#", NULL);
                                gnome_appbar_pop(appwin.msgbar);
                                gnome_appbar_push(appwin.msgbar,
                                                  search_string);
                                g_free(search_string);
                        }
                        break;
                default:
                        if ((((event->keyval) > 0x02f) & ((event->keyval) <
                                                          0x07b))
                            || (event->keyval == GDK_space)) {
                                search_char =
                                    g_strdup(gdk_keyval_name
                                             (event->keyval));

                                if (!strcmp(search_char, "space")) {
                                        g_free(search_char);
                                        search_char = g_strdup(" ");
                                }
                                search_string =
                                    g_strconcat(romlookup.search_string,
                                                search_char, NULL);

                                lookup_game(search_string);

                                g_free(search_string);
                                select_game(romlookup.pos);
                                search_string =
                                    g_strconcat(_("Finding >"),
                                                romlookup.search_string,
                                                "#", NULL);
                                gnome_appbar_pop(appwin.msgbar);
                                gnome_appbar_push(appwin.msgbar,
                                                  search_string);
                                g_free(search_char);
                                g_free(search_string);
                        }
                        break;
                }
        }

        key_pressed = FALSE;
        return FALSE;
}

gboolean
on_game_clist_focus_out_event(GtkWidget * widget,
                              GdkEventFocus * event, gpointer user_data)
{
        key_pressed = FALSE;
        gamelist_focus = FALSE;

        return FALSE;
}

gboolean
on_game_clist_focus_in_event(GtkWidget * widget,
                             GdkEventFocus * event, gpointer user_data)
{
        gamelist_focus = TRUE;

        return FALSE;
}

void
on_game_clist_resize_column(GtkCList * clist,
                            gint column, gint width, gpointer user_data)
{
        gint col_number;
        GList *tmplist, *column_list;
        CListColumnLabel *resized;

        tmplist = g_list_nth(general_prefs.active_columns, column);
        col_number = GPOINTER_TO_INT(tmplist->data);
        column_list = g_list_nth(CListColumnLabels, col_number);

        resized = column_list->data;
        resized->size = width;
}

void
 on_main_romlist_scrolledwindow_size_allocate
    (GtkWidget * widget, GtkAllocation * allocation, gpointer user_data) {
        general_prefs.pane_position = appwin.scrolled->allocation.width;
}

void
on_game_clist_click_column(GtkCList * clist, gint column,
                           gpointer user_data)
{
        lookup_free();
        gnome_appbar_pop(appwin.msgbar);
        set_sort_by_column (clist, column);
}

/* Current game settings tab callbacks */
void
on_default_settings_checkbutton_toggled(GtkToggleButton * togglebutton,
                                        gpointer user_data)
{
        RomEntry *romdata;
        gchar *game;

        if (visible_games > 0) {
                game_settings.default_options = togglebutton->active;
                gtk_widget_set_sensitive(lookup_widget
                                         (GTK_WIDGET(main_window),
                                          "vbox17"),
                                         !game_settings.default_options);

                /* This cludge is needed to avoid creating a default_settings_toggled loop */
                if (togglebutton->active) {
                        game_settings_set_to_default();
                        game_settings.default_options = TRUE;
                        set_game_settings_widgets();
                } else {
                        romdata =
                            gtk_clist_get_row_data(appwin.list,
                                                   general_prefs.
                                                   current_row);
                        if (romdata) {
                                game = romdata->romname;
                                load_game_settings(game);
                        }
                        game_settings.default_options = FALSE;
                        set_game_settings_widgets();

                }
        }
}

void
on_favourite_button_toggled(GtkToggleButton * togglebutton,
                            gpointer user_data)
{
        RomEntry *romdata;

        if (visible_games > 0) {
                romdata =
                    gtk_clist_get_row_data(appwin.list,
                                           general_prefs.current_row);
                if (romdata->favourite != togglebutton->active) {
                        if (togglebutton->active) {
                                romdata->favourite = TRUE;
                        } else {
                                romdata->favourite = FALSE;
                        }

                        update_gamelist_row (appwin.list, general_prefs.current_row);
                        save_favourites();
                        if (general_prefs.ListFilter.favourites)
                                rebuild_game_list();
                        gtk_clist_sort(appwin.list);
                }
        }
}

void
on_save_settings_button_released(GtkButton * button, gpointer user_data)
{
        RomEntry *romdata;
        gchar *game;
        gchar *gamename = g_malloc(50);

        if (visible_games > 0) {
                romdata =
                    gtk_clist_get_row_data(appwin.list,
                                           general_prefs.current_row);
                if (romdata) {
                        game = romdata->romname;
                        g_snprintf(gamename, 50, "/gRustibus.d/games/%s",
                                   game);
                        save_game_prefs(gamename, game_settings);
                }
                g_free(gamename);
        }
}

/* Display */
void
on_dga_checkbutton_toggled(GtkToggleButton * togglebutton,
                           gpointer user_data)
{
        game_settings.fullscreen = togglebutton->active;
}

void
on_scanlines_checkbutton_toggled(GtkToggleButton * togglebutton,
                                 gpointer user_data)
{
        game_settings.scanlines = togglebutton->active;
}

void
on_artwork_checkbutton_toggled(GtkToggleButton * togglebutton,
                               gpointer user_data)
{
        game_settings.extra_artwork = togglebutton->active;
}

void
on_autoframeskip_checkbutton_toggled(GtkToggleButton * togglebutton,
                                     gpointer user_data)
{
        game_settings.autoframeskip = togglebutton->active;
}

void
on_colordepth_checkbutton_toggled(GtkToggleButton * togglebutton,
                                  gpointer user_data)
{
        game_settings.auto_colordepth = togglebutton->active;
}

void
on_rotl_checkbutton_toggled(GtkToggleButton * togglebutton,
                            gpointer user_data)
{
        game_settings.rot_left = togglebutton->active;
}

void
on_rotr_checkbutton_toggled(GtkToggleButton * togglebutton,
                            gpointer user_data)
{
        game_settings.rot_right = togglebutton->active;

}

void
on_xflip_checkbutton_toggled(GtkToggleButton * togglebutton,
                             gpointer user_data)
{
        game_settings.flipx = togglebutton->active;
}

void
on_yflip_checkbutton_toggled(GtkToggleButton * togglebutton,
                             gpointer user_data)
{
        game_settings.flipy = togglebutton->active;
}

gboolean
on_scale_hscale_button_release_event(GtkWidget * widget,
                                     GdkEventButton * event,
                                     gpointer user_data)
{
        GtkAdjustment *adj = gtk_range_get_adjustment(GTK_RANGE(widget));
        game_settings.scale = (gint) GTK_ADJUSTMENT(adj)->value;
        return FALSE;
}

/* Vector */
void
on_vectorres_option_selected(GtkMenuShell * menu_shell, gpointer user_data)
{
        GtkWidget *active_item;
        gint item_index;

        active_item = gtk_menu_get_active(GTK_MENU(menu_shell));
        item_index = g_list_index(menu_shell->children, active_item);

        game_settings.vectorres = item_index;
}

void
 on_vector_antialias_checkbutton_toggled
    (GtkToggleButton * togglebutton, gpointer user_data) {
        game_settings.antialias = togglebutton->active;
}

void
 on_vector_translucency_checkbutton_toggled
    (GtkToggleButton * togglebutton, gpointer user_data) {
        game_settings.translucency = togglebutton->active;
}

gboolean
    on_vector_beam_hscale_button_release_event
    (GtkWidget * widget, GdkEventButton * event, gpointer user_data) {
        GtkAdjustment *adj = gtk_range_get_adjustment(GTK_RANGE(widget));
        game_settings.beam = (gfloat) GTK_ADJUSTMENT(adj)->value;
        return FALSE;
}

gboolean
    on_vector_flicker_hscale_button_release_event
    (GtkWidget * widget, GdkEventButton * event, gpointer user_data) {
        GtkAdjustment *adj = gtk_range_get_adjustment(GTK_RANGE(widget));
        game_settings.flicker = (gfloat) GTK_ADJUSTMENT(adj)->value;
        return FALSE;
}

/* Sound */
void
on_sound_checkbutton_toggled(GtkToggleButton * togglebutton,
                             gpointer user_data)
{
        game_settings.sound = togglebutton->active;
}

void
on_samples_checkbutton_toggled(GtkToggleButton * togglebutton,
                               gpointer user_data)
{
        game_settings.samples = togglebutton->active;
}

void
on_fakesound_checkbutton_toggled(GtkToggleButton * togglebutton,
                                 gpointer user_data)
{
        game_settings.fake_sound = togglebutton->active;
}

gboolean
on_volume_hscale_button_release_event(GtkWidget * widget,
                                      GdkEventButton * event,
                                      gpointer user_data)
{
        GtkAdjustment *adj = gtk_range_get_adjustment(GTK_RANGE(widget));
        game_settings.volume = (gint) GTK_ADJUSTMENT(adj)->value;
        return FALSE;
}

/* Input */
void
on_analog_joy_checkbutton_toggled(GtkToggleButton * togglebutton,
                                  gpointer user_data)
{
        game_settings.analog_joy = togglebutton->active;
}

void
on_mouse_checkbutton_toggled(GtkToggleButton * togglebutton,
                             gpointer user_data)
{
        game_settings.mouse = togglebutton->active;
}

void
on_winkeys_checkbutton_toggled(GtkToggleButton * togglebutton,
                               gpointer user_data)
{
        game_settings.winkeys = togglebutton->active;
}

void
on_grab_mouse_checkbutton_toggled(GtkToggleButton * togglebutton,
                                  gpointer user_data)
{
        game_settings.grab_mouse = togglebutton->active;
}

void on_joy_option_selected(GtkMenuShell * menu_shell, gpointer user_data)
{
        GtkWidget *active_item;
        gint item_index;

        active_item = gtk_menu_get_active(GTK_MENU(menu_shell));
        item_index = g_list_index(menu_shell->children, active_item);

        game_settings.joytype = item_index;
}

/* Misc */
/* Two functions to allow typing in the entrybox without the list picking up the keypresses */
gboolean
    on_extra_options_combo_entry_focus_in_event
    (GtkWidget * widget, GdkEventFocus * event, gpointer user_data) {
        may_steal_focus = FALSE;
        return FALSE;
}

gboolean
    on_extra_options_combo_entry_focus_out_event
    (GtkWidget * widget, GdkEventFocus * event, gpointer user_data) {
        may_steal_focus = TRUE;
        return FALSE;
}

void
on_cheat_checkbutton_toggled(GtkToggleButton * togglebutton,
                             gpointer user_data)
{
        game_settings.cheat = togglebutton->active;
}

void
on_extra_options_combo_entry_changed(GtkEditable * editable,
                                     gpointer user_data)
{
        if (game_settings.extra_options)
                g_free(game_settings.extra_options);
        game_settings.extra_options =
            gtk_editable_get_chars(GTK_EDITABLE(editable), 0, -1);

}

/* Prefs window callbacks */
void
on_prefs_checkbutton_toggled(GtkToggleButton * togglebutton,
                             gpointer user_data)
{
        gnome_property_box_changed(GNOME_PROPERTY_BOX(prefs_window));
}

gboolean
on_prefs_hscale_button_release_event(GtkWidget * widget,
                                     GdkEventButton * event,
                                     gpointer user_data)
{
        gnome_property_box_changed(GNOME_PROPERTY_BOX(prefs_window));
        return FALSE;
}

void
on_prefs_combo_entry_changed(GtkEditable * editable, gpointer user_data)
{
        gnome_property_box_changed(GNOME_PROPERTY_BOX(prefs_window));
}

void
on_prefs_colorpicker_color_set(GnomeColorPicker * gnomecolorpicker,
                               guint arg1,
                               guint arg2,
                               guint arg3, guint arg4, gpointer user_data)
{
        gnome_property_box_changed(GNOME_PROPERTY_BOX(prefs_window));
        need_to_rebuild_gamelist = TRUE;
        rebuild_prefs_list(prefs_window);
}

void
 on_prefs_favouritecolor_checkbutton_toggled
    (GtkToggleButton * togglebutton, gpointer user_data) {
        gnome_property_box_changed(GNOME_PROPERTY_BOX(prefs_window));
        need_to_rebuild_gamelist = TRUE;
        rebuild_prefs_list(prefs_window);
}

void
on_prefs_option_selected(GtkMenuShell * menu_shell, gpointer user_data)
{
        gnome_property_box_changed(GNOME_PROPERTY_BOX(prefs_window));
}

void
 on_listprefs_listfont_fontpicker_font_set
    (GnomeFontPicker * gnomefontpicker, GString arg1, gpointer user_data) {
        gnome_property_box_changed(GNOME_PROPERTY_BOX(prefs_window));
        gtk_entry_set_text(prefswin.fontentry, arg1.str);
        need_to_rebuild_gamelist = TRUE;
        rebuild_prefs_list(prefs_window);
}

void
on_prefs_listfont_font_entry_activate(GtkEditable * editable,
                                      gpointer user_data)
{
        gnome_property_box_changed(GNOME_PROPERTY_BOX(prefs_window));
        need_to_rebuild_gamelist = TRUE;
        rebuild_prefs_list(prefs_window);
}

void on_column_add_button_clicked(GtkButton * button, gpointer user_data)
{
        gchar *get;
        gchar **insert;
        gpointer *data;
        gint newrow = 0, number;

        insert = g_new(gchar *, 1);
        if (gtk_clist_get_text
            (prefswin.avail_col_list, available_row, 0, &get)) {
                data =
                    gtk_clist_get_row_data(prefswin.avail_col_list,
                                           available_row);
                number = GPOINTER_TO_INT(data);
                insert[0] = get;
                newrow =
                    gtk_clist_append(prefswin.select_col_list, insert);
                gtk_clist_set_row_data(prefswin.select_col_list, newrow,
                                       GINT_TO_POINTER(number));
                gtk_clist_remove(prefswin.avail_col_list, available_row);
                if (gtk_clist_get_selectable
                    (prefswin.avail_col_list, available_row)) {
                        gtk_clist_select_row(prefswin.avail_col_list,
                                             available_row, 0);
                        gtk_clist_moveto(prefswin.avail_col_list,
                                         available_row, 0, 0.5, 1.0);
                } else {
                        gtk_clist_select_row(prefswin.avail_col_list,
                                             available_row - 1, 0);
                        gtk_clist_moveto(prefswin.avail_col_list,
                                         available_row - 1, 0, 0.5, 1.0);
                }
                need_to_rebuild_gamelist_widget = TRUE;
                gnome_property_box_changed(GNOME_PROPERTY_BOX
                                           (prefs_window));
        }
}


void
on_column_remove_button_clicked(GtkButton * button, gpointer user_data)
{

        gchar *get;
        gchar **insert;
        gpointer *data;
        gint newrow = 0, number;

        insert = g_new(gchar *, 1);
        if (gtk_clist_get_text (prefswin.select_col_list, selected_row, 0, &get) && 
                        ((gtk_clist_get_text (prefswin.select_col_list, selected_row-1,0, NULL)) ||
                         (gtk_clist_get_text (prefswin.select_col_list, selected_row+1,0, NULL)))) {
                data =
                    gtk_clist_get_row_data(prefswin.select_col_list,
                                           selected_row);
                number = GPOINTER_TO_INT(data);
                insert[0] = get;
                newrow = gtk_clist_append(prefswin.avail_col_list, insert);
                gtk_clist_set_row_data(prefswin.avail_col_list, newrow,
                                       GINT_TO_POINTER(number));
                gtk_clist_remove(prefswin.select_col_list, selected_row);
                if (gtk_clist_get_selectable
                    (prefswin.select_col_list, selected_row)) {
                        gtk_clist_select_row(prefswin.select_col_list,
                                             selected_row, 0);
                        gtk_clist_moveto(prefswin.select_col_list,
                                         selected_row, 0, 0.5, 1.0);
                } else
                    if (gtk_clist_get_selectable
                        (prefswin.select_col_list, selected_row - 1)) {
                        gtk_clist_select_row(prefswin.select_col_list,
                                             selected_row - 1, 0);
                        gtk_clist_moveto(prefswin.select_col_list,
                                         selected_row - 1, 0, 0.5, 1.0);
                } else {
                        selected_row = -1;
                }
                need_to_rebuild_gamelist_widget = TRUE;
                gnome_property_box_changed(GNOME_PROPERTY_BOX
                                           (prefs_window));
        }
}

void on_column_up_button_clicked(GtkButton * button, gpointer user_data)
{
        if (gtk_clist_get_selectable
            (prefswin.select_col_list, selected_row - 1)) {
                gtk_clist_swap_rows(prefswin.select_col_list, selected_row,
                                    selected_row - 1);
                selected_row--;
                need_to_rebuild_gamelist_widget = TRUE;
                gnome_property_box_changed(GNOME_PROPERTY_BOX
                                           (prefs_window));
        }
}

void on_column_down_button_clicked(GtkButton * button, gpointer user_data)
{
        if (gtk_clist_get_selectable
            (prefswin.select_col_list, selected_row)
            && gtk_clist_get_selectable(prefswin.select_col_list,
                                        selected_row + 1)) {
                gtk_clist_swap_rows(prefswin.select_col_list, selected_row,
                                    selected_row + 1);
                selected_row++;
                need_to_rebuild_gamelist_widget = TRUE;
                gnome_property_box_changed(GNOME_PROPERTY_BOX
                                           (prefs_window));
        }
}

void
on_column_default_button_clicked(GtkButton * button, gpointer user_data)
{
        gchar *tmp_string = NULL, **tmp_array = NULL;
        gint i = 0;

        tmp_string = g_malloc(30);
        g_snprintf(tmp_string, 30, "0,1,2,3,4,5,6,7,8,9,10,11,12");
        tmp_array = g_strsplit(tmp_string, ",", COLUMN_COUNT);

        g_list_free (general_prefs.active_columns);
        general_prefs.active_columns=NULL;
        while (tmp_array[i] != NULL) {
                general_prefs.active_columns = g_list_append
                    (general_prefs.active_columns,
                     GINT_TO_POINTER(atoi(tmp_array[i])));
                i++;
        }
        g_free(tmp_string);
        g_strfreev(tmp_array);

        populate_column_lists(general_prefs.active_columns, prefswin.select_col_list, prefswin.avail_col_list);
        need_to_rebuild_gamelist_widget = TRUE;
        gnome_property_box_changed(GNOME_PROPERTY_BOX(prefs_window));
}


void
on_available_columns_list_select_row(GtkCList * clist,
                                     gint row,
                                     gint column,
                                     GdkEvent * event, gpointer user_data)
{
        available_row = row;
}

void
on_used_columns_list_select_row(GtkCList * clist,
                                gint row,
                                gint column,
                                GdkEvent * event, gpointer user_data)
{
        selected_row = row;
}


void
on_xmame_exe_combo_entry_changed(GtkEditable * editable,
                                 gpointer user_data)
{
        gnome_property_box_changed(GNOME_PROPERTY_BOX(prefs_window));
        need_to_make_new_gamelist = TRUE;
}

void
on_rompath_combo_entry_changed(GtkEditable * editable, gpointer user_data)
{
        gnome_property_box_changed(GNOME_PROPERTY_BOX(prefs_window));
        need_to_make_new_gamelist = TRUE;

}

/* FIXME: This function is run twice for some strange reason */
void
on_prefs_window_apply(GnomePropertyBox * gnomepropertybox,
                      gint arg1, gpointer user_data)
{
        GtkMenuShell *menu;
        GtkMenuItem *menuitem;
        guint8 r, g, b, a;
        gint n, number;
        gpointer *data;
        gchar **buttons;

        GtkAdjustment *scale_adj =
            gtk_range_get_adjustment(GTK_RANGE(prefswin.scale_hscale));
        GtkAdjustment *volume_adj =
            gtk_range_get_adjustment(GTK_RANGE(prefswin.volume_hscale));
        GtkAdjustment *beam_adj =
            gtk_range_get_adjustment(GTK_RANGE(prefswin.beam_hscale));
        GtkAdjustment *flicker_adj =
            gtk_range_get_adjustment(GTK_RANGE(prefswin.flicker_hscale));
        
        buttons = g_new (gchar*, 3);
        buttons[0]=GNOME_STOCK_BUTTON_YES;
        buttons[1]=GNOME_STOCK_BUTTON_NO;
        buttons[2]=NULL;

        /* This is run to check if the fonts are ok */
        /* TODO: Make a separate check_font function instead */
        rebuild_prefs_list(prefs_window);

        general_prefs.playback_no_highscore =
            prefswin.playback_no_highscore_togglebutton->active;
        general_prefs.iconify_while_xmame =
            prefswin.iconify_while_mame_togglebutton->active;
        general_prefs.loadsa_image_formats =
            prefswin.loadsa_image_formats_togglebutton->active;
        general_prefs.look_for_clone_images =
            prefswin.look_for_clone_images_togglebutton->active;
        general_prefs.indicate_notebook_changes =
            prefswin.indicate_notebook_changes_togglebutton->active;
        if (general_prefs.indicate_notebook_changes)
                notebooktab_status_show_all_but
                    (general_prefs.notebook_page);
        else
                notebooktab_status_hide_all();

        global_settings.fullscreen =
            prefswin.fullscreen_togglebutton->active;
        global_settings.scanlines =
            prefswin.scanlines_togglebutton->active;
        global_settings.extra_artwork =
            prefswin.x_artwork_togglebutton->active;
        global_settings.autoframeskip =
            prefswin.autoframeskip_togglebutton->active;
        global_settings.auto_colordepth =
            prefswin.auto_colordepth_togglebutton->active;
        global_settings.rot_left = prefswin.rotatel_togglebutton->active;
        global_settings.rot_right = prefswin.rotater_togglebutton->active;
        global_settings.flipx = prefswin.flipx_togglebutton->active;
        global_settings.flipy = prefswin.flipy_togglebutton->active;
        global_settings.scale = (gint) GTK_ADJUSTMENT(scale_adj)->value;

        global_settings.antialias =
            prefswin.antialias_togglebutton->active;
        global_settings.translucency =
            prefswin.translucency_togglebutton->active;
        global_settings.beam = (gfloat) GTK_ADJUSTMENT(beam_adj)->value;
        global_settings.flicker =
            (gfloat) GTK_ADJUSTMENT(flicker_adj)->value;
        menu =
            GTK_MENU_SHELL(gtk_option_menu_get_menu
                           (prefswin.vectorres_option));
        menuitem = GTK_MENU_ITEM(gtk_menu_get_active(GTK_MENU(menu)));
        global_settings.vectorres = g_list_index(menu->children, menuitem);

        global_settings.sound = prefswin.sound_togglebutton->active;
        global_settings.samples = prefswin.samples_togglebutton->active;
        global_settings.fake_sound =
            prefswin.fakesound_togglebutton->active;
        global_settings.volume = (gint) GTK_ADJUSTMENT(volume_adj)->value;

        menu =
            GTK_MENU_SHELL(gtk_option_menu_get_menu(prefswin.joy_option));
        menuitem = GTK_MENU_ITEM(gtk_menu_get_active(GTK_MENU(menu)));
        global_settings.joytype = g_list_index(menu->children, menuitem);
        global_settings.analog_joy =
            prefswin.analog_joy_togglebutton->active;
        global_settings.mouse = prefswin.mouse_togglebutton->active;
        global_settings.winkeys = prefswin.winkeys_togglebutton->active;
        global_settings.grab_mouse =
            prefswin.grab_mouse_togglebutton->active;

        global_settings.cheat = prefswin.cheat_togglebutton->active;
        if (global_settings.extra_options)
                g_free(global_settings.extra_options);
        global_settings.extra_options =
            gtk_editable_get_chars(GTK_EDITABLE(prefswin.extra_options), 0,
                                   -1);

        general_prefs.favourites_overrides =
            prefswin.favourite_overrides_togglebutton->active;
        gnome_color_picker_get_i8(prefswin.correct_colorpicker, &r, &g, &b,
                                  &a);
        general_prefs.correct_color_r = r;
        general_prefs.correct_color_b = b;
        general_prefs.correct_color_g = g;
        gnome_color_picker_get_i8(prefswin.incorrect_colorpicker, &r, &g,
                                  &b, &a);
        general_prefs.incorrect_color_r = r;
        general_prefs.incorrect_color_b = b;
        general_prefs.incorrect_color_g = g;
        gnome_color_picker_get_i8(prefswin.missing_colorpicker, &r, &g, &b,
                                  &a);
        general_prefs.missing_color_r = r;
        general_prefs.missing_color_b = b;
        general_prefs.missing_color_g = g;
        gnome_color_picker_get_i8(prefswin.favourite_colorpicker, &r, &g,
                                  &b, &a);
        general_prefs.favourite_color_r = r;
        general_prefs.favourite_color_b = b;
        general_prefs.favourite_color_g = g;
        gnome_color_picker_get_i8(prefswin.may_work_colorpicker, &r, &g,
                                  &b, &a);
        general_prefs.may_work_color_r = r;
        general_prefs.may_work_color_b = b;
        general_prefs.may_work_color_g = g;

        g_free(general_prefs.font);
        general_prefs.font =
            gtk_editable_get_chars(GTK_EDITABLE(prefswin.fontentry), 0,
                                   MAX_FONTNAME);

        n = 0;
        general_prefs.active_columns = NULL;
        while (gtk_clist_get_text(prefswin.select_col_list, n, 0, NULL)) {
                data = gtk_clist_get_row_data(prefswin.select_col_list, n);
                number = GPOINTER_TO_INT(data);
                general_prefs.active_columns =
                    g_list_prepend(general_prefs.active_columns,
                                   GINT_TO_POINTER(number));
                n++;
        }
        general_prefs.active_columns =
            g_list_reverse(general_prefs.active_columns);

        if (general_prefs.xmame_exe)
                g_free(general_prefs.xmame_exe);
        general_prefs.xmame_exe =
            gtk_editable_get_chars(GTK_EDITABLE(prefswin.xmame_exe_entry),
                                   0, -1);

        if (general_prefs.rom_dir)
                g_free(general_prefs.rom_dir);
        general_prefs.rom_dir =
            gtk_editable_get_chars(GTK_EDITABLE(prefswin.rom_path_entry),
                                   0, MAX_DIR);

        if (general_prefs.highscore_dir)
                g_free(general_prefs.highscore_dir);
        general_prefs.highscore_dir =
            gtk_editable_get_chars(GTK_EDITABLE
                                   (prefswin.highscore_dir_entry), 0,
                                   MAX_DIR);

        if (general_prefs.game_history_file)
                g_free(general_prefs.game_history_file);
        general_prefs.game_history_file =
            gtk_editable_get_chars(GTK_EDITABLE
                                   (prefswin.gamehistory_path_entry), 0,
                                   -1);

        if (general_prefs.screenshot_dir)
                g_free(general_prefs.screenshot_dir);
        general_prefs.screenshot_dir =
            gtk_editable_get_chars(GTK_EDITABLE
                                   (prefswin.screenshots_dir_entry), 0,
                                   MAX_DIR);

        if (general_prefs.flyer_dir)
                g_free(general_prefs.flyer_dir);
        general_prefs.flyer_dir =
            gtk_editable_get_chars(GTK_EDITABLE(prefswin.flyers_dir_entry),
                                   0, MAX_DIR);

        if (general_prefs.cabinet_dir)
                g_free(general_prefs.cabinet_dir);
        general_prefs.cabinet_dir =
            gtk_editable_get_chars(GTK_EDITABLE
                                   (prefswin.cabinets_dir_entry), 0,
                                   MAX_DIR);

        if (general_prefs.cheat_file)
                g_free(general_prefs.cheat_file);
        general_prefs.cheat_file =
            gtk_editable_get_chars(GTK_EDITABLE
                                   (prefswin.cheatfile_path_entry), 0, -1);

        
        if (general_prefs.playback_dir)
                g_free(general_prefs.playback_dir);
        general_prefs.playback_dir =
                gtk_editable_get_chars(GTK_EDITABLE
                                   (prefswin.playback_dir_entry), 0,
                                   MAX_DIR);

        if (general_prefs.rom_url) g_free (general_prefs.rom_url);
        general_prefs.rom_url = gtk_editable_get_chars (GTK_EDITABLE (prefswin.rom_url_entry), 0, MAX_URL);

        if (general_prefs.screenshot_url) g_free (general_prefs.screenshot_url);
        general_prefs.screenshot_url = gtk_editable_get_chars (GTK_EDITABLE (prefswin.screenshot_url_entry), 0, MAX_URL);

        if (general_prefs.flyer_url) g_free (general_prefs.flyer_url);
        general_prefs.flyer_url = gtk_editable_get_chars (GTK_EDITABLE (prefswin.flyer_url_entry), 0, MAX_URL);

        if (general_prefs.cabinet_url) g_free (general_prefs.cabinet_url);
        general_prefs.cabinet_url = gtk_editable_get_chars (GTK_EDITABLE (prefswin.cabinet_url_entry), 0, MAX_URL);

        
        
        if (need_to_make_new_gamelist) {
                if (check_xmame_exe() && xmame_version_ok){
                        if (!grustibus_question(buttons, _("You have changed the xmame executable.\nYou need to rebuild the gamelist\nWould you like to rebuild the list now?"))){
                                gtk_widget_hide(prefs_window);
                                build_new_game_list(); 
                                gtk_widget_show(prefs_window);
                        }
                }
                else if (xmame_version_ok && check_exists_dirs(general_prefs.rom_dir)){
                        if (!grustibus_question(buttons, _("You have changed the rompath.\nYou need to rebuild the gamelist.\nWould you like to rebuild the list now?"))){
                                gtk_widget_hide(prefs_window);
                                build_new_game_list(); 
                                gtk_widget_show(prefs_window);
                         }
                }
                grustibus_appbar_set_default();
                need_to_make_new_gamelist = FALSE;
        }
        if (need_to_rebuild_gamelist_widget) {
                create_gamelist_widget(&appwin.list);
                rebuild_game_list();
                need_to_rebuild_gamelist = FALSE;
        }

        if (need_to_rebuild_gamelist) {
                rebuild_game_list();
        }

        check_exists_dirs (general_prefs.screenshot_dir);
        check_exists_dirs (general_prefs.flyer_dir);
        check_exists_dirs (general_prefs.cabinet_dir);
        check_exists_dirs (general_prefs.highscore_dir);
        check_exists_dirs (general_prefs.playback_dir);
        need_to_rebuild_gamelist = FALSE;
        need_to_rebuild_gamelist_widget = FALSE;
        need_to_save_prefs = TRUE;
        select_game(general_prefs.current_row);
}

gint on_prefs_window_close(GnomeDialog * gnomedialog, gpointer user_data)
{
        gtk_widget_hide(prefs_window);
        return (TRUE);
}

void
on_prefs_window_help(GnomePropertyBox * gnomepropertybox,
                     gint arg1, gpointer user_data)
{
        grustibus_warning(_("The documentation is yet to be written."));
}

/* Main Window */
gboolean
on_main_window_delete_event(GtkWidget * widget,
                            GdkEvent * event, gpointer user_data)
{
        quit();
        return TRUE;
}

void
on_main_notebook_switch_page(GtkNotebook * notebook,
                             GtkNotebookPage * page,
                             gint page_num, gpointer user_data)
{
        /* There is no need to do anything about the contents of the pages
         * from this function, since cahnging pages also creates a size_allocate
         * event. And the pages are dealt with from that callback.
         */
        GtkCheckMenuItem *menuitem = NULL;

        if (page_num != general_prefs.notebook_page) {
                switch (page_num) {
                case (SCREENSHOT_PAGE):
                        menuitem =
                            GTK_CHECK_MENU_ITEM(gtk_object_get_data
                                                (GTK_OBJECT(main_window),
                                                 "screenshot_menuitem"));
                        break;
                case (SETTINGS_PAGE):
                        menuitem =
                            GTK_CHECK_MENU_ITEM(gtk_object_get_data
                                                (GTK_OBJECT(main_window),
                                                 "game_settings_menuitem"));
                        break;
                case (INFO_PAGE):
                        menuitem =
                            GTK_CHECK_MENU_ITEM(gtk_object_get_data
                                                (GTK_OBJECT(main_window),
                                                 "game_info_menuitem"));
                        break;
                case (FLYER_PAGE):
                        menuitem =
                            GTK_CHECK_MENU_ITEM(gtk_object_get_data
                                                (GTK_OBJECT(main_window),
                                                 "flyer_menuitem"));
                        break;
                case (CABINET_PAGE):
                        menuitem =
                            GTK_CHECK_MENU_ITEM(gtk_object_get_data
                                                (GTK_OBJECT(main_window),
                                                 "cabinet_photo_menuitem"));
                        break;
                }
                general_prefs.notebook_page = page_num;
                if (general_prefs.indicate_notebook_changes)
                        notebooktab_status_show_all_but(page_num);
                gtk_menu_item_activate(GTK_MENU_ITEM(menuitem));
        }
}

void
on_main_notebook_size_allocate(GtkWidget * widget,
                               GtkAllocation * allocation,
                               gpointer user_data)
{
        RomEntry *romdata;
        gchar *game;
        gchar *filename = NULL;
        gint newysize;
        gint newxsize;
        gint diff = 0;

        romdata =
            gtk_clist_get_row_data(appwin.list, general_prefs.current_row);
        if (romdata) {
                game = romdata->romname;
                gdk_window_get_size(main_window->window, &newxsize,
                                    &newysize);

                /* FIXME: The pictures are drawn even when the window is just resized over the x axis.
                 * This is because a second size_allocate event is created when i set the pane_pos after
                 * the window has been resized. Maybe I need to create a new global var keeping a boolean
                 * value, saying if the event has been handled or not....
                 */

                /* Dont bother with the images if there is no need to */
                if ((newysize != general_prefs.y_size) || (diff == 0) ||
                    (gtk_notebook_get_current_page(appwin.notebook) !=
                     general_prefs.notebook_page)) {
                        if (visible_games > 0) {
                                switch (gtk_notebook_get_current_page
                                        (appwin.notebook)) {
                                case (SCREENSHOT_PAGE):
                                        if (strcmp
                                            (game,
                                             screenshot_canvas.name)) {
                                                find_image
                                                    (general_prefs.
                                                     screenshot_dir, game,
                                                     romdata->cloneof,
                                                     &filename);
                                        }
                                        draw_image(&screenshot_canvas,
                                                   game, filename);
                                        break;
                                case (SETTINGS_PAGE):
                                        set_game_settings_widgets();
                                        break;
                                case (INFO_PAGE):
                                        set_game_history(game);
                                        set_game_info_widgets();
                                        break;
                                case (FLYER_PAGE):
                                        if (strcmp
                                            (game, flyer_canvas.name)) {
                                                find_image
                                                    (general_prefs.
                                                     flyer_dir, game,
                                                     romdata->cloneof,
                                                     &filename);
                                        }
                                        draw_image(&flyer_canvas, game,
                                                   filename);
                                        break;
                                case (CABINET_PAGE):
                                        if (strcmp
                                            (game, cabinet_canvas.name)) {
                                                find_image
                                                    (general_prefs.
                                                     cabinet_dir, game,
                                                     romdata->cloneof,
                                                     &filename);
                                        }
                                        draw_image(&cabinet_canvas, game,
                                                   filename);
                                        break;
                                }
                                if (filename)
                                        g_free(filename);
                        }
                }
        }

        general_prefs.notebook_page =
            gtk_notebook_get_current_page(appwin.notebook);
        general_prefs.x_size = newxsize;
        general_prefs.y_size = newysize;
}

/* Advanced filters dialog callbacks */
void
 on_filter_manufacturer_checkbutton_toggled
    (GtkToggleButton * togglebutton, gpointer user_data) {
        gtk_widget_set_sensitive(lookup_widget
                                 (GTK_WIDGET(filter_dialog),
                                  "filter_manufacturer_hbox"),
                                 togglebutton->active);
}

void
on_filter_driver_checkbutton_toggled(GtkToggleButton * togglebutton,
                                     gpointer user_data)
{
        gtk_widget_set_sensitive(lookup_widget
                                 (GTK_WIDGET(filter_dialog),
                                  "filter_driver_hbox"),
                                 togglebutton->active);
}

void
on_filter_category_checkbutton_toggled(GtkToggleButton * togglebutton,
                                       gpointer user_data)
{
        gtk_widget_set_sensitive(lookup_widget
                                 (GTK_WIDGET(filter_dialog),
                                  "filter_category_hbox"),
                                 togglebutton->active);
}

void
on_filter_year_checkbutton_toggled(GtkToggleButton * togglebutton,
                                   gpointer user_data)
{
        gtk_widget_set_sensitive(lookup_widget
                                 (GTK_WIDGET(filter_dialog),
                                  "filter_year_hbox"),
                                 togglebutton->active);
}

void
on_filter_players_checkbutton_toggled(GtkToggleButton * togglebutton,
                                      gpointer user_data)
{
        gtk_widget_set_sensitive(lookup_widget
                                 (GTK_WIDGET(filter_dialog),
                                  "filter_players_hbox"),
                                 togglebutton->active);
}

void
on_filter_controls_checkbutton_toggled(GtkToggleButton * togglebutton,
                                       gpointer user_data)
{
        gtk_widget_set_sensitive(lookup_widget
                                 (GTK_WIDGET(filter_dialog),
                                  "filter_controls_hbox"),
                                 togglebutton->active);
}

void
 on_filter_driver_status_checkbutton_toggled
    (GtkToggleButton * togglebutton, gpointer user_data) {
        gtk_widget_set_sensitive(lookup_widget
                                 (GTK_WIDGET(filter_dialog),
                                  "filter_driver_status_hbox"),
                                 togglebutton->active);
}

void
on_filter_video_type_toggled(GtkToggleButton * togglebutton,
                             gpointer user_data)
{
        gtk_widget_set_sensitive(lookup_widget
                                 (GTK_WIDGET(filter_dialog),
                                  "filter_video_type_hbox"),
                                 togglebutton->active);
}

void
 on_filter_screenshot_checkbutton_toggled
    (GtkToggleButton * togglebutton, gpointer user_data) {
        gtk_widget_set_sensitive(lookup_widget
                                 (GTK_WIDGET(filter_dialog),
                                  "filter_screenshot_hbox"),
                                 togglebutton->active);
}


void
on_filter_flyer_checkbutton_toggled(GtkToggleButton * togglebutton,
                                    gpointer user_data)
{
        gtk_widget_set_sensitive(lookup_widget
                                 (GTK_WIDGET(filter_dialog),
                                  "filter_flyer_hbox"),
                                 togglebutton->active);

}


void
on_filter_cabinet_checkbutton_toggled(GtkToggleButton * togglebutton,
                                      gpointer user_data)
{
        gtk_widget_set_sensitive(lookup_widget
                                 (GTK_WIDGET(filter_dialog),
                                  "filter_cabinet_hbox"),
                                 togglebutton->active);
}


void on_filter_apply_button_clicked(GtkButton * button, gpointer user_data)
{
        GtkWidget *tmpwidget;

        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_driver_status_checkbutton");
        general_prefs.ListFilter.driver_status_active =
            GTK_TOGGLE_BUTTON(tmpwidget)->active;
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_status_good_radiobutton");
        general_prefs.ListFilter.driver_status_good =
            GTK_TOGGLE_BUTTON(tmpwidget)->active;

        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_video_type_checkbutton");
        general_prefs.ListFilter.video_type_active =
            GTK_TOGGLE_BUTTON(tmpwidget)->active;
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_video_type_vector_radiobutton");
        general_prefs.ListFilter.video_type_vector =
            GTK_TOGGLE_BUTTON(tmpwidget)->active;

        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_year_checkbutton");
        general_prefs.ListFilter.year_active =
            GTK_TOGGLE_BUTTON(tmpwidget)->active;
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_year_lt_radiobutton");
        if (GTK_TOGGLE_BUTTON(tmpwidget)->active)
                general_prefs.ListFilter.year_operator = 0;
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_year_gt_radiobutton");
        if (GTK_TOGGLE_BUTTON(tmpwidget)->active)
                general_prefs.ListFilter.year_operator = 1;
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_year_ne_radiobutton");
        if (GTK_TOGGLE_BUTTON(tmpwidget)->active)
                general_prefs.ListFilter.year_operator = 2;
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_year_e_radiobutton");
        if (GTK_TOGGLE_BUTTON(tmpwidget)->active)
                general_prefs.ListFilter.year_operator = 3;
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_year_spinbutton");
        general_prefs.ListFilter.year_value =
            gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(tmpwidget));

        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_players_checkbutton");
        general_prefs.ListFilter.players_active =
            GTK_TOGGLE_BUTTON(tmpwidget)->active;
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_players_lt_radiobutton");
        if (GTK_TOGGLE_BUTTON(tmpwidget)->active)
                general_prefs.ListFilter.players_operator = 0;
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_players_gt_radiobutton");
        if (GTK_TOGGLE_BUTTON(tmpwidget)->active)
                general_prefs.ListFilter.players_operator = 1;
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_players_ne_radiobutton");
        if (GTK_TOGGLE_BUTTON(tmpwidget)->active)
                general_prefs.ListFilter.players_operator = 2;
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_players_e_radiobutton");
        if (GTK_TOGGLE_BUTTON(tmpwidget)->active)
                general_prefs.ListFilter.players_operator = 3;
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_players_spinbutton");
        general_prefs.ListFilter.players_value =
            gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(tmpwidget));

        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_controls_checkbutton");
        general_prefs.ListFilter.controls_active =
            GTK_TOGGLE_BUTTON(tmpwidget)->active;
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_controls_not_checkbutton");
        general_prefs.ListFilter.not_controls_active =
            GTK_TOGGLE_BUTTON(tmpwidget)->active;
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_controls_combo_entry");
        if (general_prefs.ListFilter.controls_value)
                g_free(general_prefs.ListFilter.controls_value);
        general_prefs.ListFilter.controls_value =
            gtk_editable_get_chars(GTK_EDITABLE(tmpwidget), 0, -1);

        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_manufacturer_checkbutton");
        general_prefs.ListFilter.manufacturer_active =
            GTK_TOGGLE_BUTTON(tmpwidget)->active;
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_manufacturer_not_checkbutton");
        general_prefs.ListFilter.not_manufacturer_active =
            GTK_TOGGLE_BUTTON(tmpwidget)->active;
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_manufacturer_combo_entry");
        if (general_prefs.ListFilter.manufacturer_value)
                g_free(general_prefs.ListFilter.manufacturer_value);
        general_prefs.ListFilter.manufacturer_value =
            gtk_editable_get_chars(GTK_EDITABLE(tmpwidget), 0, -1);

        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_driver_checkbutton");
        general_prefs.ListFilter.driver_active =
            GTK_TOGGLE_BUTTON(tmpwidget)->active;
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_driver_not_checkbutton");
        general_prefs.ListFilter.not_driver_active =
            GTK_TOGGLE_BUTTON(tmpwidget)->active;
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_driver_combo_entry");
        if (general_prefs.ListFilter.driver_value)
                g_free(general_prefs.ListFilter.driver_value);
        general_prefs.ListFilter.driver_value =
            gtk_editable_get_chars(GTK_EDITABLE(tmpwidget), 0, -1);

        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_category_checkbutton");
        general_prefs.ListFilter.category_active =
            GTK_TOGGLE_BUTTON(tmpwidget)->active;
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_category_not_checkbutton");
        general_prefs.ListFilter.not_category_active =
            GTK_TOGGLE_BUTTON(tmpwidget)->active;
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_category_combo_entry");
        if (general_prefs.ListFilter.category_value)
                g_free(general_prefs.ListFilter.category_value);
        general_prefs.ListFilter.category_value =
            gtk_editable_get_chars(GTK_EDITABLE(tmpwidget), 0, -1);

        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_screenshot_checkbutton");
        general_prefs.ListFilter.screenshot_active =
            GTK_TOGGLE_BUTTON(tmpwidget)->active;
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_screenshot_not_installed_radiobutton");
        general_prefs.ListFilter.not_screenshot_active =
            GTK_TOGGLE_BUTTON(tmpwidget)->active;

        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_flyer_checkbutton");
        general_prefs.ListFilter.flyer_active =
            GTK_TOGGLE_BUTTON(tmpwidget)->active;
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_flyer_not_installed_radiobutton");
        general_prefs.ListFilter.not_flyer_active =
            GTK_TOGGLE_BUTTON(tmpwidget)->active;

        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_cabinet_checkbutton");
        general_prefs.ListFilter.cabinet_active =
            GTK_TOGGLE_BUTTON(tmpwidget)->active;
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_cabinet_not_installed_radiobutton");
        general_prefs.ListFilter.not_cabinet_active =
            GTK_TOGGLE_BUTTON(tmpwidget)->active;


        gtk_widget_hide(filter_dialog);
        if (general_prefs.ListFilter.advanced_filters_active)
                rebuild_game_list();
}

void
on_filter_cancel_button_clicked(GtkButton * button, gpointer user_data)
{
        gtk_widget_hide(filter_dialog);
}

/* Report druid callbacks */
void
on_create_report_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
        GtkCList *select_col_list=NULL, *avail_col_list=NULL;
        GtkWidget *tmpwidget;
        gchar *tmp_string = NULL, **tmp_array = NULL;
        gint i = 0;
        GList *columns;
        selected_row = -1;
        available_row = -1;

        key_pressed = FALSE;

        if (!reportdruid) {
                reportdruid = create_reportdruid_window();

                tmpwidget = GTK_WIDGET(lookup_widget (reportdruid, "reportdruid_filters_category_combo"));
                if (categories_combo_entries) gtk_combo_set_popdown_strings (GTK_COMBO(tmpwidget), categories_combo_entries); 
        
                tmpwidget = GTK_WIDGET(lookup_widget (reportdruid, "reportdruid_filters_driver_combo"));
                if (drivers_combo_entries) gtk_combo_set_popdown_strings (GTK_COMBO(tmpwidget), drivers_combo_entries); 
                
                tmpwidget = GTK_WIDGET(lookup_widget (reportdruid, "reportdruid_filters_manufacturer_combo"));
                if (manufacturers_combo_entries) gtk_combo_set_popdown_strings (GTK_COMBO(tmpwidget), manufacturers_combo_entries); 
        
                tmpwidget = GTK_WIDGET(lookup_widget (reportdruid, "reportdruid_filters_controls_combo"));
                if (controls_combo_entries) gtk_combo_set_popdown_strings (GTK_COMBO(tmpwidget), controls_combo_entries); 

                /* Column selector setup */
                avail_col_list = GTK_CLIST (lookup_widget (GTK_WIDGET(reportdruid), "reportdruid_columns_available_clist"));
                select_col_list = GTK_CLIST (lookup_widget (GTK_WIDGET(reportdruid), "reportdruid_columns_selected_clist"));

                tmp_string = g_malloc(30);
                g_snprintf(tmp_string, 30, "1,2,5,6");
                tmp_array = g_strsplit(tmp_string, ",", COLUMN_COUNT);

                columns=NULL;
                while (tmp_array[i] != NULL) {
                        columns = g_list_append (columns,  GINT_TO_POINTER(atoi(tmp_array[i])));
                        i++;
                }
                g_free(tmp_string);
                g_strfreev(tmp_array);
        
                populate_column_lists(columns, select_col_list, avail_col_list);
                g_list_free ( columns);
                
                /* Filters setup */
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_romstatus_correct_checkbutton");
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), general_prefs.ListFilter.correct);

                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_romstatus_incorrect_checkbutton");
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), general_prefs.ListFilter.incorrect);
                
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_romstatus_notfound_checkbutton");
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), general_prefs.ListFilter.missing);
                
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_romstatus_maywork_checkbutton");
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), general_prefs.ListFilter.may_work);
                
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_romstatus_favourites_checkbutton");
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), general_prefs.ListFilter.favourites);
                
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_romstatus_clones_checkbutton");
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), general_prefs.ListFilter.clones);

                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_driverstatus_checkbutton");
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), general_prefs.ListFilter.driver_status_active);
                if (general_prefs.ListFilter.driver_status_good) {
                        tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_driverstatus_good_radiobutton");
                } else {
                        tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_driverstatus_preliminary_radiobutton");
                }
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), TRUE);
        
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_videotype_checkbutton");
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), general_prefs.ListFilter.video_type_active);
                if (general_prefs.ListFilter.video_type_vector) {
                        tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_videotype_vector_radiobutton");
                } else {
                        tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_videotype_raster_radiobutton");
                }
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), TRUE);
        
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_year_checkbutton");
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), general_prefs.ListFilter.year_active);
                switch (general_prefs.ListFilter.year_operator) {
                case 0:
                        tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_year_lt_radiobutton");
                        break;
                case 1:
                        tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_year_gt_radiobutton");
                        break;
                case 2:
                        tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_year_ne_radiobutton");
                        break;
                case 3:
                        tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_year_eq_radiobutton");
                        break;
                }
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), TRUE);
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_year_spinbutton");
                gtk_spin_button_set_value(GTK_SPIN_BUTTON(tmpwidget), general_prefs.ListFilter.year_value);
        
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_players_checkbutton");
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), general_prefs.ListFilter.players_active);
                switch (general_prefs.ListFilter.players_operator) {
                case 0:
                        tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_players_lt_radiobutton");
                        break;
                case 1:
                        tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_players_gt_radiobutton");
                        break;
                case 2:
                        tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_players_ne_radiobutton");
                        break;
                case 3:
                        tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_players_eq_radiobutton");
                        break;
                }
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), TRUE);
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_players_spinbutton");
                gtk_spin_button_set_value(GTK_SPIN_BUTTON(tmpwidget), general_prefs.ListFilter.players_value);
        
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_controls_checkbutton");
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), general_prefs.ListFilter.controls_active);
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_controls_not_checkbutton");
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), general_prefs.ListFilter.not_controls_active);
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_controls_entry");
                gtk_entry_set_text(GTK_ENTRY(tmpwidget), general_prefs.ListFilter.controls_value);
        
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_manufacturer_checkbutton");
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), general_prefs.ListFilter.manufacturer_active);
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_manufacturer_not_checkbutton");
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), general_prefs.ListFilter.not_manufacturer_active);
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_manufacturer_entry");
                gtk_entry_set_text(GTK_ENTRY(tmpwidget), general_prefs.ListFilter.manufacturer_value);
        
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_driver_checkbutton");
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), general_prefs.ListFilter.driver_active);
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_driver_not_checkbutton");
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), general_prefs.ListFilter.not_driver_active);
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_driver_entry");
                gtk_entry_set_text(GTK_ENTRY(tmpwidget), general_prefs.ListFilter.driver_value);
        
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_category_checkbutton");
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), general_prefs.ListFilter.category_active);
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_category_not_checkbutton");
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), general_prefs.ListFilter.not_category_active);
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_category_entry");
                gtk_entry_set_text(GTK_ENTRY(tmpwidget), general_prefs.ListFilter.category_value);
        
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_screenshot_checkbutton");
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), general_prefs.ListFilter.screenshot_active);
                if (general_prefs.ListFilter.not_screenshot_active) {
                        tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_screenshot_not_installed_radiobutton");
                } else {
                        tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_screenshot_installed_radiobutton");
                }
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), TRUE);
        
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_flyer_checkbutton");
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), general_prefs.ListFilter.flyer_active);
                if (general_prefs.ListFilter.not_flyer_active) {
                        tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_flyer_not_installed_radiobutton");
                } else {
                        tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_flyer_installed_radiobutton");
                }
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), TRUE);
        
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_cabinet_checkbutton");
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), general_prefs.ListFilter.cabinet_active);
                if (general_prefs.ListFilter.not_cabinet_active) {
                        tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_cabinet_not_installed_radiobutton");
                } else {
                        tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_cabinet_installed_radiobutton");
                }
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), TRUE);
                                
        }
        
        gnome_druid_set_page  (gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid"),
                        gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_pagestart"));

        gtk_widget_show(reportdruid);
}

void
on_report_druid_cancel                 (GnomeDruid      *gnomedruid,
                                        gpointer         user_data)
{
        gtk_widget_hide(reportdruid);
}


gboolean
on_report_druidpagestart_next          (GnomeDruidPage  *gnomedruidpage,
                                        gpointer         arg1,
                                        gpointer         user_data)
{

        return FALSE;
}


gboolean
on_report_druidpage_filters_next       (GnomeDruidPage  *gnomedruidpage,
                                        gpointer         arg1,
                                        gpointer         user_data)
{

        return FALSE;
}


gboolean
on_report_druidpage_filters_back       (GnomeDruidPage  *gnomedruidpage,
                                        gpointer         arg1,
                                        gpointer         user_data)
{
        return FALSE;
}


gboolean
on_report_druidpage_columns_back       (GnomeDruidPage  *gnomedruidpage,
                                        gpointer         arg1,
                                        gpointer         user_data)
{
        return FALSE;
}


gboolean
on_report_druidpage_columns_next       (GnomeDruidPage  *gnomedruidpage,
                                        gpointer         arg1,
                                        gpointer         user_data)
{
        return FALSE;
}


gboolean
on_report_druidpage_format_back        (GnomeDruidPage  *gnomedruidpage,
                                        gpointer         arg1,
                                        gpointer         user_data)
{
        return FALSE;
}


gboolean
on_report_druidpage_format_next        (GnomeDruidPage  *gnomedruidpage,
                                        gpointer         arg1,
                                        gpointer         user_data)
{
        return FALSE;
}


gboolean
on_report_druidpagefinish_back         (GnomeDruidPage  *gnomedruidpage,
                                        gpointer         arg1,
                                        gpointer         user_data)
{
        return FALSE;
}


void
on_report_druidpagefinish_finish       (GnomeDruidPage  *gnomedruidpage,
                                        gpointer         arg1,
                                        gpointer         user_data)
{
        GList *columns=NULL;
        GameFilter *filter;
        GtkCList *select_col_list=NULL;
        GtkWidget *tmpwidget;
        gint n=0, number=0;
        gpointer *data;
        gchar *filename=NULL;
        gchar *tmpprefix=NULL;
        gchar *command=NULL;
                
        gtk_widget_hide(reportdruid);
                
        select_col_list = GTK_CLIST (lookup_widget (GTK_WIDGET(reportdruid), "reportdruid_columns_selected_clist"));
        while (gtk_clist_get_text(select_col_list, n, 0, NULL)) {
                data = gtk_clist_get_row_data(select_col_list, n);
                number = GPOINTER_TO_INT(data);
                columns = g_list_prepend(columns, GINT_TO_POINTER(number));
                n++;
        }
        columns = g_list_reverse(columns);

        if (g_list_length(columns) <= 0) {
                grustibus_warning (_("I think you should specify at least one column :)"));
                g_list_free(columns); 
        }
        else {
        
                filter = g_malloc(sizeof(GameFilter));
        
                filter->advanced_filters_active=TRUE;
        
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_romstatus_correct_checkbutton");
                filter->correct = GTK_TOGGLE_BUTTON(tmpwidget)->active;
        
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_romstatus_incorrect_checkbutton");
                filter->incorrect = GTK_TOGGLE_BUTTON(tmpwidget)->active;
                    
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_romstatus_notfound_checkbutton");
                filter->missing = GTK_TOGGLE_BUTTON(tmpwidget)->active;
                        
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_romstatus_maywork_checkbutton");
                filter->may_work = GTK_TOGGLE_BUTTON(tmpwidget)->active;
                        
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_romstatus_favourites_checkbutton");
                filter->favourites = GTK_TOGGLE_BUTTON(tmpwidget)->active;
                        
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_romstatus_clones_checkbutton");
                filter->clones = GTK_TOGGLE_BUTTON(tmpwidget)->active;
                
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_driverstatus_checkbutton");
                filter->driver_status_active = GTK_TOGGLE_BUTTON(tmpwidget)->active;
        
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_driverstatus_good_radiobutton");
                filter->driver_status_good = GTK_TOGGLE_BUTTON(tmpwidget)->active;
        
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_videotype_checkbutton");
                filter->video_type_active = GTK_TOGGLE_BUTTON(tmpwidget)->active;
        
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_videotype_vector_radiobutton");
                filter->video_type_vector = GTK_TOGGLE_BUTTON(tmpwidget)->active;
        
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_year_checkbutton");
                filter->year_active = GTK_TOGGLE_BUTTON(tmpwidget)->active;
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_year_lt_radiobutton");
                if (GTK_TOGGLE_BUTTON(tmpwidget)->active) filter->year_operator = 0;
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_year_gt_radiobutton");
                if (GTK_TOGGLE_BUTTON(tmpwidget)->active) filter->year_operator = 1;
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_year_ne_radiobutton");
                if (GTK_TOGGLE_BUTTON(tmpwidget)->active) filter->year_operator = 2;
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_year_eq_radiobutton");
                if (GTK_TOGGLE_BUTTON(tmpwidget)->active) filter->year_operator = 3;
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_year_spinbutton");
                filter->year_value = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(tmpwidget));
        
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_players_checkbutton");
                filter->players_active = GTK_TOGGLE_BUTTON(tmpwidget)->active;
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_players_lt_radiobutton");
                if (GTK_TOGGLE_BUTTON(tmpwidget)->active) filter->players_operator = 0;
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_players_gt_radiobutton");
                if (GTK_TOGGLE_BUTTON(tmpwidget)->active) filter->players_operator = 1;
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_players_ne_radiobutton");
                if (GTK_TOGGLE_BUTTON(tmpwidget)->active) filter->players_operator = 2;
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_players_eq_radiobutton");
                if (GTK_TOGGLE_BUTTON(tmpwidget)->active) filter->players_operator = 3;
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_players_spinbutton");
                filter->players_value = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(tmpwidget));
        
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_controls_checkbutton");
                filter->controls_active = GTK_TOGGLE_BUTTON(tmpwidget)->active;
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_controls_not_checkbutton");
                filter->not_controls_active = GTK_TOGGLE_BUTTON(tmpwidget)->active;
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_controls_entry");
                filter->controls_value = gtk_editable_get_chars(GTK_EDITABLE(tmpwidget), 0, -1);
        
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_manufacturer_checkbutton");
                filter->manufacturer_active = GTK_TOGGLE_BUTTON(tmpwidget)->active;
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_manufacturer_not_checkbutton");
                filter->not_manufacturer_active = GTK_TOGGLE_BUTTON(tmpwidget)->active;
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_manufacturer_entry");
                filter->manufacturer_value = gtk_editable_get_chars(GTK_EDITABLE(tmpwidget), 0, -1);
        
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_driver_checkbutton");
                filter->driver_active = GTK_TOGGLE_BUTTON(tmpwidget)->active;
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_driver_not_checkbutton");
                filter->not_driver_active = GTK_TOGGLE_BUTTON(tmpwidget)->active;
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_driver_entry");
                filter->driver_value = gtk_editable_get_chars(GTK_EDITABLE(tmpwidget), 0, -1);
        
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_category_checkbutton");
                filter->category_active = GTK_TOGGLE_BUTTON(tmpwidget)->active;
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_category_not_checkbutton");
                filter->not_category_active = GTK_TOGGLE_BUTTON(tmpwidget)->active;
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_category_entry");
                filter->category_value = gtk_editable_get_chars(GTK_EDITABLE(tmpwidget), 0, -1);
        
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_screenshot_checkbutton");
                filter->screenshot_active = GTK_TOGGLE_BUTTON(tmpwidget)->active;
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_screenshot_not_installed_radiobutton");
                filter->not_screenshot_active = GTK_TOGGLE_BUTTON(tmpwidget)->active;
        
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_flyer_checkbutton");
                filter->flyer_active = GTK_TOGGLE_BUTTON(tmpwidget)->active;
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_flyer_not_installed_radiobutton");
                filter->not_flyer_active = GTK_TOGGLE_BUTTON(tmpwidget)->active;
        
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_cabinet_checkbutton");
                filter->cabinet_active = GTK_TOGGLE_BUTTON(tmpwidget)->active;
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_filters_cabinet_not_installed_radiobutton");
                filter->not_cabinet_active = GTK_TOGGLE_BUTTON(tmpwidget)->active;
        
                
                tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_format_medium_file_radiobutton");
                if (GTK_TOGGLE_BUTTON(tmpwidget)->active)
                {
                        filename=g_malloc(100);
                        tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_format_fileformat_xhtml_radiobutton");
                        if (GTK_TOGGLE_BUTTON(tmpwidget)->active) {
                                g_snprintf(filename, 100, "%s/grustibus_report.html", getenv("HOME"));
                                if (!create_html_report (filter, columns, filename)) grustibus_warning (_("Error saving file")) ;
                        }
                       
                        tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_format_fileformat_mtext_radiobutton");
                        if (GTK_TOGGLE_BUTTON(tmpwidget)->active) {
                                g_snprintf(filename, 100, "%s/grustibus_report.txt", getenv("HOME"));
                                if (!create_text_report (filter, columns, filename)) grustibus_warning (_("Error saving file")) ;
                        }
                       
                        tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_format_fileformat_htext_radiobutton");
                        if (GTK_TOGGLE_BUTTON(tmpwidget)->active) {
                                g_snprintf(filename, 100, "%s/grustibus_report.txt", getenv("HOME"));
                                if (!create_pretty_text_report (filter, columns, filename)) grustibus_warning (_("Error saving file")) ;
                        }
                        g_free(filename);
                }        
        
                else {
                        filename=g_malloc(100);
                        tmpprefix=g_malloc(50);
                        g_snprintf(tmpprefix,50, "grustibus_report-XXXXXX");
                        mkstemp(tmpprefix);
                        tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_format_fileformat_xhtml_radiobutton");
                        if (GTK_TOGGLE_BUTTON(tmpwidget)->active) {
                                g_snprintf(filename, 100, "/tmp/%s.html", tmpprefix);
                                if (!create_html_report (filter, columns, filename)) grustibus_warning (_("Error saving file")) ;
                                else {
                                        command=g_malloc(150);
                                        g_snprintf(command, 150, "file://%s", filename);
                                        gnome_url_show(command);
                                        g_free (command);
                                }
                        }
                        
                        tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_format_fileformat_mtext_radiobutton");
                        if (GTK_TOGGLE_BUTTON(tmpwidget)->active) {
                                g_snprintf(filename, 100, "/tmp/%s.txt", tmpprefix);
                                command=g_malloc(150);
                                g_snprintf(command, 150, "gnome-edit %s", filename);
                                if (!create_text_report (filter, columns, filename)) grustibus_warning (_("Error saving file")) ;
                                else if (!gnome_execute_shell (getenv("HOME"), command)) grustibus_warning (_("Could not open gnome-edit"));
                                g_free (command);
        
        
                        }
                        
                        tmpwidget = gtk_object_get_data(GTK_OBJECT(reportdruid), "reportdruid_format_fileformat_htext_radiobutton");
                        if (GTK_TOGGLE_BUTTON(tmpwidget)->active) {
                                g_snprintf(filename, 100, "/tmp/%s.txt", tmpprefix);
                                command=g_malloc(150);
                                g_snprintf(command, 150, "gnome-edit %s", filename);
                                if (!create_pretty_text_report (filter, columns, filename)) grustibus_warning (_("Error saving file")) ;
                                else if (!gnome_execute_shell (getenv("HOME"), command)) grustibus_warning (_("Could not open gnome-edit"));
                                g_free (command);
                        }
                        g_free(filename);
                        g_free(tmpprefix);
                }
        
                if (filter->controls_value) g_free(filter->controls_value);
                if (filter->driver_value) g_free(filter->driver_value);
                if (filter->manufacturer_value) g_free(filter->manufacturer_value);
                if (filter->category_value) g_free(filter->category_value);
                g_list_free(columns); 
                g_free(filter);
        }
}


void
on_reportdruid_filters_cabinet_checkbutton_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
        gtk_widget_set_sensitive(lookup_widget
                                 (GTK_WIDGET(reportdruid),
                                  "reportdruid_filters_cabinet_hbox"),
                                 togglebutton->active);

}


void
on_reportdruid_filters_flyer_checkbutton_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
        gtk_widget_set_sensitive(lookup_widget
                                 (GTK_WIDGET(reportdruid),
                                  "reportdruid_filters_flyer_hbox"),
                                 togglebutton->active);

}


void
on_reportdruid_filters_screenshot_checkbutton_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
        gtk_widget_set_sensitive(lookup_widget
                                 (GTK_WIDGET(reportdruid),
                                  "reportdruid_filters_screenshot_hbox"),
                                 togglebutton->active);

}


void
on_reportdruid_filters_category_checkbutton_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
        gtk_widget_set_sensitive(lookup_widget
                                 (GTK_WIDGET(reportdruid),
                                  "reportdruid_filters_category_hbox"),
                                 togglebutton->active);

}


void
on_reportdruid_filters_driver_checkbutton_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
        gtk_widget_set_sensitive(lookup_widget
                                 (GTK_WIDGET(reportdruid),
                                  "reportdruid_filters_driver_hbox"),
                                 togglebutton->active);

}


void
on_reportdruid_filters_manufacturer_checkbutton_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
        gtk_widget_set_sensitive(lookup_widget
                                 (GTK_WIDGET(reportdruid),
                                  "reportdruid_filters_manufacturer_hbox"),
                                 togglebutton->active);

}


void
on_reportdruid_filters_controls_checkbutton_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
        gtk_widget_set_sensitive(lookup_widget
                                 (GTK_WIDGET(reportdruid),
                                  "reportdruid_filters_controls_hbox"),
                                 togglebutton->active);

}


void
on_reportdruid_filters_players_checkbutton_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
        gtk_widget_set_sensitive(lookup_widget
                                 (GTK_WIDGET(reportdruid),
                                  "reportdruid_filters_players_hbox"),
                                 togglebutton->active);

}


void
on_reportdruid_filters_year_checkbutton_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
        gtk_widget_set_sensitive(lookup_widget
                                 (GTK_WIDGET(reportdruid),
                                  "reportdruid_filters_year_hbox"),
                                 togglebutton->active);

}


void
on_reportdruid_filters_videotype_checkbutton_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
        gtk_widget_set_sensitive(lookup_widget
                                 (GTK_WIDGET(reportdruid),
                                  "reportdruid_filters_videotype_hbox"),
                                 togglebutton->active);

}


void
on_reportdruid_filters_driverstatus_checkbutton_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
        gtk_widget_set_sensitive(lookup_widget
                                 (GTK_WIDGET(reportdruid),
                                  "reportdruid_filters_driverstatus_hbox"),
                                 togglebutton->active);

}

void
on_reportdruid_columns_available_clist_select_row
                                        (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
        available_row = row;
}


void
on_reportdruid_columns_selected_clist_select_row
                                        (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
        selected_row = row;
}


void
on_reportdruid_columns_controls_down_button_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
        GtkCList *select_col_list=NULL;
        select_col_list = GTK_CLIST (lookup_widget (GTK_WIDGET(reportdruid), "reportdruid_columns_selected_clist"));

        if (gtk_clist_get_selectable (select_col_list, selected_row) &&
            gtk_clist_get_selectable(select_col_list, selected_row + 1)) {
                gtk_clist_swap_rows(select_col_list, selected_row,
                                    selected_row + 1);
                selected_row++;
        }
}


void
on_reportdruid_columns_controls_up_button_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
        GtkCList *select_col_list=NULL;
        select_col_list = GTK_CLIST(lookup_widget (GTK_WIDGET(reportdruid), "reportdruid_columns_selected_clist"));

        if (gtk_clist_get_selectable(select_col_list, selected_row - 1)) {
                gtk_clist_swap_rows(select_col_list, selected_row,
                                    selected_row - 1);
                selected_row--;
        }

}


void
on_reportdruid_columns_control_right_button_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
        gchar *get;
        gchar **insert;
        gpointer *data;
        gint newrow = 0, number;
        GtkCList *select_col_list=NULL, *avail_col_list=NULL;
        
        avail_col_list = GTK_CLIST(lookup_widget (GTK_WIDGET(reportdruid), "reportdruid_columns_available_clist"));
        select_col_list = GTK_CLIST(lookup_widget (GTK_WIDGET(reportdruid), "reportdruid_columns_selected_clist"));

        insert = g_new(gchar *, 1);
        if (gtk_clist_get_text
            (avail_col_list, available_row, 0, &get)) {
                data = gtk_clist_get_row_data(avail_col_list, available_row);
                number = GPOINTER_TO_INT(data);
                insert[0] = get;
                newrow = gtk_clist_append(select_col_list, insert);
                gtk_clist_set_row_data(select_col_list, newrow, GINT_TO_POINTER(number));
                gtk_clist_remove(avail_col_list, available_row);
                if (gtk_clist_get_selectable
                    (avail_col_list, available_row)) {
                        gtk_clist_select_row(avail_col_list, available_row, 0);
                        gtk_clist_moveto(avail_col_list, available_row, 0, 0.5, 1.0);
                } else {
                        gtk_clist_select_row(avail_col_list, available_row - 1, 0);
                        gtk_clist_moveto(avail_col_list, available_row - 1, 0, 0.5, 1.0);
                }
        }
}


void
on_reportdruid_columns_controls_left_button_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
        gchar *get;
        gchar **insert;
        gpointer *data;
        gint newrow = 0, number;
        GtkCList *select_col_list=NULL, *avail_col_list=NULL;
        
        avail_col_list = GTK_CLIST(lookup_widget (GTK_WIDGET(reportdruid), "reportdruid_columns_available_clist"));
        select_col_list = GTK_CLIST(lookup_widget (GTK_WIDGET(reportdruid), "reportdruid_columns_selected_clist"));

        insert = g_new(gchar *, 1);

        if (gtk_clist_get_text (select_col_list, selected_row, 0, &get) && 
                        ((gtk_clist_get_text (select_col_list, selected_row-1,0, NULL)) ||
                         (gtk_clist_get_text (select_col_list, selected_row+1,0, NULL)))) {
                data = gtk_clist_get_row_data(select_col_list, selected_row);
                number = GPOINTER_TO_INT(data);
                insert[0] = get;
                newrow = gtk_clist_append(avail_col_list, insert);
                gtk_clist_set_row_data(avail_col_list, newrow, GINT_TO_POINTER(number));
                gtk_clist_remove(select_col_list, selected_row);
                if (gtk_clist_get_selectable
                    (select_col_list, selected_row)) {
                        gtk_clist_select_row(select_col_list, selected_row, 0);
                        gtk_clist_moveto(select_col_list, selected_row, 0, 0.5, 1.0);
                } else if (gtk_clist_get_selectable
                        (select_col_list, selected_row - 1)) {
                        gtk_clist_select_row(select_col_list, selected_row - 1, 0);
                        gtk_clist_moveto(select_col_list, selected_row - 1, 0, 0.5, 1.0);
                } else {
                        selected_row = -1;
                }
        }

}


void
on_reportdruid_columns_controls_default_button_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
        gchar *tmp_string = NULL, **tmp_array = NULL;
        gint i = 0;
        GList *columns;
        GtkCList *select_col_list=NULL, *avail_col_list=NULL;
                
        avail_col_list = GTK_CLIST (lookup_widget (GTK_WIDGET(reportdruid), "reportdruid_columns_available_clist"));
        select_col_list = GTK_CLIST (lookup_widget (GTK_WIDGET(reportdruid), "reportdruid_columns_selected_clist"));

        tmp_string = g_malloc(30);
        g_snprintf(tmp_string, 30, "1,2,5,6");
        tmp_array = g_strsplit(tmp_string, ",", COLUMN_COUNT);

        columns=NULL;
        while (tmp_array[i] != NULL) {
                columns = g_list_append (columns,  GINT_TO_POINTER(atoi(tmp_array[i])));
                i++;
        }
        g_free(tmp_string);
        g_strfreev(tmp_array);

        populate_column_lists(columns, select_col_list, avail_col_list);
        g_list_free (columns);
}

// vim: ai sw=8 ts=8
