/*
 * fileio.c -- gRustibus file input/output functions
 *
 * gRustibus (c) 2000-2001 Kjetil Thuen <kjetilt@users.sourceforge.net>
 * MAME frontend for the GNOME desktop environment.
 *
 * gRustibus is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * This code is indented with GNU indent using k&r style with 8 character
 * indents as suggested by the GNOME project. Please keep it that way.
 * ($ indent -kr -ts8 -i8 *.[ch])
 *
 * Last modified: 28 May 2001 <kjetilt@users.sourceforge.net>
 *
 */


#include "grustibus.h"
#include "fileio.h"
#include "gui.h"

void make_dir()
{
        gchar *dir = g_malloc(60);
        gchar *cmd = g_malloc(70);

        g_snprintf(dir, 60, "%s/.gnome/gRustibus.d", getenv("HOME"));
        if (!fopen(dir, "r")) {
                g_snprintf(cmd, 70, "mkdir %s", dir);
                system(cmd);
        }
        g_free(dir);
        g_free(cmd);
}

void load_catver()
{
        RomEntry *rom;
        gchar *configstring = NULL;
        gchar *tmpstring = NULL;
        GList *listpoint = NULL;

        grustibus_debug("Loading categories");

        configstring = g_malloc0(50);
        listpoint = g_list_first(roms);

        gnome_config_push_prefix("gRustibus.d/catver/category/");
        while ((listpoint != NULL)) {
                tmpstring = '\0';
                rom = (RomEntry *) listpoint->data;
                g_snprintf(configstring, 50, _("%s=[Unknown]"),
                           rom->romname);
                tmpstring = gnome_config_get_string(configstring);
                g_snprintf(rom->category, MAX_CATEGORY, "%s", tmpstring);
                g_free(tmpstring);
                listpoint = g_list_next(listpoint);
        }
        gnome_config_pop_prefix();
        g_free(configstring);

        grustibus_debug("Loading mameversion");
        
        configstring = g_malloc(50);
        listpoint = g_list_first(roms);

        gnome_config_push_prefix("gRustibus.d/catver/VerAdded/");
        while ((listpoint != NULL)) {
                tmpstring = '\0';
                rom = (RomEntry *) listpoint->data;
                g_snprintf(configstring, 50, _("%s=[Unknown]"),
                           rom->romname);
                tmpstring = gnome_config_get_string(configstring);
                g_snprintf(rom->mame_ver_added, MAX_MAMEVER, "%s",
                           tmpstring);
                g_free(tmpstring);
                listpoint = g_list_next(listpoint);
        }
        gnome_config_pop_prefix();
        g_free(configstring);
}

void load_timesplayed()
{
        RomEntry *rom;
        gchar *configstring = NULL;
        GList *listpoint = NULL;

        grustibus_debug("Loading times played info.");

        configstring = g_malloc0(50);
        listpoint = g_list_first(roms);

        gnome_config_push_prefix("gRustibus.d/timesplayed/played/");
        while ((listpoint != NULL)) {
                rom = (RomEntry *) listpoint->data;
                g_snprintf(configstring, 50, ("%s=0"), rom->romname);
                rom->timesplayed = gnome_config_get_int(configstring);
                listpoint = g_list_next(listpoint);
        }
        gnome_config_pop_prefix();
        g_free(configstring);
}

void save_timesplayed(){
        GList *listpoint = NULL;
        RomEntry *rom;

        grustibus_debug("Saving times played info");

        listpoint = g_list_first(roms);
        gnome_config_push_prefix("gRustibus.d/timesplayed/played/");
        while ((listpoint != NULL)) {
                rom = (RomEntry *) listpoint->data;
                if (rom->timesplayed >0 ) { 
                        gnome_config_set_int(rom->romname, rom->timesplayed);
                }
                listpoint = g_list_next(listpoint);
        }
        gnome_config_pop_prefix();
        gnome_config_sync();
}

void load_prefs()
{
        gchar *tmp_string = NULL, **tmp_array = NULL;
        gint i = 0;

        /* Set general preferences */
        gnome_config_push_prefix("gRustibus/Settings/");

        /* The default paths are the default paths of xmame */
        general_prefs.xmame_exe =
            gnome_config_get_string("xmame_exe=/usr/games/xmame");
        general_prefs.rom_dir =
            gnome_config_get_string("rom_dir=/usr/games/lib/xmame/roms/");
        general_prefs.screenshot_dir =
            gnome_config_get_string
            ("screenshot_dir=/usr/games/lib/xmame/snaps/");
        general_prefs.highscore_dir =
            gnome_config_get_string
            ("highscore_dir=/usr/games/lib/xmame/hi/");
        general_prefs.flyer_dir =
            gnome_config_get_string
            ("flyer_dir=/usr/games/lib/xmame/flyers/");
        general_prefs.cabinet_dir =
            gnome_config_get_string
            ("cabinet_dir=/usr/games/lib/xmame/cabinets/");
        general_prefs.game_history_file =
            gnome_config_get_string
            ("game_history_file=/usr/games/lib/xmame/history.dat");
        general_prefs.cheat_file =
            gnome_config_get_string
            ("cheat_file=/usr/games/lib/xmame/cheat.dat");
        general_prefs.playback_dir =
            gnome_config_get_string
            ("playback_dir=/usr/games/lib/xmame/inp/");
        general_prefs.xmame_major =
            gnome_config_get_int("xmame_major_version=0");
        general_prefs.xmame_minor =
            gnome_config_get_string("xmame_minor_version=00");
        general_prefs.xmame_release =
            gnome_config_get_int("xmame_release_version=1");
        general_prefs.romdir_time = gnome_config_get_int("romdir_time=0");

        general_prefs.font = gnome_config_get_string("font=fixed");
        general_prefs.correct_color_r =
            gnome_config_get_int("correct_color_r=90");
        general_prefs.correct_color_g =
            gnome_config_get_int("correct_color_g=110");
        general_prefs.correct_color_b =
            gnome_config_get_int("correct_color_b=190");
        general_prefs.incorrect_color_r =
            gnome_config_get_int("incorrect_color_r=170");
        general_prefs.incorrect_color_g =
            gnome_config_get_int("incorrect_color_g=170");
        general_prefs.incorrect_color_b =
            gnome_config_get_int("incorrect_color_b=90");
        general_prefs.missing_color_r =
            gnome_config_get_int("missing_color_r=190");
        general_prefs.missing_color_g =
            gnome_config_get_int("missing_color_g=140");
        general_prefs.missing_color_b =
            gnome_config_get_int("missing_color_b=150");
        general_prefs.favourite_color_r =
            gnome_config_get_int("favourite_color_r=0");
        general_prefs.favourite_color_g =
            gnome_config_get_int("favourite_color_g=0");
        general_prefs.favourite_color_b =
            gnome_config_get_int("favourite_color_b=255");
        general_prefs.may_work_color_r =
            gnome_config_get_int("may_work_color_r=180");
        general_prefs.may_work_color_g =
            gnome_config_get_int("may_work_color_g=120");
        general_prefs.may_work_color_b =
            gnome_config_get_int("may_work_color_b=190");

        gnome_config_pop_prefix();
        gnome_config_push_prefix("gRustibus/Window/");

        general_prefs.sort_by = gnome_config_get_int("sort_by=1");
        general_prefs.sort_order = gnome_config_get_int("sort_order=0");
        general_prefs.current_row = gnome_config_get_int("current_row=0");
        general_prefs.current_tab = gnome_config_get_int("current_tab=0");
        general_prefs.pane_position =
            gnome_config_get_int("pane_position=400");
        general_prefs.notebook_page =
            gnome_config_get_int("notebook_page=0");
        general_prefs.x_size = gnome_config_get_int("x_size=800");
        general_prefs.y_size = gnome_config_get_int("y_size=550");
        general_prefs.save_win_on_quit =
            gnome_config_get_bool("save_win_on_quit=TRUE");

        general_prefs.favourites_overrides =
            gnome_config_get_bool("favourites_overrides=TRUE");

        tmp_string = gnome_config_get_string
            ("active_columns=0,1,9,5,6,24,12,4");
        tmp_array = g_strsplit(tmp_string, ",", COLUMN_COUNT);
        while (tmp_array[i] != NULL) {
                general_prefs.active_columns = g_list_append
                    (general_prefs.active_columns,
                     GINT_TO_POINTER(atoi(tmp_array[i])));
                i++;
        }
        g_free(tmp_string);
        g_strfreev(tmp_array);

        general_prefs.column_size_string = gnome_config_get_string
            ("column_sizes=20,120,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80");

        general_prefs.iconify_while_xmame =
            gnome_config_get_bool("iconify_while_xmame=TRUE");
        general_prefs.playback_no_highscore =
            gnome_config_get_bool("playback_no_highscores=TRUE");
        general_prefs.loadsa_image_formats =
            gnome_config_get_bool("loadsa_image_formats=FALSE");
        general_prefs.look_for_clone_images =
            gnome_config_get_bool("look_for_clone_images=TRUE");
        general_prefs.indicate_notebook_changes =
            gnome_config_get_bool("indicate_notebook_changes=TRUE");

        gnome_config_pop_prefix();

        /* Load advanced filter settings */
        gnome_config_push_prefix("gRustibus/Filters/");
        general_prefs.ListFilter.correct =
            gnome_config_get_bool("show_correct=TRUE");
        general_prefs.ListFilter.incorrect =
            gnome_config_get_bool("show_incorrect=TRUE");
        general_prefs.ListFilter.missing =
            gnome_config_get_bool("show_missing=TRUE");
        general_prefs.ListFilter.may_work =
            gnome_config_get_bool("show_may_work=TRUE");
        general_prefs.ListFilter.clones =
            gnome_config_get_bool("show_clones=TRUE");
        general_prefs.ListFilter.favourites =
            gnome_config_get_bool("show_favourites=FALSE");
        general_prefs.ListFilter.advanced_filters_active =
            gnome_config_get_bool("filters_active=FALSE");

        general_prefs.ListFilter.driver_status_active =
            gnome_config_get_bool("driver_status_active=FALSE");
        general_prefs.ListFilter.video_type_active =
            gnome_config_get_bool("video_type_active=FALSE");
        general_prefs.ListFilter.year_active =
            gnome_config_get_bool("year_active=FALSE");
        general_prefs.ListFilter.players_active =
            gnome_config_get_bool("players_active=FALSE");
        general_prefs.ListFilter.controls_active =
            gnome_config_get_bool("controls_active=FALSE");
        general_prefs.ListFilter.manufacturer_active =
            gnome_config_get_bool("manufacturer_active=FALSE");
        general_prefs.ListFilter.driver_active =
            gnome_config_get_bool("driver_active=FALSE");
        general_prefs.ListFilter.category_active =
            gnome_config_get_bool("category_active=FALSE");
        general_prefs.ListFilter.screenshot_active =
            gnome_config_get_bool("screenshot_active=FALSE");
        general_prefs.ListFilter.flyer_active =
            gnome_config_get_bool("flyer_active=FALSE");
        general_prefs.ListFilter.cabinet_active =
            gnome_config_get_bool("cabinet_active=FALSE");

        general_prefs.ListFilter.year_operator =
            gnome_config_get_int("year_operator_type=3");
        general_prefs.ListFilter.players_operator =
            gnome_config_get_int("players_operator_type=3");
        general_prefs.ListFilter.not_controls_active =
            gnome_config_get_bool("controls_not=FALSE");
        general_prefs.ListFilter.not_manufacturer_active =
            gnome_config_get_bool("manufacturer_not=FALSE");
        general_prefs.ListFilter.not_driver_active =
            gnome_config_get_bool("driver_not=FALSE");
        general_prefs.ListFilter.not_category_active =
            gnome_config_get_bool("category_not=FALSE");
        general_prefs.ListFilter.not_screenshot_active =
            gnome_config_get_bool("screenshot_not=TRUE");
        general_prefs.ListFilter.not_flyer_active =
            gnome_config_get_bool("flyer_not=TRUE");
        general_prefs.ListFilter.not_cabinet_active =
            gnome_config_get_bool("cabinet_not=TRUE");

        general_prefs.ListFilter.driver_status_good =
            gnome_config_get_bool("good_driver=TRUE");
        general_prefs.ListFilter.video_type_vector =
            gnome_config_get_bool("vector_video=TRUE");
        general_prefs.ListFilter.year_value =
            gnome_config_get_int("year=1980");
        general_prefs.ListFilter.players_value =
            gnome_config_get_int("players=1");
        general_prefs.ListFilter.controls_value =
            gnome_config_get_string("controls=joy8way");
        general_prefs.ListFilter.manufacturer_value =
            gnome_config_get_string("manufacturer=<unknown>");
        general_prefs.ListFilter.driver_value =
            gnome_config_get_string("driver=neogeo");
        general_prefs.ListFilter.category_value =
            gnome_config_get_string("category=Platform");
        gnome_config_pop_prefix();
        
        gnome_config_push_prefix("gRustibus/Internet/");
        general_prefs.rom_url =
            gnome_config_get_string("rom_url=");
        general_prefs.screenshot_url =
            gnome_config_get_string("screenshot_url=");
        general_prefs.flyer_url =
            gnome_config_get_string("flyer_url=");
        general_prefs.cabinet_url =
            gnome_config_get_string("cabinet_url=");
        gnome_config_pop_prefix();

        /* Load global game preferences */
        global_settings =
            load_game_prefs("/gRustibus.d/gameprefs_global",
                            global_settings);

        return;
}

/* FIXME: Passing a struct is too expensive. Pass a pointer instead */
struct GameSettings
load_game_prefs(const gchar * filename, struct GameSettings settings)
{
        gchar *prefix = g_malloc(50);

        g_snprintf(prefix, 50, "%s/Misc/", filename);
        gnome_config_push_prefix(prefix);
        settings.default_options =
            gnome_config_get_bool("default_options=TRUE");
        gnome_config_pop_prefix();

        g_snprintf(prefix, 50, "%s/Sound/", filename);
        gnome_config_push_prefix(prefix);
        settings.sound = gnome_config_get_bool("sound=TRUE");
        settings.samples = gnome_config_get_bool("samples=TRUE");
        settings.fake_sound = gnome_config_get_bool("fake_sound=FALSE");
        settings.volume = gnome_config_get_int("volume=-10");
        gnome_config_pop_prefix();

        g_snprintf(prefix, 50, "%s/Video/", filename);
        gnome_config_push_prefix(prefix);
        settings.fullscreen = gnome_config_get_bool("fullscreen=FALSE");
        settings.scanlines = gnome_config_get_bool("scanlines=TRUE");
        settings.extra_artwork =
            gnome_config_get_bool("extra_artwork=TRUE");
        settings.autoframeskip =
            gnome_config_get_bool("autoframeskip=TRUE");
        settings.auto_colordepth =
            gnome_config_get_bool("auto_colordepth=TRUE");
        settings.rot_left = gnome_config_get_bool("rotate_left=FALSE");
        settings.rot_right = gnome_config_get_bool("rotate_right=FALSE");
        settings.flipx = gnome_config_get_bool("flip_x=FALSE");
        settings.flipy = gnome_config_get_bool("flip_y=FALSE");
        settings.scale = gnome_config_get_int("scale=0");
        gnome_config_pop_prefix();

        g_snprintf(prefix, 50, "%s/Vector/", filename);
        gnome_config_push_prefix(prefix);
        settings.antialias = gnome_config_get_bool("antialias=FALSE");
        settings.translucency =
            gnome_config_get_bool("translucency=FALSE");
        settings.vectorres = gnome_config_get_int("vectorres=1");
        settings.beam = gnome_config_get_float("beam=1.0");
        settings.flicker = gnome_config_get_float("flicker=1.0");
        gnome_config_pop_prefix();

        g_snprintf(prefix, 50, "%s/Input/", filename);
        gnome_config_push_prefix(prefix);
        settings.mouse = gnome_config_get_bool("mouse=FALSE");
        settings.analog_joy = gnome_config_get_bool("analog_joy=FALSE");
        settings.winkeys = gnome_config_get_bool("windows_keys=FALSE");
        settings.grab_mouse = gnome_config_get_bool("grab_mouse=FALSE");
        settings.joytype = gnome_config_get_int("joytype=0");
        gnome_config_pop_prefix();

        g_snprintf(prefix, 50, "%s/Miscaleneous/", filename);
        gnome_config_push_prefix(prefix);
        settings.cheat = gnome_config_get_bool("cheat=FALSE");
        settings.extra_options = gnome_config_get_string("extra_opts=");
        gnome_config_pop_prefix();

        g_free(prefix);
        return settings;
}

void save_xmame_version() {
        gnome_config_push_prefix("gRustibus/Settings/");
        gnome_config_set_int("xmame_major_version",
                             general_prefs.xmame_major);
        gnome_config_set_string("xmame_minor_version",
                                general_prefs.xmame_minor);
        gnome_config_set_int("xmame_release_version",
                             general_prefs.xmame_release);
        gnome_config_pop_prefix();
        gnome_config_sync();
}

void save_prefs()
{
        gnome_config_push_prefix("gRustibus/Settings/");
        gnome_config_set_string("xmame_exe", general_prefs.xmame_exe);
        gnome_config_set_string("rom_dir", general_prefs.rom_dir);
        gnome_config_set_string("screenshot_dir",
                                general_prefs.screenshot_dir);
        gnome_config_set_string("highscore_dir",
                                general_prefs.highscore_dir);
        gnome_config_set_string("flyer_dir", general_prefs.flyer_dir);
        gnome_config_set_string("cabinet_dir", general_prefs.cabinet_dir);
        gnome_config_set_string("game_history_file",
                                general_prefs.game_history_file);
        gnome_config_set_string("cheat_file", general_prefs.cheat_file);
        gnome_config_set_string("playback_dir",
                                general_prefs.playback_dir);

        gnome_config_set_int("xmame_major_version",
                             general_prefs.xmame_major);
        gnome_config_set_string("xmame_minor_version",
                                general_prefs.xmame_minor);
        gnome_config_set_int("xmame_release_version",
                             general_prefs.xmame_release);
        gnome_config_set_int("romdir_time", general_prefs.romdir_time);

        gnome_config_set_string("font", general_prefs.font);
        gnome_config_set_int("correct_color_r",
                             general_prefs.correct_color_r);
        gnome_config_set_int("correct_color_g",
                             general_prefs.correct_color_g);
        gnome_config_set_int("correct_color_b",
                             general_prefs.correct_color_b);
        gnome_config_set_int("incorrect_color_r",
                             general_prefs.incorrect_color_r);
        gnome_config_set_int("incorrect_color_g",
                             general_prefs.incorrect_color_g);
        gnome_config_set_int("incorrect_color_b",
                             general_prefs.incorrect_color_b);
        gnome_config_set_int("missing_color_r",
                             general_prefs.missing_color_r);
        gnome_config_set_int("missing_color_g",
                             general_prefs.missing_color_g);
        gnome_config_set_int("missing_color_b",
                             general_prefs.missing_color_b);
        gnome_config_set_int("favourite_color_r",
                             general_prefs.favourite_color_r);
        gnome_config_set_int("favourite_color_g",
                             general_prefs.favourite_color_g);
        gnome_config_set_int("favourite_color_b",
                             general_prefs.favourite_color_b);
        gnome_config_set_int("may_work_color_r",
                             general_prefs.may_work_color_r);
        gnome_config_set_int("may_work_color_g",
                             general_prefs.may_work_color_g);
        gnome_config_set_int("may_work_color_b",
                             general_prefs.may_work_color_b);
        gnome_config_pop_prefix();
        
        gnome_config_push_prefix("gRustibus/Internet/");
        gnome_config_set_string("rom_url", general_prefs.rom_url);
        gnome_config_set_string("screenshot_url", general_prefs.screenshot_url);
        gnome_config_set_string("flyer_url", general_prefs.flyer_url);
        gnome_config_set_string("cabinet_url", general_prefs.cabinet_url);
        gnome_config_pop_prefix();

        gnome_config_sync();

        /* Save global game preferences */
        save_game_prefs("/gRustibus.d/gameprefs_global", global_settings);

        need_to_save_prefs = FALSE;
        return;
}

void save_window_prefs()
{
        gchar *tmp_string = NULL, *tmp = NULL;
        GList *tmp_list;
        CListColumnLabel *tmp_data;
        gint tmp_string_length = 0;

        gnome_config_push_prefix("gRustibus/Window/");
        gnome_config_set_int("current_row", general_prefs.current_row);
        gnome_config_set_int("current_tab", general_prefs.current_tab);
        gnome_config_set_int("sort_by", general_prefs.sort_by);
        gnome_config_set_int("sort_order", general_prefs.sort_order);
        gnome_config_set_int("pane_position", general_prefs.pane_position);
        gnome_config_set_int("notebook_page", general_prefs.notebook_page);
        gnome_config_set_int("x_size", general_prefs.x_size);
        gnome_config_set_int("y_size", general_prefs.y_size);
        gnome_config_set_bool("save_win_on_quit",
                              general_prefs.save_win_on_quit);

        gnome_config_set_bool("favourites_overrides",
                              general_prefs.favourites_overrides);

        /* Save the list of active columns */
        tmp_list = g_list_first(general_prefs.active_columns);
        tmp_string_length = (g_list_length(tmp_list) * 3) + 1;
        tmp_string = g_malloc0(tmp_string_length);

        g_snprintf(tmp_string, tmp_string_length, "%i",
                   GPOINTER_TO_INT(tmp_list->data));
        tmp_list = g_list_next(tmp_list);

        while (tmp_list) {
                tmp = g_strdup(tmp_string);
                g_snprintf(tmp_string, tmp_string_length, "%s,%i", tmp,
                           GPOINTER_TO_INT(tmp_list->data));
                g_free(tmp);
                tmp_list = g_list_next(tmp_list);
        }
        gnome_config_set_string("active_columns", tmp_string);
        g_free(tmp_string);

        /* Save the list of column sizes */
        tmp_list = g_list_first(CListColumnLabels);
        tmp_string_length = (g_list_length(tmp_list) * 4) + 1;
        tmp_string = g_malloc0(tmp_string_length);

        tmp_data = tmp_list->data;
        g_snprintf(tmp_string, tmp_string_length, "%i", tmp_data->size);
        tmp_list = g_list_next(tmp_list);
        while (tmp_list) {
                tmp_data = tmp_list->data;
                tmp = g_strdup(tmp_string);
                g_snprintf(tmp_string, tmp_string_length, "%s,%i", tmp,
                           tmp_data->size);
                g_free(tmp);
                tmp_list = g_list_next(tmp_list);
        }

        gnome_config_set_string("column_sizes", tmp_string);
        g_free(tmp_string);

        gnome_config_set_bool("iconify_while_xmame",
                              general_prefs.iconify_while_xmame);
        gnome_config_set_bool("playback_no_highscores",
                              general_prefs.playback_no_highscore);
        gnome_config_set_bool("loadsa_image_formats",
                              general_prefs.loadsa_image_formats);
        gnome_config_set_bool("look_for_clone_images",
                              general_prefs.look_for_clone_images);
        gnome_config_set_bool("indicate_notebook_changes",
                              general_prefs.indicate_notebook_changes);

        gnome_config_pop_prefix();

        gnome_config_push_prefix("gRustibus/Filters/");

        gnome_config_set_bool("show_correct",
                              general_prefs.ListFilter.correct);
        gnome_config_set_bool("show_incorrect",
                              general_prefs.ListFilter.incorrect);
        gnome_config_set_bool("show_missing",
                              general_prefs.ListFilter.missing);
        gnome_config_set_bool("show_may_work",
                              general_prefs.ListFilter.may_work);
        gnome_config_set_bool("show_clones",
                              general_prefs.ListFilter.clones);
        gnome_config_set_bool("show_favourites",
                              general_prefs.ListFilter.favourites);

        gnome_config_set_bool("filters_active",
                              general_prefs.
                              ListFilter.advanced_filters_active);

        gnome_config_set_bool("driver_status_active",
                              general_prefs.
                              ListFilter.driver_status_active);
        gnome_config_set_bool("video_type_active",
                              general_prefs.ListFilter.video_type_active);
        gnome_config_set_bool("year_active",
                              general_prefs.ListFilter.year_active);
        gnome_config_set_bool("players_active",
                              general_prefs.ListFilter.players_active);
        gnome_config_set_bool("controls_active",
                              general_prefs.ListFilter.controls_active);
        gnome_config_set_bool("manufacturer_active",
                              general_prefs.
                              ListFilter.manufacturer_active);
        gnome_config_set_bool("driver_active",
                              general_prefs.ListFilter.driver_active);
        gnome_config_set_bool("category_active",
                              general_prefs.ListFilter.category_active);
        gnome_config_set_bool("screenshot_active",
                              general_prefs.ListFilter.screenshot_active);
        gnome_config_set_bool("flyer_active",
                              general_prefs.ListFilter.flyer_active);
        gnome_config_set_bool("cabinet_active",
                              general_prefs.ListFilter.cabinet_active);

        gnome_config_set_int("year_operator_type",
                             general_prefs.ListFilter.year_operator);
        gnome_config_set_int("players_operator_type",
                             general_prefs.ListFilter.players_operator);
        gnome_config_set_bool("controls_not",
                              general_prefs.
                              ListFilter.not_controls_active);
        gnome_config_set_bool("manufacturer_not",
                              general_prefs.
                              ListFilter.not_manufacturer_active);
        gnome_config_set_bool("driver_not",
                              general_prefs.ListFilter.not_driver_active);
        gnome_config_set_bool("category_not",
                              general_prefs.ListFilter.
                              not_category_active);
        gnome_config_set_bool("screenshot_not",
                              general_prefs.ListFilter.
                              not_screenshot_active);
        gnome_config_set_bool("flyer_not",
                              general_prefs.ListFilter.not_flyer_active);
        gnome_config_set_bool("cabinet_not",
                              general_prefs.ListFilter.not_cabinet_active);

        gnome_config_set_bool("good_driver",
                              general_prefs.ListFilter.driver_status_good);
        gnome_config_set_bool("vector_video",
                              general_prefs.ListFilter.video_type_vector);
        gnome_config_set_int("year", general_prefs.ListFilter.year_value);
        gnome_config_set_int("players",
                             general_prefs.ListFilter.players_value);
        gnome_config_set_string("controls",
                                general_prefs.ListFilter.controls_value);
        gnome_config_set_string("manufacturer",
                                general_prefs.
                                ListFilter.manufacturer_value);
        gnome_config_set_string("driver",
                                general_prefs.ListFilter.driver_value);
        gnome_config_set_string("category",
                                general_prefs.ListFilter.category_value);

        gnome_config_pop_prefix();

        gnome_config_sync();
}

/* FIXME: Passing a struct is too expensive. Pass a pointer instead */
void save_game_prefs(const gchar * filename, struct GameSettings settings)
{
        gchar *prefix = g_malloc(50);

        g_snprintf(prefix, 50, "%s/Misc/", filename);
        gnome_config_push_prefix(prefix);
        gnome_config_set_bool("default_options", settings.default_options);
        gnome_config_pop_prefix();

        if (!strcmp(filename, "/gRustibus.d/gameprefs_global")
            || !settings.default_options) {

                g_snprintf(prefix, 50, "%s/Sound/", filename);
                gnome_config_push_prefix(prefix);
                gnome_config_set_bool("sound", settings.sound);
                gnome_config_set_bool("samples", settings.samples);
                gnome_config_set_bool("fake_sound", settings.fake_sound);
                gnome_config_set_int("volume", settings.volume);
                gnome_config_pop_prefix();

                g_snprintf(prefix, 50, "%s/Video/", filename);
                gnome_config_push_prefix(prefix);
                gnome_config_set_bool("fullscreen", settings.fullscreen);
                gnome_config_set_bool("scanlines", settings.scanlines);
                gnome_config_set_bool("extra_artwork",
                                      settings.extra_artwork);
                gnome_config_set_bool("autoframeskip",
                                      settings.autoframeskip);
                gnome_config_set_bool("auto_colordepth",
                                      settings.auto_colordepth);
                gnome_config_set_bool("rotate_left", settings.rot_left);
                gnome_config_set_bool("rotate_right", settings.rot_right);
                gnome_config_set_bool("flip_x", settings.flipx);
                gnome_config_set_bool("flip_y", settings.flipy);
                gnome_config_set_int("scale", settings.scale);
                gnome_config_pop_prefix();

                g_snprintf(prefix, 50, "%s/Vector/", filename);
                gnome_config_push_prefix(prefix);
                gnome_config_set_bool("antialias", settings.antialias);
                gnome_config_set_bool("translucency",
                                      settings.translucency);
                gnome_config_set_int("vectorres", settings.vectorres);
                gnome_config_set_float("beam", settings.beam);
                gnome_config_set_float("flicker", settings.flicker);
                gnome_config_pop_prefix();

                g_snprintf(prefix, 50, "%s/Input/", filename);
                gnome_config_push_prefix(prefix);
                gnome_config_set_bool("mouse", settings.mouse);
                gnome_config_set_bool("analog_joy", settings.analog_joy);
                gnome_config_set_bool("windows_keys", settings.winkeys);
                gnome_config_set_bool("grab_mouse", settings.grab_mouse);
                gnome_config_set_int("joytype", settings.joytype);
                gnome_config_pop_prefix();

                g_snprintf(prefix, 50, "%s/Miscaleneous/", filename);
                gnome_config_push_prefix(prefix);
                gnome_config_set_bool("cheat", settings.cheat);
                gnome_config_set_string("extra_opts",
                                        settings.extra_options);
                gnome_config_pop_prefix();
        }
        gnome_config_sync();
        g_free(prefix);
        return;
}

/* Checks if the selected game has any settings saved, and if so, loads them */
void load_game_settings(const gchar * game)
{
        gchar *gamesettings_filename = g_malloc(50);

        g_snprintf(gamesettings_filename, 50,
                   "%s/.gnome/gRustibus.d/games/%s", getenv("HOME"), game);
        if (fopen(gamesettings_filename, "r")) {
                g_snprintf(gamesettings_filename, 50,
                           "/gRustibus.d/games/%s", game);
                g_free(game_settings.extra_options);
                game_settings =
                    load_game_prefs(gamesettings_filename, game_settings);
        } else {
                game_settings_set_to_default();
        }

        g_free(gamesettings_filename);
}


gboolean load_list()
{
        FILE *gamelist;
        gint romindex, i, count;

        gchar *filename;
        gchar line[2000];
        gchar **tmp_array;
        gchar *tmp, *p;
        RomEntry *rom;
        gboolean version_checked = FALSE;

        romindex = 0;
        grustibus_debug("Loading gamelist");

        filename = g_malloc(200);

        g_list_foreach(roms, (GFunc) g_free, NULL);
        g_list_free(roms);
        roms = NULL;

        g_snprintf(filename, 200, "%s/.gnome/gRustibus.d/gamelist",
                   getenv("HOME"));
        gamelist = fopen(filename, "r");
        if (gamelist == NULL) {
                g_free(filename);
                return (FALSE);
        }

        while (fgets(line, 500, gamelist)) {
                count = 0;
                i = 0;

                p = line;
                tmp = p;

                /* Skip comments */
                if (*tmp != '#') {
                        while (*tmp && (*tmp++ != '\n')) {
                                i++;
                                count++;
                        }

                        tmp_array = g_strsplit(p, ";", 21);

                        rom = g_malloc(sizeof(RomEntry));
                        if (rom == NULL)
                                g_warning
                                    ("Out of memory while loading gamelist");

                        g_snprintf(rom->romname, MAX_ROMNAME,
                                   "%s", tmp_array[0]);
                        g_snprintf(rom->gamename, MAX_GAMENAME,
                                   "%s", tmp_array[1]);
                        g_snprintf(rom->year, MAX_YEAR, "%s",
                                   tmp_array[2]);
                        g_snprintf(rom->manu, MAX_MANU, "%s",
                                   tmp_array[3]);
                        g_snprintf(rom->cloneof, MAX_ROMNAME,
                                   "%s", tmp_array[4]);
                        g_snprintf(rom->romof, MAX_ROMNAME,
                                   "%s", tmp_array[5]);
                        g_snprintf(rom->driver, MAX_ROMNAME,
                                   "%s", tmp_array[6]);

                        if (!strcmp(tmp_array[7], "Correct"))
                                rom->status = CORRECT;
                        else if (!strcmp(tmp_array[7], "Incorrect"))
                                rom->status = INCORRECT;
                        else if (!strcmp(tmp_array[7], "Missing"))
                                rom->status = MISSING;
                        else
                                rom->status = MAY_WORK;

                        if (strcmp(tmp_array[8], "")) {
                                g_snprintf(rom->cpu1, MAX_CPU,
                                           "%s", tmp_array[8]);
                        }
                        if (strcmp(tmp_array[9], "")) {
                                g_snprintf(rom->cpu2, MAX_CPU,
                                           "%s", tmp_array[9]);
                        }
                        if (strcmp(tmp_array[10], "")) {
                                g_snprintf(rom->cpu3, MAX_CPU,
                                           "%s", tmp_array[10]);
                        }
                        if (strcmp(tmp_array[11], "")) {
                                g_snprintf(rom->cpu4, MAX_CPU,
                                           "%s", tmp_array[11]);
                        }
                        if (strcmp(tmp_array[12], "")) {
                                g_snprintf(rom->sound1, MAX_CPU,
                                           "%s", tmp_array[12]);
                        }
                        if (strcmp(tmp_array[13], "")) {
                                g_snprintf(rom->sound2, MAX_CPU,
                                           "%s", tmp_array[13]);
                        }
                        if (strcmp(tmp_array[14], "")) {
                                g_snprintf(rom->sound3, MAX_CPU,
                                           "%s", tmp_array[14]);
                        }
                        if (strcmp(tmp_array[15], "")) {
                                g_snprintf(rom->sound4, MAX_CPU,
                                           "%s", tmp_array[15]);
                        }
                        rom->num_players = atoi(tmp_array[16]);
                        rom->num_buttons = atoi(tmp_array[17]);
                        if (strcmp(tmp_array[18], "")) {
                                g_snprintf(rom->control,
                                           MAX_CONTROL, "%s",
                                           tmp_array[18]);
                        }
                        if (!strcmp(tmp_array[19], "True"))
                                rom->vector = TRUE;
                        else
                                rom->vector = FALSE;

                        if (!strcmp(tmp_array[20], "True\n"))
                                rom->working = TRUE;
                        else
                                rom->working = FALSE;

                        g_strfreev(tmp_array);
                        roms = g_list_prepend(roms, (gpointer) rom);
                /* TODO: It might be better to call an external function for this */
                } else if (!version_checked) {
                        /* Check for the gRustibus version in the top comment line */
                        while (*tmp && (*tmp++ != '\n')) {
                                i++;
                                count++;
                        }
                        tmp_array = g_strsplit(p, " ", 5);

                        grustibus_debug
                            ("Checking version of gamelist file: %s - %s",
                             tmp_array[1], tmp_array[2]);
                        grustibus_debug("gamelist version: %f",
                                        g_strtod(tmp_array[2], NULL));

                        if (strcmp(tmp_array[1], "gRustibus")) {
                                grustibus_warning(_("Could not recognize format of gamelist.\nYou need to build a new gamelist."));
                                g_strfreev(tmp_array);
                                return (FALSE);
                        } else if (g_strtod(tmp_array[2], NULL) < 0.43) {
                                grustibus_debug
                                    ("Gamelist from too old gRustibus");
                                g_strfreev(tmp_array);

                                grustibus_warning (_("The format of the gamelist has changed in this version.\nYou need to build a new gamelist."));
                                return (TRUE);
                        }
                        g_strfreev(tmp_array);
                        version_checked = TRUE;
                }
        }

        fclose(gamelist);
        g_free(filename);
        roms = g_list_reverse(roms);

        if (!load_favourites()) {
                save_favourites();
                load_favourites();
        }

        load_catver();
        load_timesplayed();
        grustibus_debug("Finished loading data from disk.");
        

        get_entries_for_multiple_choice_filters ();
        return (TRUE);
}

gboolean save_list()
{
        gchar *filename;
        gchar *comment;
        gchar *tmp;
        FILE *gamelist;
        RomEntry *rom;
        GList *listpointer;

        grustibus_debug("Saving gamelist");

        filename = g_malloc(50);
        comment = g_malloc(500);
        tmp = g_malloc(10);
        g_snprintf(comment, 500,
                   "# gRustibus 0.43\n# Do not edit this file by hand unless you know what you are doing.\n# The fileformat is: romname;gamename;year;manufacturer;cloneof;romof;driver;status;cpu1;cpu2;cpu3;cpu4;sound1;sound2;sound3;sound4;num_players;num_buttons;control;vector;working_driver\n");
        g_snprintf(filename, 50, "%s/.gnome/gRustibus.d/gamelist",
                   getenv("HOME"));

        if ((gamelist = fopen(filename, "w")) == NULL) {
                g_free(filename);
                g_free(comment);
                return (FALSE);
        }

        fputs(comment, gamelist);

        listpointer = g_list_first(roms);
        while ((listpointer != NULL)) {
                rom = (RomEntry *) listpointer->data;
                fputs(rom->romname, gamelist);
                fputs(";", gamelist);
                fputs(rom->gamename, gamelist);
                fputs(";", gamelist);
                fputs(rom->year, gamelist);
                fputs(";", gamelist);
                fputs(rom->manu, gamelist);
                fputs(";", gamelist);
                fputs(rom->cloneof, gamelist);
                fputs(";", gamelist);
                fputs(rom->romof, gamelist);
                fputs(";", gamelist);
                fputs(rom->driver, gamelist);
                fputs(";", gamelist);

                switch (rom->status) {
                case (CORRECT):
                        fputs("Correct", gamelist);
                        break;
                case (INCORRECT):
                        fputs("Incorrect", gamelist);
                        break;
                case (MISSING):
                        fputs("Missing", gamelist);
                        break;
                case (MAY_WORK):
                        fputs("May work", gamelist);
                        break;
                }
                fputs(";", gamelist);

                if (rom->cpu1)
                        fputs(rom->cpu1, gamelist);
                fputs(";", gamelist);
                if (rom->cpu2)
                        fputs(rom->cpu2, gamelist);
                fputs(";", gamelist);
                if (rom->cpu3)
                        fputs(rom->cpu3, gamelist);
                fputs(";", gamelist);
                if (rom->cpu4)
                        fputs(rom->cpu4, gamelist);
                fputs(";", gamelist);
                if (rom->sound1)
                        fputs(rom->sound1, gamelist);
                fputs(";", gamelist);
                if (rom->sound2)
                        fputs(rom->sound2, gamelist);
                fputs(";", gamelist);
                if (rom->sound3)
                        fputs(rom->sound3, gamelist);
                fputs(";", gamelist);
                if (rom->sound4)
                        fputs(rom->sound4, gamelist);
                fputs(";", gamelist);

                g_snprintf(tmp, 10, "%i", rom->num_players);
                fputs(tmp, gamelist);
                fputs(";", gamelist);
                g_snprintf(tmp, 10, "%i", rom->num_buttons);
                fputs(tmp, gamelist);
                fputs(";", gamelist);
                fputs(rom->control, gamelist);
                fputs(";", gamelist);

                if (rom->vector)
                        fputs("True", gamelist);
                else
                        fputs("False", gamelist);
                fputs(";", gamelist);
                if (rom->working)
                        fputs("True", gamelist);
                else
                        fputs("False", gamelist);
                fputs("\n", gamelist);

                listpointer = g_list_next(listpointer);
        }

        g_free(filename);
        g_free(comment);
        fclose(gamelist);
        save_favourites();
        return (TRUE);
}


/* load_favourites() loads and parses the favourite file. Both the favourite
 * file and the list of games (the roms list) needs to be sorted. This way
 * the whole operation can be done in O(n+m) time.
 */
gboolean load_favourites()
{
        gint count, i, bogous = 0;
        gchar line[80];
        gchar *filename;
        gchar *romname;
        gchar *tmp;
        GList *favourites = NULL;
        GList *listpoint_fav = NULL;
        GList *listpoint_rom = NULL;
        GList *listpoint_lasthit = NULL;
        RomEntry *rom;
        FILE *favouritefile;

        grustibus_debug("Loading favourites");
        filename = g_malloc(50);
        g_snprintf(filename, 50, "%s/.gnome/gRustibus.d/favourites",
                   getenv("HOME"));

        if ((favouritefile = fopen(filename, "r")) == NULL) {
                g_free(filename);
                return (FALSE);
        }

        while (fgets(line, 80, favouritefile)) {
                count = 0;
                i = 0;

                romname = line;
                tmp = romname;

                /* Skip comments */
                if (*tmp != '#') {
                        while (*tmp && (*tmp++ != '\n')) {
                                i++;
                                count++;
                        }
                        romname[i] = 0;
                        favourites =
                            g_list_prepend(favourites,
                                           (gpointer) g_strdup(romname));
                }
        }

        fclose(favouritefile);
        g_free(filename);

        favourites = g_list_reverse(favourites);
        favourites = g_list_sort(favourites, (GCompareFunc) strcmp);

        listpoint_fav = g_list_first(favourites);
        listpoint_rom = g_list_first(roms);

        while ((listpoint_rom != NULL) && (listpoint_fav != NULL)) {
                rom = (RomEntry *) listpoint_rom->data;
                if (!strcmp(rom->romname, (gchar *) (listpoint_fav->data))) {
                        rom->favourite = TRUE;
                        g_free(listpoint_fav->data);
                        listpoint_fav = g_list_next(listpoint_fav);
                        listpoint_lasthit = listpoint_rom;
                } else {
                        rom->favourite = FALSE;
                }
                if (listpoint_rom->next == NULL
                    && listpoint_fav->next != NULL) {
                        bogous++;
                        listpoint_rom = listpoint_lasthit;
                        listpoint_fav = g_list_next(listpoint_fav);
                }
                listpoint_rom = g_list_next(listpoint_rom);
        }
        /* The rest of the list is not favourites */
        while ((listpoint_rom != NULL)) {
                rom = (RomEntry *) listpoint_rom->data;
                rom->favourite = FALSE;
                listpoint_rom = g_list_next(listpoint_rom);
        }

        if (bogous) {
                grustibus_warning(_
                                  ("Removed %i bogous (misspelled or non-existant)\n entries from your favourites list"),
                                  bogous);
        }

        g_list_free(favourites);

        save_favourites();
        return (TRUE);
}


void save_favourites()
{
        gchar *filename;
        gchar *comment;
        gchar *defaults;
        gchar *entry;
        GList *favourites = NULL;
        GList *listpoint_fav = NULL;
        GList *listpoint_rom = NULL;
        FILE *favouritefile;
        RomEntry *rom;

        comment = g_malloc(700);
        defaults = g_malloc(500);
        filename = g_malloc(50);

        grustibus_debug("Saving favourites");
        g_snprintf(comment, 700,
                   "# You can edit this file by hand. If you delete this\n"
                   "# file, the default list will be generated the next time\n"
                   "# gRustibus starts.\n" "#\n"
                   "# The default favourites are a collection of games that on one\n"
                   "# time or another was available in the legendary arcadehall /\n"
                   "# junkfood place Rustibus.");

        g_snprintf(filename, 50, "%s/.gnome/gRustibus.d/favourites",
                   getenv("HOME"));

        favourites =
            g_list_prepend(favourites, (gpointer) g_strdup(comment));

        listpoint_rom = g_list_first(roms);
        if (fopen(filename, "r")) {
                /* create favourite data since the roms list is already sorted, this is straight forward */
                while ((listpoint_rom != NULL)) {
                        rom = (RomEntry *) listpoint_rom->data;
                        if (rom->favourite) {
                                favourites =
                                    g_list_prepend(favourites, (gpointer)
                                                   g_strdup(rom->romname));
                        }
                        listpoint_rom = g_list_next(listpoint_rom);
                }

        } else {
                /* No file found, create default data */
                g_snprintf(defaults, 500, DEFAULT_FAVOURITES);
                favourites =
                    g_list_prepend(favourites,
                                   (gpointer) g_strdup(defaults));
        }

        favourites = g_list_reverse(favourites);
        favouritefile = fopen(filename, "w");

        /* save the list */
        listpoint_fav = g_list_first(favourites);
        while ((listpoint_fav != NULL)) {
                entry = listpoint_fav->data;
                fputs(entry, favouritefile);
                fputs("\n", favouritefile);
                g_free(entry);
                listpoint_fav = g_list_next(listpoint_fav);
        }

        fclose(favouritefile);
        g_free(filename);
        g_free(comment);
        g_free(defaults);
        g_list_free(favourites);
}

gboolean check_romdirs()
{
        time_t modified_seconds = 0;
        struct stat filestatbuf;
        gchar **romdir_array = NULL;
        gint array_count = 0;
        gint latest_modified_romdir = 0;
        gboolean changed = FALSE;

        /* Check if the romdir has changed */
        romdir_array = g_strsplit(general_prefs.rom_dir, ":", MAX_DIR);

        if (check_exists_dirs(general_prefs.rom_dir)) {
                while ((romdir_array[array_count] != NULL)) {
                        if (stat(romdir_array[array_count], &filestatbuf) >= 0) {            
                                modified_seconds = filestatbuf.st_mtime;
                                latest_modified_romdir = MAX((gint) modified_seconds, latest_modified_romdir);
                        }
                        array_count++;
                }

                /* Check the dir that changed last against the stored value */
                if ((gint) latest_modified_romdir > general_prefs.romdir_time) changed = TRUE;

                general_prefs.romdir_time = latest_modified_romdir;
                gnome_config_push_prefix("gRustibus/Settings/");
                gnome_config_set_int("romdir_time", general_prefs.romdir_time);
                gnome_config_pop_prefix();
                gnome_config_sync();
        }
        else changed=FALSE;

        g_strfreev(romdir_array);
        return (changed);
}

gboolean check_exists_dirs(gchar *dirstring){
        struct stat filestatbuf;
        gboolean missing=FALSE;
        gint array_count=0;
        gchar **romdir_array = NULL;

        romdir_array = g_strsplit(dirstring, ":", MAX_DIR);

        while ((romdir_array[array_count] != NULL)) {
               if (!missing) {
                       missing=(stat(romdir_array[array_count], &filestatbuf) < 0);
                       array_count++;
               }
               if (missing) {
                       grustibus_warning(_("Could not open directory \"%s\"\nPlease check your preferences."), romdir_array[array_count-1]);       
                       while ((romdir_array[array_count] != NULL)) array_count++;
               }
        } 
        
        g_strfreev(romdir_array);
        return (!missing);
}

gchar *graphic_formats[] =
    { "png", "gif", "jpg", "jpeg", "xpm", "bmp", "pnm", "tif", "tiff",
        NULL
};

gboolean
find_image(const gchar * directories,
           const gchar * game, const gchar * cloneof, gchar ** result)
{
        gint i, j, k;
        gchar **dirs = NULL;
        gchar *imagename = g_malloc(200);
        gchar *games[3];
        FILE *imagefile;

        *result = NULL;

        games[0] = g_strdup(game);
        if (!strcmp(cloneof, "-") || !general_prefs.look_for_clone_images) {
                games[1] = NULL;
        } else {
                games[1] = g_strdup(cloneof);
                games[2] = NULL;
        }

        if (directories != NULL)
                dirs = g_strsplit(directories, ":", MAX_DIR);
        if ((dirs == NULL) && (directories != NULL))
                dirs[0] = g_strdup(directories);

        for (i = 0; (games[i] != NULL); i++) {
                for (j = 0; (graphic_formats[j] != NULL); j++) {
                        for (k = 0; dirs[k]; k++) {
                                g_snprintf(imagename, 200, "%s/%s.%s",
                                           dirs[k], games[i],
                                           graphic_formats[j]);
                                grustibus_debug("Trying to open image %s",
                                                imagename);
                                if ((imagefile = fopen(imagename, "r"))) {
                                        *result = g_strdup(imagename);
                                        grustibus_debug("Found it!");
                                        fclose(imagefile);
                                }
                                if (*result != NULL)
                                        break;
                        }
                        /* NOTE: This requires that png is the first format in the list */
                        if (*result != NULL
                            || !general_prefs.loadsa_image_formats)
                                break;
                }
                if (*result != NULL)
                        break;
        }

        for (i = 0; games[i]; i++) {
                g_free(games[i]);
        }

        g_free(imagename);
        g_strfreev(dirs);

        return (*result != NULL);
}

gboolean create_text_report (const GameFilter *Filter, GList *columns, const gchar *filename) {
        GList *listpointer;
        RomEntry *tmprom;
        gchar **rom;
        gchar *tmpstring;
        gint i=0;
        FILE *reportfile;

        tmpstring = g_malloc(500);

        if ((reportfile = fopen(filename, "w")) == NULL) {
                return (FALSE);
        }

        listpointer = g_list_first (columns);
        
        listpointer = g_list_first(roms);
        
        while ((listpointer != NULL)) {
                tmprom = (RomEntry *) listpointer->data;
                if (filter_rom(tmprom, Filter)) {
                        rom = make_gamelist_entry(tmprom, columns);
                        for (i=0; i<g_list_length(columns); i++) {
                                if (i>0) {
                                        fputs(";", reportfile);
                                }
                                g_snprintf(tmpstring, 500, "%s", rom[i]);
                                fputs(tmpstring, reportfile);
                        }
                        fputs("\n", reportfile);
                }
                listpointer=g_list_next(listpointer);
        }
        
        g_free(tmpstring);
        fclose(reportfile);
        
        return (TRUE);
}


gboolean create_pretty_text_report (const GameFilter *Filter, GList *columns, const gchar *filename) {
        GList *listpointer;
        RomEntry *tmprom;
        ColumnLabel *column;
        gchar **rom;
        gchar *countstring;
        gchar *tmpstring;
        gchar *date;
        time_t now;
        gint i=0, length=0;
        gint columnwidth[g_list_length(columns)];
        FILE *reportfile;

        tmpstring = g_malloc(500);

        if ((reportfile = fopen(filename, "w")) == NULL) {
                return (FALSE);
        }

        date=g_malloc(100);
        time (&now);
        g_snprintf (date, 100, "%s", ctime (&now));

        for (i=0; i<g_list_length(columns); i++) {
                columnwidth[i]=0;
        }
        
        /* Find the max width of each column */
        listpointer = g_list_first(roms);
        while ((listpointer != NULL)) {
                tmprom = (RomEntry *) listpointer->data;
                if (filter_rom(tmprom, Filter)) {
                        rom = make_gamelist_entry(tmprom, columns);
                        for (i=0; i<g_list_length(columns); i++) {
                                while (*rom[i]++ != '\0') length++;
                                if (length > columnwidth[i]) {
                                        columnwidth[i]=length;
                                }
                                length=0;
                        }
                }
                listpointer=g_list_next(listpointer);
        } 

        /* The header might have longer strings than the actual list.. */
        listpointer=g_list_first(columns);
        for (i=0; i<g_list_length(columns); i++) {
                column = g_list_nth_data (all_columns, GPOINTER_TO_INT(listpointer->data));           
                countstring=column->label;
                length=-1;
                while (*countstring++ != '\0') length++;
                if (length > columnwidth[i]) {
                        columnwidth[i]=length;
                }
                listpointer=g_list_next(listpointer);
        } 
   

        g_snprintf (tmpstring, 500, _("This list was created by gRustibus %s on %s using xmame(%s)-%i.%s.%i.\nThe following rules were applied:\n\n"), 
                        VERSION, date, general_prefs.xmame_display_target, general_prefs.xmame_major, general_prefs.xmame_minor, general_prefs.xmame_release);
        fputs (tmpstring, reportfile);

        if (!Filter->clones) {
                fputs (" * ", reportfile); 
                g_snprintf (tmpstring, 500, _("Don't list clones"));
                fputs (tmpstring, reportfile);
                fputs ("\n", reportfile); 
        }
        if (!Filter->correct) {
                fputs (" * ", reportfile); 
                g_snprintf (tmpstring, 500, _("Don't list correct romsets"));
                fputs (tmpstring, reportfile);
                fputs ("\n", reportfile); 
        }
        if (!Filter->incorrect) {
                fputs (" * ", reportfile); 
                g_snprintf (tmpstring, 500, _("Don't list incorrect romsets"));
                fputs (tmpstring, reportfile);
                fputs ("\n", reportfile); 
        }
        if (!Filter->may_work) {
                fputs (" * ", reportfile); 
                g_snprintf (tmpstring, 500, _("Don't list may work romsets"));
                fputs (tmpstring, reportfile);
                fputs ("\n", reportfile); 
        }
        if (!Filter->missing) {
                fputs (" * ", reportfile); 
                g_snprintf (tmpstring, 500, _("Don't list missing romsets"));
                fputs (tmpstring, reportfile);
                fputs ("\n", reportfile); 
        }
        if (Filter->favourites) { 
                fputs (" * ", reportfile); 
                g_snprintf (tmpstring, 500, _("List favourites only"));
                fputs (tmpstring, reportfile);
                fputs ("\n", reportfile); 
        }

        if (Filter->advanced_filters_active) {
                if (Filter->driver_status_active) {
                        fputs(" * ", reportfile);
                        if (Filter->driver_status_good) g_snprintf (tmpstring, 500, _("List games with driver status \"good\" only"));
                        else g_snprintf (tmpstring, 500, _("List games with driver status \"not good\" only"));
                        fputs (tmpstring, reportfile);
                        fputs ("\n", reportfile); 
                }
                if (Filter->video_type_active) {
                        fputs(" * ", reportfile);
                        if (Filter->video_type_vector)g_snprintf (tmpstring, 500, _("List games that use vector graphics only"));
                        else g_snprintf (tmpstring, 500, _("List games that use raster graphics only"));
                        fputs (tmpstring, reportfile);
                        fputs ("\n", reportfile); 
                }
                if (Filter->year_active) {
                        printf(" * ");
                        switch (Filter->year_operator) {
                        case 0:
                                g_snprintf (tmpstring, 500, _("List games with release year less than %i only"), Filter->year_value);
                                break;
                        case 1:
                                g_snprintf (tmpstring, 500, _("List games whth release year larger than  %i only"), Filter->year_value);
                                break;
                        case 2:
                                g_snprintf (tmpstring, 500, _("Don't list games with release year equal to %i"), Filter->year_value);
                                break;
                        case 3:
                                g_snprintf (tmpstring, 500, _("List games with release year equal to %i only"), Filter->year_value);
                                break;
                        }
                        fputs (tmpstring, reportfile);
                        fputs ("\n", reportfile); 
                }
                if (Filter->players_active) {
                        fputs(" * ", reportfile);
                        switch (Filter->players_operator) {
                        case 0:
                                g_snprintf (tmpstring, 500, _("List games that support less than %i players only"), Filter->players_value);
                                break;
                        case 1:
                                g_snprintf (tmpstring, 500, _("List games that support more than %i players only"), Filter->players_value);
                                break;
                        case 2:
                                g_snprintf (tmpstring, 500, _("Don't list games that support excactly %i players"), Filter->players_value);
                                break;
                        case 3:
                                g_snprintf (tmpstring, 500, _("List games that support excactly %i players only"), Filter->players_value);
                                break;
                        }
                        fputs (tmpstring, reportfile);
                        fputs ("\n", reportfile); 
                }
                if (Filter->controls_active) {
                        fputs(" * ", reportfile);
                        if (Filter->not_controls_active) g_snprintf(tmpstring, 500, _("Don't list games that use %s controls"), Filter->controls_value);
                        else g_snprintf(tmpstring, 500, _("List games that use %s controls only"), Filter->controls_value);
                        fputs (tmpstring, reportfile);
                        fputs ("\n", reportfile); 
                            
                }
                if (Filter->manufacturer_active) {
                        fputs(" * ", reportfile);
                        if (Filter->not_manufacturer_active) g_snprintf(tmpstring, 500, _("Don't list games manufactured by %s"), Filter->manufacturer_value);
                        else g_snprintf(tmpstring, 500, _("List games manufactured by %s only"), Filter->manufacturer_value);
                        fputs (tmpstring, reportfile);
                        fputs ("\n", reportfile); 

                                          }
                if (Filter->driver_active) {
                        fputs(" * ", reportfile);
                        if (Filter->not_driver_active) g_snprintf(tmpstring, 500, _("Don't list games using the %s driver"), Filter->driver_value);
                        else g_snprintf(tmpstring, 500, _("List games using the %s driver only"), Filter->driver_value);
                        fputs (tmpstring, reportfile);
                        fputs ("\n", reportfile); 
                }
                if (Filter->category_active) {
                        fputs(" * ", reportfile);
                        if (Filter->not_category_active) g_snprintf (tmpstring, 500, _("Don't list games in the %s category"), Filter->category_value);
                        else g_snprintf (tmpstring, 500, _("List games in the %s category only"), Filter->category_value);
                        fputs (tmpstring, reportfile);
                        fputs ("\n", reportfile); 
                }

                if (Filter->cabinet_active) {
                        fputs(" * ", reportfile);
                        if (Filter->not_cabinet_active) g_snprintf (tmpstring, 500, _("Don't list games with cabinet picture installed"));
                        else g_snprintf (tmpstring, 500, _("List games with cabinet picture installed only"));
                        fputs (tmpstring, reportfile);
                        fputs ("\n", reportfile); 
                }
                if (Filter->flyer_active) {
                        fputs(" * ", reportfile);
                        if (Filter->not_flyer_active)g_snprintf (tmpstring, 500, _("Don't list games with flyer picture installed"));
                        else g_snprintf (tmpstring, 500, _("List games with flyer picture installed only"));
                        fputs (tmpstring, reportfile);
                        fputs ("\n", reportfile); 
                }
                if (Filter->screenshot_active) {
                        fputs(" * ", reportfile);
                        if (Filter->not_screenshot_active)g_snprintf (tmpstring, 500, _("Don't list games with screenshot installed"));
                        else g_snprintf (tmpstring, 500, _("List games with screenshot installed only"));
                        fputs (tmpstring, reportfile);
                        fputs ("\n", reportfile); 
                }
        }

        fputs ("\n\n", reportfile);
        
        /* Print header */
        for (i=0; i<g_list_length(columns); i++) {
                length=-1;
                fputs ("+", reportfile);
                while  (length++ < ((columnwidth[i])+2)) fputs("-", reportfile);
        }
        fputs ("+\n", reportfile);

        listpointer=g_list_first(columns);
        for (i=0; i<g_list_length(columns); i++) {
                column = g_list_nth_data (all_columns, GPOINTER_TO_INT(listpointer->data));           
                countstring=column->label;
                length=-1;
                fputs ("|", reportfile);
                while (*countstring++ != '\0') length++;
                g_snprintf (tmpstring, 500, " %s ", column->label);
                fputs (tmpstring, reportfile);
                while  (length++ < (columnwidth[i])) fputs(" ", reportfile);
                listpointer=g_list_next(listpointer);
        }
        fputs ("|\n", reportfile);

        for (i=0; i<g_list_length(columns); i++) {
                length=-1;
                fputs ("+", reportfile);
                while  (length++ < ((columnwidth[i]+2))) fputs("-", reportfile);
        }
        fputs ("+\n", reportfile);
        
        /* Print body */
        listpointer = g_list_first(roms);
        while ((listpointer != NULL)) {
                tmprom = (RomEntry *) listpointer->data;
                if (filter_rom(tmprom, Filter)) {
                        rom = make_gamelist_entry(tmprom, columns);
                        for (i=0; i<g_list_length(columns); i++) {
                                length=-1;
                                countstring=rom[i];
                                fputs ("|", reportfile);
                                g_snprintf(tmpstring, 500, " %s ", rom[i]);
                                fputs (tmpstring, reportfile);
                                while (*countstring++ != '\0') length++;
                                while  (length++ < (columnwidth[i])) fputs(" ", reportfile);
                        }
                        fputs("|\n", reportfile);
                }
                listpointer=g_list_next(listpointer);
        }
        
        /* Print footer */
        for (i=0; i<g_list_length(columns); i++) {
                length=-1;
                fputs ("+", reportfile);
                while  (length++ < ((columnwidth[i]+2))) fputs ("-", reportfile);
        }
        fputs ("+\n", reportfile);

        g_free (tmpstring);
        g_free (date);
        fclose (reportfile);
        
        return (TRUE);
}

gboolean create_html_report (const GameFilter *Filter, GList *columns, const gchar *filename) {
        GList *listpointer;
        RomEntry *tmprom;
        ColumnLabel *column;
        gchar **rom;
        gchar *tablerowstring[2];
        gchar *date;
        gchar *tmpstring;
        time_t now;
        gint i=0, k=0, rowtype=0;
        FILE *reportfile;


        if ((reportfile = fopen(filename, "w")) == NULL) {
                return (FALSE);
        }
        
        date=g_malloc(100);
        tmpstring=g_malloc(500);

        time (&now);
        g_snprintf (date, 100, "%s", ctime (&now));

        fputs ("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\"\n\t\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n", reportfile);
        fputs ("<html xmlns=\"http://www.w3.org/1999/xhtml\">\n", reportfile);
        fputs ("\t<head>\n", reportfile);
        g_snprintf (tmpstring, 500, _("\t\t<title>gRustibus Game Collection report</title>\n"));
        fputs (tmpstring, reportfile);
        g_snprintf(tmpstring, 500, "\t\t<meta name=\"Author\" content=\"gRustibus %s\" />\n",VERSION);
        fputs (tmpstring, reportfile);
        fputs ("\t\t<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\" />\n", reportfile);
                
        fputs (REPORTSTYLESHEET, reportfile);
        fputs ("\t</head>\n", reportfile);

        fputs ("\t<body>\n", reportfile);
        g_snprintf (tmpstring, 500, _("\t\t<h1>gRustibus Game Collection Report</h1>\n"));
        fputs (tmpstring, reportfile);
        fputs ("\t\t<div class=\"info\">\n", reportfile);
        fputs ("\t\t\t<p>\n", reportfile);
        g_snprintf (tmpstring, 500, _("\t\t\t\tThis list was created by gRustibus %s on %s using xmame(%s)-%i.%s.%i.\n\t\t\t\t<br />\n\t\t\t\tThe following rules were applied:\n"), 
                        VERSION, date, general_prefs.xmame_display_target, general_prefs.xmame_major, general_prefs.xmame_minor, general_prefs.xmame_release);
        fputs (tmpstring, reportfile);
        fputs ("\t\t\t</p>\n", reportfile);
        fputs ("\t\t\t<ul>\n", reportfile);
        
        if (!Filter->clones) {
                fputs ("\t\t\t\t<li>", reportfile);
                g_snprintf (tmpstring, 500, _("Don't list clones"));
                fputs (tmpstring, reportfile);
                fputs ("</li>\n", reportfile);
        }
        if (!Filter->correct) {
                fputs ("\t\t\t\t<li>", reportfile);
                g_snprintf (tmpstring, 500, _("Don't list correct romsets"));
                fputs (tmpstring, reportfile);
                fputs ("</li>\n", reportfile);
        }
        if (!Filter->incorrect) { 
                fputs ("\t\t\t\t<li>", reportfile);
                g_snprintf (tmpstring, 500, _("Don't list incorrect romsets"));
                fputs (tmpstring, reportfile);
                fputs ("</li>\n", reportfile);
        }
        if (!Filter->may_work) {
                fputs ("\t\t\t\t<li>", reportfile);
                g_snprintf (tmpstring, 500, _("Don't list may work romsets"));
                fputs (tmpstring, reportfile);
                fputs ("</li>\n", reportfile);
        }
        if (!Filter->missing) {
                fputs ("\t\t\t\t<li>", reportfile);
                g_snprintf (tmpstring, 500, _("Don't list missing romsets"));
                fputs (tmpstring, reportfile);
                fputs ("</li>\n", reportfile);
        }
        if (Filter->favourites) {
                fputs ("\t\t\t\t<li>", reportfile);
                g_snprintf (tmpstring, 500, _("List favourites only"));
                fputs (tmpstring, reportfile);
                fputs ("</li>\n", reportfile);
        }
        if (Filter->advanced_filters_active) {
                if (Filter->driver_status_active) {
                        fputs ("\t\t\t\t<li>", reportfile);
                        if (Filter->driver_status_good) g_snprintf (tmpstring, 500, _("List games with driver status \"good\" only"));
                        else g_snprintf (tmpstring, 500, _("List games with driver status \"not good\" only"));
                        fputs (tmpstring, reportfile);
                        fputs ("</li>\n", reportfile);
                }
                if (Filter->video_type_active) {
                        fputs ("\t\t\t\t<li>", reportfile);
                        if (Filter->video_type_vector)g_snprintf (tmpstring, 500, _("List games that use vector graphics only"));
                        else g_snprintf (tmpstring, 500, _("List games that use raster graphics only"));
                        fputs (tmpstring, reportfile);
                        fputs ("</li>\n", reportfile);
                }
                if (Filter->year_active) {
                        fputs ("\t\t\t\t<li>", reportfile);
                        switch (Filter->year_operator) {
                        case 0:
                                g_snprintf (tmpstring, 500, _("List games with release year less than %i only"), Filter->year_value); 
                                break; 
                        case 1:
                                g_snprintf (tmpstring, 500, _("List games with release year larger than to %i only"), Filter->year_value);
                                break; 
                        case 2:
                                g_snprintf (tmpstring, 500, _("Don't list games with release year equal to %i"), Filter->year_value); 
                                break; 
                        case 3:
                                g_snprintf (tmpstring, 500, _("List games with release year equal to %i only"), Filter->year_value); 
                                break; 
                        }
                        fputs (tmpstring, reportfile);
                        fputs ("</li>\n", reportfile);
                }
                if (Filter->players_active) {
                        fputs ("\t\t\t\t<li>", reportfile);
                        switch (Filter->players_operator) {
                        case 0:
                                g_snprintf (tmpstring, 500, _("List games that support less than %i players only"), Filter->players_value); 
                                break; 
                        case 1:
                                g_snprintf (tmpstring, 500, _("List games that support more than %i players only"), Filter->players_value); 
                                break; 
                        case 2:
                                g_snprintf (tmpstring, 500, _("Don't list games that support excactly %i players"), Filter->players_value); 
                                break; 
                        case 3:
                                g_snprintf (tmpstring, 500, _("List games that support excactly %i players only"), Filter->players_value); 
                                break; 
                        }
                        fputs (tmpstring, reportfile);
                        fputs ("</li>\n", reportfile);
                }
                if (Filter->controls_active) {
                        fputs ("\t\t\t\t<li>", reportfile);
                        if (Filter->not_controls_active) g_snprintf (tmpstring, 500, _("Don't list games that use %s controls"), Filter->controls_value);
                        else g_snprintf (tmpstring, 500, _("List games that use %s controls only"), Filter->controls_value);
                        fputs (tmpstring, reportfile);
                        fputs ("</li>\n", reportfile);
                }
                if (Filter->manufacturer_active) {
                        fputs ("\t\t\t\t<li>", reportfile);
                        if (Filter->not_manufacturer_active) g_snprintf (tmpstring, 500, _("Don't list games manufactured by %s"), Filter->manufacturer_value);
                        else g_snprintf (tmpstring, 500, _("List games manufactured by %s only"), Filter->manufacturer_value);
                        fputs (tmpstring, reportfile);
                        fputs ("</li>\n", reportfile);
                }
                if (Filter->driver_active) {
                        fputs ("\t\t\t\t<li>", reportfile);
                        if (Filter->not_driver_active) g_snprintf (tmpstring, 500, _("Don't list games using the %s driver"), Filter->driver_value); 
                        else g_snprintf (tmpstring, 500, _("List games using the %s driver only"), Filter->driver_value);
                        fputs (tmpstring, reportfile);
                        fputs ("</li>\n", reportfile);
                }
                if (Filter->category_active) {
                        fputs ("\t\t\t\t<li>", reportfile);
                        if (Filter->not_category_active) g_snprintf (tmpstring, 500, _("Don't list games in the %s category"), Filter->category_value); 
                        else g_snprintf (tmpstring, 500, _("List games in the %s category only"), Filter->category_value);
                        fputs (tmpstring, reportfile);
                        fputs ("</li>\n", reportfile);
                }

                if (Filter->cabinet_active) {
                        fputs ("\t\t\t\t<li>", reportfile);
                        if (Filter->not_cabinet_active) g_snprintf (tmpstring, 500, _("Don't list games with cabinet picture installed")); 
                        else g_snprintf (tmpstring, 500, _("List games with cabinet picture installed only"));
                        fputs (tmpstring, reportfile);
                        fputs ("</li>\n", reportfile);
                }
                if (Filter->flyer_active) {
                        fputs ("\t\t\t\t<li>", reportfile);
                        if (Filter->not_flyer_active)g_snprintf (tmpstring, 500, _("Don't list games with flyer picture installed")); 
                        else g_snprintf (tmpstring, 500, _("List games with flyer picture installed only"));
                        fputs (tmpstring, reportfile);
                        fputs ("</li>\n", reportfile);
                }
                if (Filter->screenshot_active) {
                        fputs ("\t\t\t\t<li>", reportfile);
                        if (Filter->not_screenshot_active)g_snprintf (tmpstring, 500, _("Don't list games with screenshot installed"));
                        else g_snprintf (tmpstring, 500, _("List games with screenshot installed only"));
                        fputs (tmpstring, reportfile);
                        fputs ("</li>\n", reportfile);
                }
        }
        fputs ("\t\t\t</ul>\n", reportfile);
        fputs ("\t\t</div>\n\t\t<br />\n", reportfile);

        /* Print table header */
        fputs ("\t\t<table border=\"0\" width=\"100%%\" cellspacing=\"0\" cellpadding=\"5\">\n\t\t\t<thead>\n\t\t\t\t<tr class=\"header\">\n" ,reportfile);
        for (listpointer=g_list_first(columns); listpointer != NULL ; listpointer=g_list_next(listpointer)) {
                column = g_list_nth_data (all_columns, GPOINTER_TO_INT(listpointer->data));           
                g_snprintf (tmpstring, 500, "\t\t\t\t\t<th> %s </th>\n", column->label);
                fputs (tmpstring, reportfile);
        }
        fputs("\t\t\t\t</tr>\n\t\t\t</thead>\n\t\t\t<tbody>\n", reportfile);

        /* Print table body */
        listpointer = g_list_first(roms);
        tablerowstring[0] = g_strdup ("even");
        tablerowstring[1] = g_strdup ("odd");
        while ((listpointer != NULL)) {
                tmprom = (RomEntry *) listpointer->data;
                if (filter_rom(tmprom, Filter)) {
                        if (k%4 == 0) rowtype = !rowtype;
                        g_snprintf (tmpstring, 500, "\t\t\t\t<tr class=\"%s\">\n", tablerowstring[rowtype]);
                        fputs (tmpstring, reportfile);
                        rom = make_gamelist_entry(tmprom, columns);
                        for (i=0; i<g_list_length(columns); i++) {
                                g_snprintf(tmpstring, 500, "\t\t\t\t\t<td> %s </td>\n", rom[i]);
                                fputs (tmpstring, reportfile);
                        }
                        k++;
                        fputs ("\t\t\t\t</tr>\n", reportfile);
                }
                listpointer=g_list_next(listpointer);
        }
        g_free (tablerowstring[0]);
        g_free (tablerowstring[1]);
        
        /* Print footer */
        fputs ("\t\t\t</tbody>\n\t\t</table>\n\t</body>\n</html>\n", reportfile);

        g_free (tmpstring);
        g_free (date);
        fclose (reportfile);

        return (TRUE);
}

void set_timestamp () {
        time_t now = 0;

        time(&now);
        general_prefs.romdir_time = (gint) now;
        gnome_config_push_prefix("gRustibus/Settings/");
        gnome_config_set_int("romdir_time", general_prefs.romdir_time);
        gnome_config_pop_prefix();
}

// vim: ai sw=8 ts=8
