/*
 * gRustibus.c -- Main sourcefile. Initializes things. It also contains some
 * general functions.  
 *
 * gRustibus (c) 2000-2001 Kjetil Thuen <kjetilt@users.sourceforge.net>
 * MAME frontend for the GNOME desktop environment.
 *
 * gRustibus is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * This code is indented with GNU indent using k&r style with 8 character
 * indents as suggested by the GNOME project. Please keep it that way.
 * ($ indent -kr -ts8 -i8 *.[ch])
 *
 * Last modified: 25 April 2001 <kjetilt@users.sourceforge.net>
 *
 */

#include "grustibus.h"
#include "fileio.h"
#include "gui.h"
#include "mameio.h"
#include "support.h"
#include "interface.h"
#include "callbacks.h"

int main(int argc, char *argv[])
{
        CORBA_Environment ev;
        GnorbaInitFlags gnorba_flags = 0;
        

        bindtextdomain(PACKAGE, PACKAGE_LOCALE_DIR);
        textdomain(PACKAGE);

        CORBA_exception_init (&ev);
        gnorba_CORBA_init (&argc, argv, gnorba_flags, &ev);
        
        gnome_init("grustibus", VERSION, argc, argv);

        main_window = create_main_window();

        make_dir();

        load_prefs();
        startup_main();
        startup_gui(main_window);

        grustibus_debug("gRustibus Version %s started, succesfull init.",
                        VERSION);
        gtk_main();
        return 0;
}

void startup_main()
{
        ColumnLabel *tmpcolumnlabelstruct;
        gboolean gamelistexists;
        gint i;
        gchar **buttons;
        buttons = g_new (gchar*, 3);
        buttons[0]=GNOME_STOCK_BUTTON_YES;
        buttons[1]=GNOME_STOCK_BUTTON_NO;
        buttons[2]=NULL;

        /* Initialize global variables */
        lookup_filo = NULL;
        romlookup.pos = general_prefs.current_row;
        romlookup.search_string[0] = '\0';
        xmame_version_ok = FALSE;

        srand((int) (time(NULL)));


        /* Set up the titles for the columns */
        for (i = 0; i < COLUMN_COUNT; i++) {
                columntitles[i] = g_malloc0(MAX_COLUMN_NAME);
        }
        g_snprintf(columntitles[FAVORITE], MAX_COLUMN_NAME, _("Favourite"));
        g_snprintf(columntitles[GAMENAME], MAX_COLUMN_NAME, _("Game name"));
        g_snprintf(columntitles[ROMNAME], MAX_COLUMN_NAME, _("Rom name"));
        g_snprintf(columntitles[MANU], MAX_COLUMN_NAME, _("Manufacturer"));
        g_snprintf(columntitles[YEAR], MAX_COLUMN_NAME, _("Year"));
        g_snprintf(columntitles[CATEGORY], MAX_COLUMN_NAME, _("Category"));
        g_snprintf(columntitles[STATUS], MAX_COLUMN_NAME, _("Status"));
        g_snprintf(columntitles[CLONE], MAX_COLUMN_NAME, _("Clone of"));
        g_snprintf(columntitles[ROMOF], MAX_COLUMN_NAME, _("Rom of"));
        g_snprintf(columntitles[NUMPLAYERS], MAX_COLUMN_NAME, _("Players"));
        g_snprintf(columntitles[CONTROL], MAX_COLUMN_NAME, _("Control"));
        g_snprintf(columntitles[NUMBUTTONS], MAX_COLUMN_NAME, _("Buttons"));
        g_snprintf(columntitles[MAMEVER], MAX_COLUMN_NAME, _("Mame version"));
        g_snprintf(columntitles[DRIVER], MAX_COLUMN_NAME, _("Driver"));
        g_snprintf(columntitles[VECTOR], MAX_COLUMN_NAME, _("Vector"));
        g_snprintf(columntitles[DRIVERSTATUS], MAX_COLUMN_NAME, _("Driver status"));
        g_snprintf(columntitles[CPU1], MAX_COLUMN_NAME, _("CPU 1"));
        g_snprintf(columntitles[CPU2], MAX_COLUMN_NAME, _("CPU 2"));
        g_snprintf(columntitles[CPU3], MAX_COLUMN_NAME, _("CPU 3"));
        g_snprintf(columntitles[CPU4], MAX_COLUMN_NAME, _("CPU 4"));
        g_snprintf(columntitles[SOUND1], MAX_COLUMN_NAME, _("Sound 1"));
        g_snprintf(columntitles[SOUND2], MAX_COLUMN_NAME, _("Sound 2"));
        g_snprintf(columntitles[SOUND3], MAX_COLUMN_NAME, _("Sound 3"));
        g_snprintf(columntitles[SOUND4], MAX_COLUMN_NAME, _("Sound 4"));
        g_snprintf(columntitles[TIMESPLAYED], MAX_COLUMN_NAME, _("Times Played"));

        for (i = 0; i < COLUMN_COUNT; i++) {
                tmpcolumnlabelstruct = g_malloc(sizeof(ColumnLabel));
                tmpcolumnlabelstruct->label = g_strdup(columntitles[i]);
                tmpcolumnlabelstruct->number = i;
                all_columns = g_list_append(all_columns, tmpcolumnlabelstruct);
        }

        /* Load the gamelist */
        /* Print warnings if some setting points to none existant or wrong files/dirs */
        /* Just print one error so the screen is not filled with errors on the first run */
        gamelistexists = load_list();
        if (!gamelistexists) {
                grustibus_warning(_("No gamelist found, please check your preferences,\nthen select \"Rebuild gamelist\" in the file menu"));
        }
        else if (check_xmame_exe() && xmame_version_ok) {
                save_prefs();
                if (!grustibus_question(buttons, _("A new xmame version has been installed since you last ran gRustibus.\nYou should rebuild your gamelist to make gRustibus reflect the changes.\nWould you like to build a new gamelist now?"))){
                        build_new_game_list(); 
                } 
        }
        else if (check_romdirs()) {
                save_prefs();
                if (!grustibus_question(buttons, _("One or more of your romdirs has changed since you last ran gRustibus.\nYou should rebuild your gamelist to make gRustibus reflect the changes.\nWould you like to build a new gamelist now?"))){
                        build_new_game_list(); 
                }
        }
        else if (check_exists_dirs (general_prefs.screenshot_dir) &&
                 check_exists_dirs (general_prefs.flyer_dir) &&
                 check_exists_dirs (general_prefs.cabinet_dir) &&
                 check_exists_dirs (general_prefs.highscore_dir) &&
                 check_exists_dirs (general_prefs.playback_dir)) {/*Nothing :)*/ }
}

/* Called when a game is selected, this function takes care of all the game setting stuff */
void set_game(const gchar * game)
{
        load_game_settings(game);

        if (game_settings.default_options) {
                game_settings_set_to_default();
                game_settings.default_options = TRUE;
        }

}

void game_settings_set_to_default()
{
        g_free(game_settings.extra_options);

        game_settings.default_options = global_settings.default_options;

        game_settings.fullscreen = global_settings.fullscreen;
        game_settings.scanlines = global_settings.scanlines;
        game_settings.extra_artwork = global_settings.extra_artwork;
        game_settings.autoframeskip = global_settings.autoframeskip;
        game_settings.auto_colordepth = global_settings.auto_colordepth;
        game_settings.rot_left = global_settings.rot_left;
        game_settings.rot_right = global_settings.rot_right;
        game_settings.flipx = global_settings.flipx;
        game_settings.flipy = global_settings.flipy;
        game_settings.scale = global_settings.scale;

        game_settings.vectorres = global_settings.vectorres;
        game_settings.antialias = global_settings.antialias;
        game_settings.translucency = global_settings.translucency;
        game_settings.beam = global_settings.beam;
        game_settings.flicker = global_settings.flicker;

        game_settings.analog_joy = global_settings.analog_joy;
        game_settings.mouse = global_settings.mouse;
        game_settings.winkeys = global_settings.winkeys;
        game_settings.grab_mouse = global_settings.grab_mouse;
        game_settings.joytype = global_settings.joytype;

        game_settings.sound = global_settings.sound;
        game_settings.samples = global_settings.samples;
        game_settings.fake_sound = global_settings.fake_sound;
        game_settings.volume = global_settings.volume;

        game_settings.cheat = global_settings.cheat;
        game_settings.extra_options =
            g_strdup(global_settings.extra_options);
}

gint compare_roms(const void *a, const void *b)
{
        const RomEntry *rom1 = a;
        const RomEntry *rom2 = b;

        return strcmp(rom1->romname, rom2->romname);
}

gboolean filter_rom(const RomEntry * rom, const GameFilter * Filter)
{
        gchar *tmp = NULL;
        gboolean foundimage = FALSE;

        if (!Filter->clones && !((!strcmp(rom->cloneof, "-")
                                  ||
                                  (!strcmp(rom->romname, rom->cloneof)))))
                return (FALSE);

        if (Filter->favourites && (!rom->favourite))
                 return (FALSE);
        if (!Filter->correct && (rom->status == CORRECT))
                return (FALSE);
        if (!Filter->incorrect && (rom->status == INCORRECT))
                return (FALSE);
        if (!Filter->may_work && (rom->status == MAY_WORK))
                return (FALSE);
        if (!Filter->missing && (rom->status == MISSING))
                return (FALSE);

        if (Filter->advanced_filters_active) {
                if (Filter->driver_status_active) {
                        if (Filter->driver_status_good && !rom->working)
                                return (FALSE);
                        if (!Filter->driver_status_good && rom->working)
                                return (FALSE);
                }
                if (Filter->video_type_active) {
                        if (Filter->video_type_vector && !rom->vector)
                                return (FALSE);
                        if (!Filter->video_type_vector && rom->vector)
                                return (FALSE);
                }
                if (Filter->year_active) {
                        switch (Filter->year_operator) {
                        case 0:
                                if (atoi(rom->year) >= Filter->year_value)
                                        return (FALSE);
                                break;
                        case 1:
                                if (atoi(rom->year) <= Filter->year_value)
                                        return (FALSE);
                                break;
                        case 2:
                                if (atoi(rom->year) == Filter->year_value)
                                        return (FALSE);
                                break;
                        case 3:
                                if (atoi(rom->year) != Filter->year_value)
                                        return (FALSE);
                                break;
                        }
                }
                if (Filter->players_active) {
                        switch (Filter->players_operator) {
                        case 0:
                                if (rom->num_players >=
                                    Filter->players_value)
                                        return (FALSE);
                                break;
                        case 1:
                                if (rom->num_players <=
                                    Filter->players_value)
                                        return (FALSE);
                                break;
                        case 2:
                                if (rom->num_players ==
                                    Filter->players_value)
                                        return (FALSE);
                                break;
                        case 3:
                                if (rom->num_players !=
                                    Filter->players_value)
                                        return (FALSE);
                                break;
                        }
                }
                if (Filter->controls_active) {
                        if (!rom->control || !Filter->controls_value)
                                return (FALSE);
                        if (Filter->not_controls_active &&
                            !strcmp(rom->control, Filter->controls_value))
                                return (FALSE);
                        if (!Filter->not_controls_active &&
                            strcmp(rom->control, Filter->controls_value))
                                return (FALSE);
                }
                if (Filter->manufacturer_active) {
                        if (!rom->manu || !Filter->manufacturer_value)
                                return (FALSE);
                        if (Filter->not_manufacturer_active &&
                            !strcmp(rom->manu, Filter->manufacturer_value))
                                return (FALSE);
                        if (!Filter->not_manufacturer_active &&
                            strcmp(rom->manu, Filter->manufacturer_value))
                                return (FALSE);
                }
                if (Filter->driver_active) {
                        if (!rom->driver || !Filter->driver_value)
                                return (FALSE);
                        if (Filter->not_driver_active &&
                            !strcmp(rom->driver, Filter->driver_value))
                                return (FALSE);
                        if (!Filter->not_driver_active &&
                            strcmp(rom->driver, Filter->driver_value))
                                return (FALSE);
                }
                if (Filter->category_active) {
                        if (!rom->category || !Filter->category_value)
                                return (FALSE);
                        if (Filter->not_category_active &&
                            !strcmp(rom->category, Filter->category_value))
                                return (FALSE);
                        if (!Filter->not_category_active &&
                            strcmp(rom->category, Filter->category_value))
                                return (FALSE);
                }

                if (Filter->cabinet_active) {
                        foundimage = FALSE;
                        if (find_image
                            (general_prefs.cabinet_dir, rom->romname,
                             rom->cloneof, &tmp)) {
                                g_free(tmp);
                                foundimage = TRUE;
                        }
                        if ((foundimage && Filter->not_cabinet_active) ||
                            (!foundimage && !Filter->not_cabinet_active))
                                return FALSE;
                }
                if (Filter->flyer_active) {
                        foundimage = FALSE;
                        if (find_image
                            (general_prefs.flyer_dir, rom->romname,
                             rom->cloneof, &tmp)) {
                                g_free(tmp);
                                foundimage = TRUE;
                        }
                        if ((foundimage && Filter->not_flyer_active) ||
                            (!foundimage && !Filter->not_flyer_active))
                                return FALSE;
                }
                if (Filter->screenshot_active) {
                        foundimage = FALSE;
                        if (find_image
                            (general_prefs.screenshot_dir, rom->romname,
                             rom->cloneof, &tmp)) {
                                g_free(tmp);
                                foundimage = TRUE;
                        }
                        if ((foundimage && Filter->not_screenshot_active)
                            || (!foundimage
                                && !Filter->not_screenshot_active))
                                return FALSE;
                }
        }

        return (TRUE);
}

gchar **make_gamelist_entry(const RomEntry * in, GList *columns)
{
        gchar **out = NULL, *tmpstring = NULL;
        GList *active_list, *romentry_list = NULL;
        gint numcols;

        numcols = g_list_length(columns);

        g_return_val_if_fail(in != NULL, NULL);
        g_return_val_if_fail(numcols > 0, NULL);

        active_list = g_list_first(columns);
        while (active_list != NULL) {
                switch (GPOINTER_TO_INT(active_list->data)) {
                case FAVORITE:
                        tmpstring = g_malloc0(MAX_FAVORITE);
                        in->favourite ? g_snprintf(tmpstring, MAX_FAVORITE,
                                                   _("Yes")) :
                            g_snprintf(tmpstring, MAX_FAVORITE, _("No"));
                        break;
                case GAMENAME:
                        tmpstring = g_strndup(in->gamename, MAX_GAMENAME);
                        break;
                case ROMNAME:
                        tmpstring = g_strndup(in->romname, MAX_ROMNAME);
                        break;
                case MANU:
                        tmpstring = g_strndup(in->manu, MAX_MANU);
                        break;
                case YEAR:
                        tmpstring = g_strndup(in->year, MAX_YEAR);
                        break;
                case CATEGORY:
                        tmpstring = g_strndup(in->category, MAX_CATEGORY);
                        break;
                case STATUS:
                        tmpstring = g_malloc0(MAX_STATUS);
                        switch (in->status) {
                        case CORRECT:
                                g_snprintf(tmpstring, MAX_STATUS,
                                           _("Correct"));
                                break;
                        case INCORRECT:
                                g_snprintf(tmpstring, MAX_STATUS,
                                           _("Incorrect"));
                                break;
                        case MISSING:
                                g_snprintf(tmpstring, MAX_STATUS,
                                           _("Missing"));
                                break;
                        case MAY_WORK:
                                g_snprintf(tmpstring, MAX_STATUS,
                                           _("May work"));
                                break;
                        }
                        break;
                case CLONE:
                        tmpstring = g_strndup(in->cloneof, MAX_ROMNAME);
                        break;
                case ROMOF:
                        tmpstring = g_strndup(in->romof, MAX_ROMNAME);
                        break;
                case NUMPLAYERS:
                        tmpstring = g_malloc0(MAX_NUMPLAYERS);
                        g_snprintf(tmpstring, MAX_NUMPLAYERS, "%i",
                                   in->num_players);
                        break;
                case CONTROL:
                        tmpstring = g_strndup(in->control, MAX_CONTROL);
                        break;
                case NUMBUTTONS:
                        tmpstring = g_malloc0(MAX_NUMBUTTONS);
                        g_snprintf(tmpstring, MAX_NUMBUTTONS, "%i",
                                   in->num_buttons);
                        break;
                case MAMEVER:
                        tmpstring =
                            g_strndup(in->mame_ver_added, MAX_MAMEVER);
                        break;
                case DRIVER:
                        tmpstring = g_strndup(in->driver, MAX_ROMNAME);
                        break;
                case VECTOR:
                        tmpstring = g_malloc0(MAX_FAVORITE);
                        in->vector ? g_snprintf(tmpstring, MAX_FAVORITE,
                                                _("Yes")) :
                            g_snprintf(tmpstring, MAX_FAVORITE, _("No"));
                        break;
                case DRIVERSTATUS:
                        tmpstring = g_malloc0(MAX_DRIVERSTATUS);
                        in->working ? g_snprintf(tmpstring,
                                                 MAX_DRIVERSTATUS,
                                                 _("Good")) :
                            g_snprintf(tmpstring, MAX_DRIVERSTATUS,
                                       _("Preliminary"));
                        break;
                case CPU1:
                        tmpstring = g_strndup(in->cpu1, MAX_CPU);
                        break;
                case CPU2:
                        tmpstring = g_strndup(in->cpu2, MAX_CPU);
                        break;
                case CPU3:
                        tmpstring = g_strndup(in->cpu3, MAX_CPU);
                        break;
                case CPU4:
                        tmpstring = g_strndup(in->cpu4, MAX_CPU);
                        break;
                case SOUND1:
                        tmpstring = g_strndup(in->sound1, MAX_CPU);
                        break;
                case SOUND2:
                        tmpstring = g_strndup(in->sound2, MAX_CPU);
                        break;
                case SOUND3:
                        tmpstring = g_strndup(in->sound3, MAX_CPU);
                        break;
                case SOUND4:
                        tmpstring = g_strndup(in->sound4, MAX_CPU);
                        break;
                case TIMESPLAYED:
                        if (in->timesplayed > 0) {
                                tmpstring = g_malloc0(MAX_TIMESPLAYED);
                                g_snprintf(tmpstring, MAX_TIMESPLAYED, "%i",
                                           in->timesplayed);
                        }
                        else{
                                tmpstring = g_strndup(_("Never played"), MAX_TIMESPLAYED);
                        }
                        break;
                }
                romentry_list = g_list_prepend(romentry_list, tmpstring);
                active_list = g_list_next(active_list);
        }
        /* 
         * This is the way to convert a list to an array. The code is taken
         * from glibs g_strsplit
         */

        out = g_new(gchar *, numcols--);
        for (romentry_list = g_list_first(romentry_list);
             (romentry_list != NULL);
             romentry_list = g_list_next(romentry_list)) {
                out[numcols--] = romentry_list->data;
        }
//        g_list_free (romentry_list);

        return out;
}

void lookup_pop()
{
        GameLookUp *tmp;

        tmp = &romlookup;

        if (lookup_filo != NULL) {
                tmp = lookup_filo->data;
                g_snprintf(romlookup.search_string, MAX_SEARCH_LENGTH,
                           "%s", tmp->search_string);
                romlookup.pos = tmp->pos;
                g_free(tmp->search_string);
                if (lookup_filo->next)
                        lookup_filo = g_list_next(lookup_filo);
                else
                        lookup_filo = NULL;
        } else {
                romlookup.pos = general_prefs.current_row;
                romlookup.search_string[0] = '\0';
        }
}

void lookup_free()
{
        GameLookUp *tmp;

        while ((lookup_filo != NULL)) {
                tmp = lookup_filo->data;
                g_free(tmp);
                lookup_filo = g_list_next(lookup_filo);
        }

        g_list_free(lookup_filo);
        lookup_filo = NULL;

        romlookup.pos = general_prefs.current_row;
        romlookup.search_string[0] = '\0';
}

/* Makes the commandline to start xmame. ** exec needs to be freed by the caller */
void makecmd_line(const gchar * game, gchar ** exec, RomEntry * romentry)
{
        gchar **screenshot_dirs = NULL;
        gchar *volume = g_malloc0(5);
        gchar *scale = g_malloc0(5);
	gchar *vectoropts = g_malloc0(80);
	gchar *beam = g_malloc0(25);
	gchar *flicker = g_malloc0(25);
        gchar *vectorres = g_malloc0(25);
        gchar *joytype = g_malloc0(5);
        gchar *fullscreen = g_malloc0(20);
        gchar *windowed = g_malloc0(20);
        gchar *winkeys = g_malloc0(20);
        gchar *nowinkeys = g_malloc0(20);
        gchar *grabmouse = g_malloc0(20);
        gchar *nograbmouse = g_malloc0(20);
        gchar *screenshotdir = g_malloc0(50);

        gchar *tmp = g_malloc0(11);

        /* Need to do these ugly hacks to keep it from returning 0.0 as the scale value
           * or 0.000 for the beam value when no games are in the list (i.e
           * when gRustibus is started for the first time. -scale 0.0 and -beam
           * 0.00 makes xmame complain and quit.
         */
        if (game_settings.scale != 0)
                g_snprintf(scale, 5, "%i", game_settings.scale);
        else
                g_snprintf(scale, 5, "1");

        if (game_settings.beam != 0)
                g_snprintf(beam, 25, "%f", game_settings.beam);
        else
                g_snprintf(beam, 25, "1.0");

        g_snprintf(flicker, 25, "%f", game_settings.flicker);

	// JAU
	if (romentry!=NULL && romentry->vector)
	{
		g_snprintf(vectoropts, 80, " -beam %s -flicker %s",
			beam, flicker);

		switch (game_settings.vectorres) {
		case 0:
			g_snprintf(vectorres, 25, " ");
			break;
		case 1:
			g_snprintf(vectorres, 25, " -vectorres 640x480");
			break;
		case 2:
			g_snprintf(vectorres, 25, " -vectorres 800x600");
			break;
		case 3:
			g_snprintf(vectorres, 25, " -vectorres 1024x768");
			break;
		case 4:
			g_snprintf(vectorres, 25, " -vectorres 1280x1024");
			break;
		case 5:
			g_snprintf(vectorres, 25, " -vectorres 1600x1200");
			break;
		}

	} else {
		g_snprintf(vectoropts, 80, " ");
		g_snprintf(vectorres, 25, " ");
	}

        if (general_prefs.screenshot_dir) {
                screenshot_dirs =
                    g_strsplit(general_prefs.screenshot_dir, ":", MAX_DIR);
                g_snprintf(screenshotdir, 50, "%s", screenshot_dirs[0]);
        } else {
                g_snprintf(screenshotdir, 50, "  ");
        }

        g_snprintf(volume, 5, "%i", game_settings.volume);
        g_snprintf(joytype, 5, "%i", game_settings.joytype);

        if (!strcmp(general_prefs.xmame_display_target, "x11")) {
                g_snprintf(fullscreen, 20, " -x11-mode 1");
                g_snprintf(windowed, 20, " -x11-mode 0");
                g_snprintf(winkeys, 20, " -winkeys");
                g_snprintf(nowinkeys, 20, " -nowinkeys");
                g_snprintf(grabmouse, 20, " -grabmouse");
                g_snprintf(nograbmouse, 20, " -nograbmouse");
        } else if (!strcmp(general_prefs.xmame_display_target, "xgl")) {
                /* Check for version. 0.37b4->uses -nocabview */
                /* FIXME: This is a very ugly hack. It will be replaced by a
                 * more generic aproach shortly */
                g_snprintf(tmp, 10, "%s", general_prefs.xmame_minor);

                if (!strcmp(tmp, "37 BETA 4")) {
                        g_snprintf(fullscreen, 20, " -nocabview");
                } else {
                        g_snprintf(fullscreen, 20, " -fullview");
                }
                if (!strcmp(tmp, "37 BETA 5")) {
                        g_snprintf(fullscreen, 20, " -nocabview");
                } else {
                        g_snprintf(fullscreen, 20, " -fullview");
                }
                g_snprintf(windowed, 20, " -cabview");
                g_snprintf(winkeys, 20, " -winkeys");
                g_snprintf(nowinkeys, 20, " -nowinkeys");
                g_snprintf(grabmouse, 20, " -grabmouse");
                g_snprintf(nograbmouse, 20, " -nograbmouse");
        } else if (!strcmp(general_prefs.xmame_display_target, "xfx")) {
                g_snprintf(fullscreen, 20, " ");
                g_snprintf(windowed, 20, " ");
                g_snprintf(winkeys, 20, " -winkeys");
                g_snprintf(nowinkeys, 20, " -nowinkeys");
                g_snprintf(grabmouse, 20, " -grabmouse");
                g_snprintf(nograbmouse, 20, " -nograbmouse");
        }
        else if (!strcmp(general_prefs.xmame_display_target, "SDL")) {
                g_snprintf(fullscreen, 20, " -fullscreen");
                g_snprintf(windowed, 20, " -nofullscreen");
                g_snprintf(winkeys, 20, " ");
                g_snprintf(nowinkeys, 20, " ");
                g_snprintf(grabmouse, 20, " ");
                g_snprintf(nograbmouse, 20, " ");
        } else if (!strcmp(general_prefs.xmame_display_target, "ggi")) {
                g_snprintf(fullscreen, 20, " ");
                g_snprintf(windowed, 20, " ");
                g_snprintf(winkeys, 20, " ");
                g_snprintf(nowinkeys, 20, " ");
                g_snprintf(grabmouse, 20, " ");
                g_snprintf(nograbmouse, 20, " ");
                if (game_settings.fullscreen && !getuid())
                        putenv("GGI_DISPLAY=DGA");
                else
                        putenv("GGI_DISPLAY=X");
        }

        *exec = g_strconcat("\"", general_prefs.xmame_exe, "\"",
                            " -rompath ", "\"", general_prefs.rom_dir,
                            "\"", " -cheatfile ", "\"",
                            general_prefs.cheat_file, "\"",
                            " -historyfile ", "\"",
                            general_prefs.game_history_file, "\"",
                            " -screenshotdir ", "\"", screenshotdir, "\"",
                            " -spooldir ", "\"",
                            general_prefs.highscore_dir, "\"",
                            game_settings.fullscreen ? fullscreen :
                            windowed,
                            game_settings.scanlines ? " -scanlines" :
                            " -noscanlines",
                            game_settings.extra_artwork ? " -artwork" :
                            " -noartwork",
                            game_settings.autoframeskip ? " -autoframeskip"
                            : " -noautoframeskip",
                            game_settings.auto_colordepth ? " -bpp 0" :
                            " ", game_settings.rot_left ? " -rol" : "",
                            game_settings.rot_right ? " -ror" : "",
                            game_settings.flipx ? " -flipx" : "",
                            game_settings.flipy ? " -flipy" : "",
                            " -scale ", scale,
                            game_settings.antialias ? " -antialias" :
                            " -noantialias",
                            game_settings.translucency ? " -translucency" :
                            " -notranslucency",
			    vectoropts,
			    vectorres,
                            game_settings.analog_joy ? " -analogstick" :
                            " -noanalogstick",
                            game_settings.mouse ? " -mouse" : " -nomouse",
                            game_settings.winkeys ? winkeys : nowinkeys,
                            game_settings.grab_mouse ? grabmouse :
                            nograbmouse, " -joytype ", joytype,
                            game_settings.sound ? " -sound" : " -nosound",
                            game_settings.samples ? " -samples" :
                            " -nosamples",
                            game_settings.fake_sound ? " -fakesound" : "",
                            " -volume ", volume, " ",
                            game_settings.cheat ? " -cheat " :
                            " -nocheat ",
                            game_settings.extra_options ? game_settings.
                            extra_options : " ", " ", game, NULL);

        g_free(volume);
        g_free(joytype);
        g_free(scale);
	g_free(vectoropts);
        g_free(beam);
        g_free(flicker);
        g_free(vectorres);
        g_free(fullscreen);
        g_free(windowed);
        g_free(winkeys);
        g_free(nowinkeys);
        g_free(grabmouse);
        g_free(nograbmouse);
        g_free(screenshotdir);
        g_strfreev(screenshot_dirs);

        g_free(tmp);

        grustibus_debug("Executing commandline: %s", *exec);
}

void download_selected_rom(RomEntry * romdata)
{
        gchar *romof;
        gchar *game;
        gchar *driver;
        gchar *url = g_malloc(100);
        gchar *msg = g_malloc(300);
        gchar **buttons;
        buttons = g_new (gchar*, 3);
        buttons[0]=GNOME_STOCK_BUTTON_YES;
        buttons[1]=GNOME_STOCK_BUTTON_NO;
        buttons[2]=NULL;

        if (romdata) {
                game = romdata->romname;
                romof = romdata->romof;
                driver = romdata->driver;
                if (strcmp(driver, game)) {
                        if (strcmp(driver, romof) && strcmp(romof, "-")
                            && strcmp(romof, game)) {
                                         
                                if (!grustibus_question(buttons, _("This game also needs files from the romsets %s and %s\nDo you want to download these as well?"), driver, romof)){
                                        download_file (romdata->romof, general_prefs.rom_dir, general_prefs.rom_url);
                                        download_file (romdata->driver, general_prefs.rom_dir, general_prefs.rom_url);
                                }
                        } else {
                                g_snprintf(msg, 300, _("This game also needs files from the romset %s\nDo you want to download this as well?"), driver);
                                if (!grustibus_question(buttons, msg)) {
                                        download_file (romdata->romof, general_prefs.rom_dir, general_prefs.rom_url);
                                }
                        }
                } else if (strcmp(romof, game) && strcmp(romof, "-")) {
                        g_snprintf(msg, 300, _("This game also needs files from the romset %s\nDo you want to download this as well?"), romof);
                        if (!grustibus_question(buttons, msg)) {
                                download_file (romdata->driver, general_prefs.rom_dir, general_prefs.rom_url);
                        }
                }

                download_file (romdata->romname, general_prefs.rom_dir, general_prefs.rom_url);
        }
        g_free(url);
        g_free(msg);
}


void select_random_game()
{
        gint random;
        random =
            (gint) ((gfloat) (visible_games) * rand() / (RAND_MAX + 1.0));
        select_game(random);
}

void quit()
{
        gchar **buttons;
        int answer;
        buttons = g_new (gchar*, 4);
        buttons[0]=_("Save and quit");
        buttons[1]=_("Just quit");
        buttons[2]=GNOME_STOCK_BUTTON_CANCEL;
        buttons[3]=NULL;
        
        if (need_to_save_prefs) {
                answer = grustibus_question(buttons,  
                                _("You have not saved your settings.\nDo you want to save them before quiting?"));
        } else answer = 1;
 
        switch (answer) {
        case (0):
                save_prefs();
                if (general_prefs.save_win_on_quit)
                        save_window_prefs();
                gtk_main_quit();
                break;
        case (1):
                if (general_prefs.save_win_on_quit)
                        save_window_prefs();
                gtk_main_quit();
                break;
        case (2):
                break;
        default:
                break;
        }
}

void grustibus_debug(const gchar * format, ...)
{
#ifdef DEBUG
        va_list args;
        va_start(args, format);
        g_logv(NULL, G_LOG_LEVEL_DEBUG, format, args);
        va_end(args);
#endif
}

gint grustibus_g_list_compare_func(gconstpointer ptr1, gconstpointer ptr2)
{
        RomEntry *rom1;
        RomEntry *rom2;

        rom1 = (RomEntry *) ptr1;
        rom2 = (RomEntry *) ptr2;

        return (strcmp(rom1->romname, rom2->romname));
}

gint lookup_column(Columns colnum)
{
        gint column;
        GList *tmplist;

        tmplist = g_list_first(general_prefs.active_columns);
        column = 0;
        while (GPOINTER_TO_INT(tmplist->data) != colnum) {
                column++;
                tmplist = g_list_next(tmplist);

        }
        return column;
}

void download_file (const gchar *romname, const gchar *directories, gchar *baseurl) {

        gchar *file, *url, *destfile, *dest = NULL, **dirs = NULL, **buttons, *in = NULL, *out=NULL;
        gint n, left;
        FILE *testfile;
        gboolean exists=FALSE;

        buttons = g_new (gchar*, 3);
        buttons[0] = GNOME_STOCK_BUTTON_YES;
        buttons[1] = GNOME_STOCK_BUTTON_NO;
        buttons[2] = NULL;

        url = g_malloc(MAX_URL);
        destfile = g_malloc(300);
        key_pressed = FALSE;

        in = baseurl;
        out = url;
        left = MAX_URL;

        while (*in != '\0'){
                if(*in=='%') {
                        switch(*(++in)) {
                                case '%':
                                        g_snprintf(out,left,"%%");
                                        break;
                                case 's':
                                        g_snprintf(out,left,"%s", romname);
                                        break;
                                case 'S':
                                        g_snprintf(out,left,"%s", romname);
                                        break;
                        }

                        left-=strlen(out);
                        out+=strlen(out);
                }
                else {
                        if(left>0) {
                                *out++=*in;
                                left--;
                        }
                }
                in++;
        }
        *out='\0';
        
        file = g_basename (url);
        
        if (directories != NULL)
                dirs = g_strsplit(directories, ":", MAX_DIR);
                
        if ((dirs == NULL) && (directories != NULL))
                dirs[0] = g_strdup(directories);

        /* Write permisions in the dir? */
        for (n = 0; dirs[n]; n++) {
                g_snprintf(destfile, 300, "%s/%s",dirs[n],file); 
                if ((testfile = fopen(destfile, "w")) != NULL) {
                        dest = g_malloc(250);
                        g_snprintf(dest, 250, "%s", dirs[n]);
                        fclose (testfile);
                        break;
                }
        }

        /* File exists? */
        for (n = 0; dirs[n]; n++) {
                g_snprintf(destfile, 300, "%s/%s",dirs[n],file); 
                if ((testfile = fopen(destfile, "r")) != NULL) {
                        exists=TRUE;
                        fclose (testfile);
                        break;
                }
        }

        if (!dest) {
                dest = g_malloc(250);
                g_snprintf(dest, 250, "%s", getenv("HOME"));
                grustibus_warning (_("Could not get write permissions in any of the specified directories.\nWill write file to %s"),dest);
        }
        
        if (!exists || !grustibus_question(buttons, _("This file is already installed\nDo you want to download it anyway?"))){
                gtm_add_url(url, dest, FALSE, FALSE);
        }

        g_free(dest);
        g_strfreev(dirs);
        
        g_free(url);
        g_free(destfile);

}

gboolean gtm_add_url(const gchar *url, gchar *dir, gboolean disable_proxy, gboolean disable_auto_dl)
{
    CORBA_Environment ev;
    GTM_Download gtm_download_client;
    gboolean result;

    result = FALSE;
    gtm_download_client = goad_server_activate_with_repo_id (NULL, "IDL:gtm/download:1.0", 0, NULL);

    if (gtm_download_client != NULL) {
        CORBA_exception_init (&ev);
        result = GTM_Download_add_url (gtm_download_client, url, dir,
                                       disable_proxy, disable_auto_dl, &ev);
        if (ev._major != CORBA_NO_EXCEPTION) {
            g_warning ("CORBA exception thrown: %s\n",
                     CORBA_exception_id (&ev));
        }
    }
    else {
         grustibus_warning (_("Unable to download using Gtm, not installed?"));
    }

    return result;
}

void game_played_once_more(RomEntry *romdata) {
        romdata->timesplayed++;        
        save_timesplayed();
}
        typedef struct _entry {
                gchar *name;
                gint num;
        } entry;

void get_entries_for_multiple_choice_filters () {
        GList *listpoint = NULL;
        GHashTable *controls, *manufacturers, *drivers, *categories;
        RomEntry *rom;
        
        entry *controlsentry, *manufacturersentry, *driversentry, *categoriesentry, *tmpentry;
       
        if (controls_combo_entries) g_list_free(controls_combo_entries);
        if (manufacturers_combo_entries) g_list_free(manufacturers_combo_entries);
        if (drivers_combo_entries) g_list_free(drivers_combo_entries);
        if (categories_combo_entries) g_list_free(categories_combo_entries);
        controls_combo_entries = NULL;
        manufacturers_combo_entries = NULL;
        drivers_combo_entries = NULL;
        categories_combo_entries = NULL;

        controls = g_hash_table_new(g_str_hash, g_str_equal);
        manufacturers = g_hash_table_new(g_str_hash, g_str_equal);
        drivers = g_hash_table_new(g_str_hash, g_str_equal);
        categories = g_hash_table_new(g_str_hash, g_str_equal);

        listpoint = g_list_first(roms);
        while ((listpoint != NULL)) {
                rom = (RomEntry *) listpoint->data;
                tmpentry = g_hash_table_lookup(controls, rom->control);
                if (tmpentry == NULL){
                                controlsentry = g_malloc (sizeof(entry)); 
                                controlsentry->name = rom->control;
                                controlsentry->num = 1;
                                g_hash_table_insert(controls, rom->control, controlsentry);
                }
                else {
                                tmpentry->num++;
                }
                tmpentry = g_hash_table_lookup(manufacturers, rom->manu);
                if (tmpentry == NULL){
                                manufacturersentry = g_malloc (sizeof(entry)); 
                                manufacturersentry->name = rom->manu;
                                manufacturersentry->num = 1;
                                g_hash_table_insert(manufacturers, rom->manu, manufacturersentry);
                }
                else {
                                tmpentry->num++;
                }
                tmpentry = g_hash_table_lookup(drivers, rom->driver);
                if (tmpentry == NULL){
                                driversentry = g_malloc (sizeof(entry)); 
                                driversentry->name = rom->driver;
                                driversentry->num = 1;
                                g_hash_table_insert(drivers, rom->driver, driversentry);
                }
                else {
                                tmpentry->num++;
                }
                tmpentry = g_hash_table_lookup(categories, rom->category);
                if (tmpentry == NULL){
                                categoriesentry= g_malloc (sizeof(entry)); 
                                categoriesentry->name = rom->category;
                                categoriesentry->num = 1;
                                g_hash_table_insert(categories, rom->category, categoriesentry);
                }
                else {
                                tmpentry->num++;
                }

                listpoint = g_list_next(listpoint);
        }
        g_hash_table_foreach (controls, (GHFunc)*make_combo_entries_glist_from_hashtable, &controls_combo_entries);
        g_hash_table_foreach (manufacturers, (GHFunc)*make_combo_entries_glist_from_hashtable, &manufacturers_combo_entries);
        g_hash_table_foreach (drivers, (GHFunc)*make_combo_entries_glist_from_hashtable, &drivers_combo_entries);
        g_hash_table_foreach (categories, (GHFunc)*make_combo_entries_glist_from_hashtable, &categories_combo_entries);

        g_hash_table_destroy (controls);
        g_hash_table_destroy (manufacturers);
        g_hash_table_destroy (drivers);
        g_hash_table_destroy (categories);
       
        controls_combo_entries =  g_list_sort (controls_combo_entries, (GCompareFunc) strcmp);
        manufacturers_combo_entries = g_list_sort (manufacturers_combo_entries, (GCompareFunc) strcmp);
        drivers_combo_entries = g_list_sort (drivers_combo_entries, (GCompareFunc) strcmp);
        categories_combo_entries =  g_list_sort (categories_combo_entries, (GCompareFunc) strcmp);
}

void make_combo_entries_glist_from_hashtable (gpointer key, gpointer value, GList **list) {
       entry *tmpentry;
       tmpentry = value;
       // List only entries that has more than 5 occurences.
       if (tmpentry->num > 5) {
               *list = g_list_append(*list, tmpentry->name);
       }
       g_free (value);
}
// vim: ai sw=8 ts=8
