/*
 * grustibus.h -- Function prototypes for grustibus.c, global includes 
 * and definitions and declaration of global variables. 
 *
 * gRustibus (c) 2000-2001 Kjetil Thuen <kjetilt@users.sourceforge.net
 * MAME frontend for the GNOME desktop environment.
 *
 * gRustibus is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * This code is indented with GNU indent using k&r style with 8 character
 * indents as suggested by the GNOME project. Please keep it that way.
 * ($ indent -kr -ts8 -i8 *.[ch])
 *
 * Last modified: 25 April 2001 <kjetilt@users.sourceforge.net>
 *
 */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gdk-pixbuf/gnome-canvas-pixbuf.h>
#include <gnome.h>
#include <libgnorba/gnorba.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/stat.h>
#include <unistd.h>
#include "Gtm.h"

/* Maximum length of some strings */
#define MAX_ROMNAME 10
#define MAX_GAMENAME 100
#define MAX_MANU 50
#define MAX_YEAR 5
#define MAX_STATUS 20
#define MAX_CPU 20
#define MAX_CONTROL 20
#define MAX_CATEGORY 40
#define MAX_NUMPLAYERS 3
#define MAX_NUMBUTTONS 3
#define MAX_MAMEVER 20
#define MAX_FAVORITE 10
#define MAX_DRIVERSTATUS 20
#define MAX_TIMESPLAYED 20

#define MAX_URL 200
#define MAX_FONTNAME 100
#define MAX_DIR 500
#define FLAGS 6
#define LABEL_LENGTH 20
#define MAX_SEARCH_LENGTH 30
#define SELECT_TIMEOUT 100
#define COLUMN_COUNT 25
#define MAX_COLUMN_NAME 30
#define DEFAULT_FAVOURITES "720\n\
aliens\n\
altbeast\n\
apb\n\
atetris\n\
baddudes\n\
blktiger\n\
bombjack\n\
cabal\n\
captcomm\n\
ccastles\n\
csprint\n\
dowild\n\
ffight\n\
forgottn\n\
gauntlet\n\
goldnaxe\n\
marble\n\
mk\n\
mk2\n\
neodrift\n\
p47\n\
paperboy\n\
pow\n\
rainbow\n\
salamand\n\
sectionz\n\
sf2\n\
shangon\n\
shdancer\n\
shinobi\n\
silkworm\n\
slyspy\n\
snowbros\n\
spyhunt\n\
ssprint\n\
ssrdrebd\n\
strider\n\
tehkanwc\n\
tmht2p\n\
tnzsb\n\
toobin\n\
travrusa\n\
vindictr\n\
vulcan\n\
wb3\n\
wbml\n\
wc90"

/* structs */

typedef struct _GameFilter {
        gboolean correct;
        gboolean incorrect;
        gboolean missing;
        gboolean may_work;
        gboolean clones;
        gboolean favourites;
        gboolean not_favourites;
        
        gboolean advanced_filters_active;

        gboolean driver_status_active;
        gboolean video_type_active;
        gboolean year_active;
        gboolean players_active;
        gboolean controls_active;
        gboolean manufacturer_active;
        gboolean driver_active;
        gboolean category_active;
        gboolean screenshot_active;
        gboolean flyer_active;
        gboolean cabinet_active;

        gint players_operator;
        gint year_operator;
        gboolean not_controls_active;
        gboolean not_manufacturer_active;
        gboolean not_driver_active;
        gboolean not_category_active;
        gboolean not_screenshot_active;
        gboolean not_flyer_active;
        gboolean not_cabinet_active;

        gboolean driver_status_good;
        gboolean video_type_vector;
        gint year_value;
        gint players_value;
        gchar *controls_value;
        gchar *manufacturer_value;
        gchar *driver_value;
        gchar *category_value;
} GameFilter;



struct Prefs {
        gchar *xmame_exe;
        gchar *rom_dir;
        gchar *screenshot_dir;
        gchar *highscore_dir;
        gchar *flyer_dir;
        gchar *cabinet_dir;
        gchar *game_history_file;
        gchar *cheat_file;
        gchar *playback_dir;

        gchar *xmame_display_target;
        gint xmame_major;
        gchar *xmame_minor;
        gint xmame_release;
        gint romdir_time;

        gchar *font;
        gint correct_color_r;
        gint correct_color_g;
        gint correct_color_b;
        gint incorrect_color_r;
        gint incorrect_color_g;
        gint incorrect_color_b;
        gint missing_color_r;
        gint missing_color_g;
        gint missing_color_b;
        gint favourite_color_r;
        gint favourite_color_g;
        gint favourite_color_b;
        gint may_work_color_r;
        gint may_work_color_g;
        gint may_work_color_b;

        gboolean favourites_overrides;

        gint sort_by;
        GtkSortType sort_order;
        gint current_row;
        gint current_tab;
        gint pane_position;
        gint notebook_page;
        gint x_size;
        gint y_size;
        gchar *column_size_string;

        gboolean save_win_on_quit;

        gboolean playback_no_highscore;
        gboolean iconify_while_xmame;
        gboolean loadsa_image_formats;
        gboolean look_for_clone_images;
        gboolean indicate_notebook_changes;

        GameFilter ListFilter;
        GList *active_columns;

        gchar *rom_url;
        gchar *screenshot_url;
        gchar *flyer_url;
        gchar *cabinet_url;
};


struct GameSettings {
        gboolean default_options;

        gboolean fullscreen;
        gboolean scanlines;
        gboolean extra_artwork;
        gboolean autoframeskip;
        gboolean auto_colordepth;
        gboolean rot_left;
        gboolean rot_right;
        gboolean flipx;
        gboolean flipy;
        gint scale;

        gboolean antialias;
        gboolean translucency;
        gfloat beam;
        gfloat flicker;
        gint vectorres;

        gboolean analog_joy;
        gboolean mouse;
        gboolean winkeys;
        gboolean grab_mouse;
        gint joytype;

        gboolean sound;
        gboolean samples;
        gboolean fake_sound;
        gint volume;

        gboolean cheat;
        gchar *extra_options;
};

typedef enum _RomStatus {
        CORRECT,
        INCORRECT,
        MISSING,
        MAY_WORK
} RomStatus;

typedef enum _Columns {
        FAVORITE,
        GAMENAME,
        ROMNAME,
        MANU,
        YEAR,
        CATEGORY,
        STATUS,
        CLONE,
        ROMOF,
        NUMPLAYERS,
        CONTROL,
        NUMBUTTONS,
        MAMEVER,
        DRIVER,
        VECTOR,
        DRIVERSTATUS,
        CPU1,
        CPU2,
        CPU3,
        CPU4,
        SOUND1,
        SOUND2,
        SOUND3,
        SOUND4,
        TIMESPLAYED
} Columns;

typedef struct _RomEntry {
        gchar romname[MAX_ROMNAME];
        gchar gamename[MAX_GAMENAME];
        gchar year[MAX_YEAR];
        gchar manu[MAX_MANU];
        gchar cloneof[MAX_ROMNAME];
        gchar romof[MAX_ROMNAME];
        gchar driver[MAX_ROMNAME];
        gchar cpu1[MAX_CPU];
        gchar cpu2[MAX_CPU];
        gchar cpu3[MAX_CPU];
        gchar cpu4[MAX_CPU];
        gchar sound1[MAX_CPU];
        gchar sound2[MAX_CPU];
        gchar sound3[MAX_CPU];
        gchar sound4[MAX_CPU];
        gchar control[MAX_CONTROL];
        gchar category[MAX_CATEGORY];
        gchar mame_ver_added[MAX_MAMEVER];
        gint num_players;
        gint num_buttons;

        RomStatus status;
        gboolean working;
        gboolean vector;
        gboolean favourite;

        gint timesplayed;
} RomEntry;

typedef struct _GameLookUp {
        gchar search_string[MAX_SEARCH_LENGTH];
        gint pos;
} GameLookUp;

typedef struct _ColumnLabel {
        Columns number;
        gchar *label;
} ColumnLabel;

struct Prefs general_prefs;
struct GameSettings global_settings;
struct GameSettings game_settings;

/* global variables */
GList *roms;
GList *all_columns;
GList *lookup_filo;
GList *controls_combo_entries, *manufacturers_combo_entries, *drivers_combo_entries, *categories_combo_entries;
gboolean need_to_rebuild_gamelist;
gboolean need_to_rebuild_gamelist_widget;
gboolean need_to_make_new_gamelist;
gboolean need_to_save_prefs;
gboolean xmame_version_ok;
gint visible_games;
gint supported_games;
GameLookUp romlookup;
gchar *columntitles[COLUMN_COUNT];

/* function headers*/
void startup_main(void);
void set_game(const gchar * game);
void game_settings_set_to_default(void);
gint compare_roms(const void *a, const void *b);
gboolean filter_rom(const RomEntry * rom, const GameFilter * Filter);
gchar **make_gamelist_entry(const RomEntry * in, GList *columns);
void lookup_pop(void);
void lookup_free(void);
void makecmd_line(const gchar * game, gchar ** exec, RomEntry * romentry);
void download_selected_rom(RomEntry * romdata);
void quit(void);
void select_random_game(void);
void grustibus_debug(const gchar * format, ...);
gint grustibus_g_list_compare_func(gconstpointer ptr1, gconstpointer ptr2);
gint lookup_column(Columns colnum);
void download_file (const gchar *romname, const gchar *directories, gchar *baseurl);
gboolean gtm_add_url(const gchar *url, gchar *dir, gboolean disable_proxy, gboolean disable_auto_dl);
void game_played_once_more(RomEntry *romdata);
void get_entries_for_multiple_choice_filters(void);
void make_combo_entries_glist_from_hashtable (gpointer key, gpointer value, GList **list);
// vim: ai sw=8 ts=8
