/*
 * gui.c -- Contains the code to set up and control the gui (minus 
 * the callbacks). 
 * 
 * gRustibus (c) 2000-2001 Kjetil Thuen <kjetilt@users.sourceforge.net>
 * MAME frontend for the GNOME desktop environment.
 *
 * gRustibus is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * This code is indented with GNU indent using k&r style with 8 character
 * indents as suggested by the GNOME project. Please keep it that way.
 * ($ indent -kr -ts8 -i8 *.[ch])
 *
 * Last modified: 28 April 2001 <kjetilt@users.sourceforge.net>
 *
 */

#include "grustibus.h"
#include "callbacks.h"
#include "mameio.h"
#include "fileio.h"
#include "interface.h"
#include "gui.h"
#include "grustibus-pixmaps.h"

GdkCursor *busy_cursor;

void startup_gui(GtkWidget * widget)
{
        gint n;
        GtkStyle *style;
        GtkCheckMenuItem *menuitem;
        GtkWidget *tmp_box;
        GtkWidget *tmp_pixmap;
        GtkWidget *tmp_widget;
        gchar *imagefile;
        gchar *tmp_string = NULL;


        /* Create the prefs window */
        prefs_window = create_prefs_window();
        prefs_startup(prefs_window);

        /* Create the advanced filters dialog */
        filter_dialog = create_filter_dialog();
        filter_startup(filter_dialog);

        /* Create_the fileselection dialog for record/playback game */
        fileselection = create_filedialog();
        gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileselection),
                                        general_prefs.playback_dir);

        /* Create the busy cursor */
        busy_cursor = gdk_cursor_new(GDK_WATCH);

        /* Add a pixmap to the save button. */
        tmp_box =
            GTK_WIDGET(gtk_object_get_data
                       (GTK_OBJECT(main_window), "save_settings_hbox"));
        tmp_pixmap =
            gnome_stock_pixmap_widget_new(main_window,
                                          GNOME_STOCK_PIXMAP_SAVE);
        gtk_box_pack_start(GTK_BOX(tmp_box), tmp_pixmap, FALSE, FALSE, 0);
        gtk_widget_show(tmp_pixmap);


        /* Populate the appwin struct */
        appwin.scrolled =
            GTK_WIDGET(lookup_widget
                       (main_window, "main_romlist_scrolledwindow"));
        create_column_widgets();
        create_gamelist_widget(&appwin.list);
        appwin.pane = GTK_PANED(lookup_widget(main_window, "main_hpaned"));
        appwin.notebook =
            GTK_NOTEBOOK(lookup_widget(main_window, "main_notebook"));
        appwin.msgbar =
            GNOME_APPBAR(lookup_widget(main_window, "main_appbar"));
        appwin.favourite_button =
            GTK_TOGGLE_BUTTON(lookup_widget
                              (main_window, "favourite_button"));
        appwin.vector_frame =
            GTK_WIDGET(lookup_widget(main_window, "vector_frame"));
        appwin.display_vbox =
            GTK_WIDGET(lookup_widget(main_window, "display_frame"));


       /* The game info GtkText widget needs wordwraping */
        tmp_widget = GTK_WIDGET(gtk_object_get_data
                                (GTK_OBJECT(main_window),
                                 "game_history_text"));

        gtk_text_set_word_wrap(GTK_TEXT(tmp_widget), TRUE);
        gtk_text_set_line_wrap(GTK_TEXT(tmp_widget), TRUE);

        /* Set the toglemenus */
        menuitem =
            GTK_CHECK_MENU_ITEM(gtk_object_get_data
                                (GTK_OBJECT(main_window),
                                 "save_window_state_automatically_on_quit1"));
        menuitem->active = general_prefs.save_win_on_quit ? 1 : 0;
        menuitem =
            GTK_CHECK_MENU_ITEM(gtk_object_get_data
                                (GTK_OBJECT(main_window),
                                 "show_correct_roms"));
        menuitem->active = general_prefs.ListFilter.correct ? 1 : 0;
        menuitem =
            GTK_CHECK_MENU_ITEM(gtk_object_get_data
                                (GTK_OBJECT(main_window),
                                 "show_incorect_roms"));
        menuitem->active = general_prefs.ListFilter.incorrect ? 1 : 0;
        menuitem =
            GTK_CHECK_MENU_ITEM(gtk_object_get_data
                                (GTK_OBJECT(main_window),
                                 "show_missing_roms"));
        menuitem->active = general_prefs.ListFilter.missing ? 1 : 0;
        menuitem =
            GTK_CHECK_MENU_ITEM(gtk_object_get_data
                                (GTK_OBJECT(main_window),
                                 "show_maywork_roms"));
        menuitem->active = general_prefs.ListFilter.may_work ? 1 : 0;
        menuitem =
            GTK_CHECK_MENU_ITEM(gtk_object_get_data
                                (GTK_OBJECT(main_window), "show_clones"));
        menuitem->active = general_prefs.ListFilter.clones ? 1 : 0;
        menuitem =
            GTK_CHECK_MENU_ITEM(gtk_object_get_data
                                (GTK_OBJECT(main_window),
                                 "show_favourites"));
        menuitem->active = general_prefs.ListFilter.favourites ? 1 : 0;

        menuitem =
            GTK_CHECK_MENU_ITEM(gtk_object_get_data
                                (GTK_OBJECT(main_window),
                                 "advanced_filters"));
        menuitem->active =
            general_prefs.ListFilter.advanced_filters_active ? 1 : 0;


        /* And the tabs radiomenu */
        switch (general_prefs.notebook_page) {
        case (SCREENSHOT_PAGE):
                menuitem =
                    GTK_CHECK_MENU_ITEM(gtk_object_get_data
                                        (GTK_OBJECT(main_window),
                                         "screenshot_menuitem"));
                break;
        case (SETTINGS_PAGE):
                menuitem =
                    GTK_CHECK_MENU_ITEM(gtk_object_get_data
                                        (GTK_OBJECT(main_window),
                                         "game_settings_menuitem"));
                break;
        case (INFO_PAGE):
                menuitem =
                    GTK_CHECK_MENU_ITEM(gtk_object_get_data
                                        (GTK_OBJECT(main_window),
                                         "game_info_menuitem"));
                break;
        case (FLYER_PAGE):
                menuitem =
                    GTK_CHECK_MENU_ITEM(gtk_object_get_data
                                        (GTK_OBJECT(main_window),
                                         "flyer_menuitem"));
                break;
        case (CABINET_PAGE):
                menuitem =
                    GTK_CHECK_MENU_ITEM(gtk_object_get_data
                                        (GTK_OBJECT(main_window),
                                         "cabinet_photo_menuitem"));
                break;
        }
        gtk_menu_item_activate(GTK_MENU_ITEM(menuitem));


        /* Add callbacks to the optionmenus */
        appwin.joy_option =
            GTK_OPTION_MENU(gtk_object_get_data
                            (GTK_OBJECT(main_window),
                             "joytype_optionmenu"));

        gtk_signal_connect(GTK_OBJECT
                           (GTK_OPTION_MENU(appwin.joy_option)->menu),
                           "deactivate",
                           GTK_SIGNAL_FUNC(on_joy_option_selected), NULL);

        appwin.vectorres_option =
            GTK_OPTION_MENU(gtk_object_get_data
                            (GTK_OBJECT(main_window),
                             "vector_resolution_optionmenu"));

        gtk_signal_connect(GTK_OBJECT
                           (GTK_OPTION_MENU
                            (appwin.vectorres_option)->menu), "deactivate",
                           GTK_SIGNAL_FUNC(on_vectorres_option_selected),
                           NULL);

        /* Prepare the image struct for screenshots, flyers and cabinets */
        screenshot_canvas.name = g_strdup("default");
        screenshot_canvas.canvas =
            GTK_WIDGET(gtk_object_get_data
                       (GTK_OBJECT(main_window), "screenshot_pixmap"));
        screenshot_canvas.image_item =
            gnome_canvas_item_new(gnome_canvas_root
                                  (GNOME_CANVAS(screenshot_canvas.canvas)),
                                  gnome_canvas_pixbuf_get_type(), NULL);
        imagefile = gnome_pixmap_file("/grustibus/defaultsnap.png");
        if (imagefile) {
                screenshot_canvas.imagestruct.default_image =
                    gdk_pixbuf_new_from_file(imagefile);
                g_free(imagefile);
        }

        /* If the defaultsnap.png file can't be found, use the icon instead */
        else {
                grustibus_debug("Default screenshot not found");
                screenshot_canvas.imagestruct.default_image =
                    gdk_pixbuf_new_from_xpm_data(grustibus_icon_xpm);
        }
        screenshot_canvas.imagestruct.image =
            screenshot_canvas.imagestruct.default_image;
        gdk_pixbuf_ref(screenshot_canvas.imagestruct.image);

        /* Flyer */
        flyer_canvas.name = g_strdup("default");
        flyer_canvas.canvas =
            GTK_WIDGET(gtk_object_get_data
                       (GTK_OBJECT(main_window), "flyer_pixmap"));
        flyer_canvas.image_item =
            gnome_canvas_item_new(gnome_canvas_root
                                  (GNOME_CANVAS(flyer_canvas.canvas)),
                                  gnome_canvas_pixbuf_get_type(), NULL);
        flyer_canvas.imagestruct.default_image =
            screenshot_canvas.imagestruct.default_image;
        gdk_pixbuf_ref(screenshot_canvas.imagestruct.default_image);
        flyer_canvas.imagestruct.image =
            flyer_canvas.imagestruct.default_image;
        gdk_pixbuf_ref(flyer_canvas.imagestruct.image);

        /* Cabinet photos */
        cabinet_canvas.name = g_strdup("default");
        cabinet_canvas.canvas =
            GTK_WIDGET(gtk_object_get_data
                       (GTK_OBJECT(main_window), "cabinet_pixmap"));
        cabinet_canvas.image_item =
            gnome_canvas_item_new(gnome_canvas_root
                                  (GNOME_CANVAS(cabinet_canvas.canvas)),
                                  gnome_canvas_pixbuf_get_type(), NULL);
        cabinet_canvas.imagestruct.default_image =
            screenshot_canvas.imagestruct.default_image;
        gdk_pixbuf_ref(screenshot_canvas.imagestruct.default_image);
        cabinet_canvas.imagestruct.image =
            cabinet_canvas.imagestruct.default_image;
        gdk_pixbuf_ref(cabinet_canvas.imagestruct.image);

        /* Set the size of the window and the state of some widgets */
        gtk_widget_set_name(widget, "main window");
        gtk_window_set_default_size(GTK_WINDOW(widget),
                                    general_prefs.x_size,
                                    general_prefs.y_size);
        gtk_paned_set_position(appwin.pane, general_prefs.pane_position);
        gtk_notebook_set_page(appwin.notebook,
                              general_prefs.notebook_page);
        gtk_signal_handler_block_by_func(GTK_OBJECT(appwin.notebook),
                                         GTK_SIGNAL_FUNC
                                         (on_main_notebook_size_allocate),
                                         NULL);


        /* CList stuff */

        /* Check if the font exists */
        if (!gdk_font_load(general_prefs.font)) {
                g_free(general_prefs.font);
                general_prefs.font = g_malloc0(MAX_FONTNAME);
                g_snprintf(general_prefs.font, MAX_FONTNAME, "fixed");
                grustibus_warning
                    ("Could not find the font. Using \"fixed\"");
        }

        gtk_clist_set_compare_func(appwin.list,
                                   grustibus_clist_compare_func);

        rebuild_game_list();




        /* Prepare the notebook page status leds */
        style = gtk_widget_get_style(GTK_WIDGET(appwin.notebook));

        appwin.active_led_pixmap = gdk_pixmap_colormap_create_from_xpm_d(
                        NULL, gdk_colormap_new(gdk_visual_get_best(), FALSE),
                        &appwin.active_led_mask, &style->bg[GTK_STATE_NORMAL],
                        (gchar **) led_on_xpm);

        appwin.inactive_led_pixmap = gdk_pixmap_colormap_create_from_xpm_d(
                        NULL, gdk_colormap_new(gdk_visual_get_best(), FALSE),
                        &appwin.inactive_led_mask, &style->bg[GTK_STATE_NORMAL],
                        (gchar **) led_off_xpm);
 
        for (n = 0; n < LAST_PAGE; n++) {
                tmp_string = g_malloc0(30);
                appwin.notebook_leds[n] =
                    GTK_PIXMAP(gtk_pixmap_new
                               (appwin.inactive_led_pixmap,
                                appwin.inactive_led_mask));
                gtk_widget_ref(GTK_WIDGET(appwin.notebook_leds[n]));
                g_snprintf(tmp_string, 30, "notebook_tab_%i_pixmap", n);
                gtk_object_set_data_full(GTK_OBJECT(main_window),
                                         tmp_string,
                                         appwin.notebook_leds[n],
                                         (GtkDestroyNotify)
                                         gtk_widget_unref);
                gtk_widget_show(GTK_WIDGET(appwin.notebook_leds[n]));
                g_snprintf(tmp_string, 30, "notebook_tab_%i_hbox", n);
                gtk_box_pack_start(GTK_BOX
                                   (gtk_object_get_data
                                    (GTK_OBJECT(main_window), tmp_string)),
                                   GTK_WIDGET(appwin.notebook_leds[n]),
                                   TRUE, TRUE, 0);

                g_free(tmp_string);
        }
        gtk_widget_show(GTK_WIDGET(appwin.notebook));
        if (general_prefs.indicate_notebook_changes) notebooktab_status_show_all_but(general_prefs.notebook_page);
        else notebooktab_status_hide_all(); 

        /* Actually show all the stuff*/
        gtk_widget_show(main_window);
        
        gtk_clist_moveto(appwin.list, general_prefs.current_row,
                         lookup_column(general_prefs.sort_by), 0.5, 1.0);
        
        gtk_signal_handler_unblock_by_func(GTK_OBJECT(appwin.notebook),
                                           GTK_SIGNAL_FUNC
                                           (on_main_notebook_size_allocate),
                                           NULL);

        set_icon(main_window, "gRustibus");



        return;
}

void set_icon(GtkWidget * widget, const gchar * name)
{
        GtkStyle *style;
        GdkBitmap *icon_mask = NULL;
        GdkPixmap *icon_pm = NULL;

        style = gtk_widget_get_style(widget);

        icon_pm = gdk_pixmap_create_from_xpm_d(widget->window,
                                               &icon_mask,
                                               &style->bg
                                               [GTK_STATE_NORMAL],
                                               (gchar **)
                                               grustibus_icon_xpm);

        gdk_window_set_icon(widget->window, NULL, icon_pm, icon_mask);
        gdk_window_set_icon_name(widget->window, name);

        /* These /should/ be free'd, right? */
        g_free(icon_pm);
        g_free(icon_mask);
}



void filter_startup(GtkWidget * widget)
{
        GtkWidget *tmpwidget;
        gtk_window_set_title(GTK_WINDOW(widget),
                             _("gRustibus: Configure advanced filters"));

        tmpwidget = GTK_WIDGET(lookup_widget (filter_dialog, "filter_category_combo"));
        if (categories_combo_entries) gtk_combo_set_popdown_strings (GTK_COMBO(tmpwidget), categories_combo_entries); 

        tmpwidget = GTK_WIDGET(lookup_widget (filter_dialog, "filter_driver_combo"));
        if (drivers_combo_entries) gtk_combo_set_popdown_strings (GTK_COMBO(tmpwidget), drivers_combo_entries); 
        
        tmpwidget = GTK_WIDGET(lookup_widget (filter_dialog, "filter_manufacturer_combo"));
        if (manufacturers_combo_entries) gtk_combo_set_popdown_strings (GTK_COMBO(tmpwidget), manufacturers_combo_entries); 

        tmpwidget = GTK_WIDGET(lookup_widget (filter_dialog, "filter_controls_combo"));
        if (controls_combo_entries) gtk_combo_set_popdown_strings (GTK_COMBO(tmpwidget), controls_combo_entries); 

        filter_dialog_reset();
}

void prefs_startup(GtkWidget * widget)
{
        /* Populate the prefswin struct */
        prefswin.playback_no_highscore_togglebutton =
            gtk_object_get_data(GTK_OBJECT(prefs_window),
                                "playback_no_highscore_togglebutton");
        prefswin.iconify_while_mame_togglebutton =
            gtk_object_get_data(GTK_OBJECT(prefs_window),
                                "iconify_while_mame_togglebutton");
        prefswin.loadsa_image_formats_togglebutton =
            gtk_object_get_data(GTK_OBJECT(prefs_window),
                                "loadsa_image_formats_checkbutton");
        prefswin.look_for_clone_images_togglebutton =
            gtk_object_get_data(GTK_OBJECT(prefs_window),
                                "look_for_clone_images_checkbutton");
        prefswin.indicate_notebook_changes_togglebutton =
            gtk_object_get_data(GTK_OBJECT(prefs_window),
                                "notebook_tab_indicator_checkbutton");

        prefswin.fullscreen_togglebutton = gtk_object_get_data
            (GTK_OBJECT(prefs_window), "prefs_fullscreen_checkbutton");
        prefswin.scanlines_togglebutton = gtk_object_get_data
            (GTK_OBJECT(prefs_window), "prefs_scanlines_checkbutton");
        prefswin.x_artwork_togglebutton = gtk_object_get_data
            (GTK_OBJECT(prefs_window), "prefs_x_artwork_checkbutton");
        prefswin.autoframeskip_togglebutton = gtk_object_get_data
            (GTK_OBJECT(prefs_window), "prefs_autoframeskip_checkbutton");
        prefswin.auto_colordepth_togglebutton = gtk_object_get_data
            (GTK_OBJECT(prefs_window), "prefs_colordepth_checkbutton");
        prefswin.rotatel_togglebutton = gtk_object_get_data
            (GTK_OBJECT(prefs_window), "prefs_rotatel_checkbutton");
        prefswin.rotater_togglebutton = gtk_object_get_data
            (GTK_OBJECT(prefs_window), "prefs_rotater_checkbutton");
        prefswin.flipx_togglebutton = gtk_object_get_data
            (GTK_OBJECT(prefs_window), "prefs_flipx_checkbutton");
        prefswin.flipy_togglebutton = gtk_object_get_data
            (GTK_OBJECT(prefs_window), "prefs_flipy_checkbutton");
        prefswin.scale_hscale = gtk_object_get_data(GTK_OBJECT
                                                    (prefs_window),
                                                    "prefs_scale_hscale");


        prefswin.translucency_togglebutton = gtk_object_get_data
            (GTK_OBJECT(prefs_window),
             "settings_display_vector_translucency_checkbutton");
        prefswin.antialias_togglebutton = gtk_object_get_data
            (GTK_OBJECT(prefs_window),
             "settings_display_vector_antialias_checkbutton");
        prefswin.flicker_hscale = gtk_object_get_data
            (GTK_OBJECT(prefs_window),
             "settings_display_vector_flicker_hscale");
        prefswin.beam_hscale = gtk_object_get_data
            (GTK_OBJECT(prefs_window),
             "settings_display_vector_beam_hscale");
        prefswin.vectorres_option = gtk_object_get_data
            (GTK_OBJECT(prefs_window),
             "settings_display_vector_resolution_optionmenu");


        prefswin.sound_togglebutton = gtk_object_get_data
            (GTK_OBJECT(prefs_window), "prefs_sound_checkbutton");
        prefswin.samples_togglebutton = gtk_object_get_data
            (GTK_OBJECT(prefs_window), "prefs_samples_checkbutton");
        prefswin.fakesound_togglebutton = gtk_object_get_data
            (GTK_OBJECT(prefs_window), "prefs_fakesound_checkbutton");
        prefswin.volume_hscale = gtk_object_get_data(GTK_OBJECT
                                                     (prefs_window),
                                                     "prefs_volume_hscale");


        prefswin.analog_joy_togglebutton = gtk_object_get_data
            (GTK_OBJECT(prefs_window), "prefs_analog_joy_checkbutton");
        prefswin.mouse_togglebutton = gtk_object_get_data
            (GTK_OBJECT(prefs_window), "prefs_mouse_checkbutton");
        prefswin.winkeys_togglebutton = gtk_object_get_data
            (GTK_OBJECT(prefs_window), "prefs_winkeys_checkbutton");
        prefswin.grab_mouse_togglebutton = gtk_object_get_data
            (GTK_OBJECT(prefs_window), "prefs_grab_mouse_checkbutton");
        prefswin.joy_option = gtk_object_get_data
            (GTK_OBJECT(prefs_window), "prefs_joytype_optionmenu");


        prefswin.cheat_togglebutton = gtk_object_get_data
            (GTK_OBJECT(prefs_window), "prefs_cheat_checkbutton");
        prefswin.extra_options = GTK_ENTRY
            (gnome_entry_gtk_entry
             (gtk_object_get_data
              (GTK_OBJECT(prefs_window), "prefs_extra_options_entry")));


        prefswin.favourite_colorpicker = gtk_object_get_data
            (GTK_OBJECT(prefs_window), "favourite_colorpicker");
        prefswin.correct_colorpicker = gtk_object_get_data
            (GTK_OBJECT(prefs_window), "corect_colorpicker");
        prefswin.incorrect_colorpicker = gtk_object_get_data
            (GTK_OBJECT(prefs_window), "incorect_colorpicker5");
        prefswin.missing_colorpicker = gtk_object_get_data
            (GTK_OBJECT(prefs_window), "missing_colorpicker");
        prefswin.may_work_colorpicker = gtk_object_get_data
            (GTK_OBJECT(prefs_window), "may_work_colorpicker");
        prefswin.fontpicker = gtk_object_get_data
            (GTK_OBJECT(prefs_window), "listprefs_listfont_fontpicker");
        prefswin.fontentry = GTK_ENTRY
            (gtk_object_get_data
             (GTK_OBJECT(prefs_window), "listprefs_listfont_font_entry"));
        prefswin.favourite_overrides_togglebutton =
            gtk_object_get_data
            (GTK_OBJECT(prefs_window),
             "listprefs_favouritecolor_checkbutton");
        prefswin.example_list =
            gtk_object_get_data
            (GTK_OBJECT(prefs_window), "listprefs_example_clist");

        prefswin.avail_col_list = gtk_object_get_data
            (GTK_OBJECT(prefs_window), "available_columns_list");
        prefswin.select_col_list = gtk_object_get_data
            (GTK_OBJECT(prefs_window), "used_columns_list");

        prefswin.xmame_exe_entry = GTK_ENTRY
            (gnome_file_entry_gtk_entry
             (gtk_object_get_data
              (GTK_OBJECT(prefs_window), "xmame_exe_entry")));
        prefswin.rom_path_entry = GTK_ENTRY
            (gnome_file_entry_gtk_entry
             (gtk_object_get_data
              (GTK_OBJECT(prefs_window), "rompath_entry")));
        prefswin.screenshots_dir_entry = GTK_ENTRY
            (gnome_file_entry_gtk_entry
             (gtk_object_get_data
              (GTK_OBJECT(prefs_window), "screenshot_path_fileentry")));
        prefswin.highscore_dir_entry = GTK_ENTRY
            (gnome_file_entry_gtk_entry
             (gtk_object_get_data
              (GTK_OBJECT(prefs_window), "spooldir_fileentry")));
        prefswin.flyers_dir_entry = GTK_ENTRY
            (gnome_file_entry_gtk_entry
             (gtk_object_get_data
              (GTK_OBJECT(prefs_window), "flyers_fileentry")));
        prefswin.cabinets_dir_entry = GTK_ENTRY
            (gnome_file_entry_gtk_entry
             (gtk_object_get_data
              (GTK_OBJECT(prefs_window), "cabinet_fileentry")));
        prefswin.gamehistory_path_entry = GTK_ENTRY
            (gnome_file_entry_gtk_entry
             (gtk_object_get_data
              (GTK_OBJECT(prefs_window), "game_history_fileentry")));
        prefswin.cheatfile_path_entry = GTK_ENTRY
            (gnome_file_entry_gtk_entry
             (gtk_object_get_data
              (GTK_OBJECT(prefs_window), "cheatfile_fileentry")));
        prefswin.playback_dir_entry = GTK_ENTRY
            (gnome_file_entry_gtk_entry
             (gtk_object_get_data
              (GTK_OBJECT(prefs_window), "playback_entry")));


        prefswin.rom_url_entry = GTK_ENTRY
            (gnome_entry_gtk_entry
             (gtk_object_get_data
              (GTK_OBJECT(prefs_window), "rom_url_entry")));
        
        prefswin.screenshot_url_entry = GTK_ENTRY
            (gnome_entry_gtk_entry
             (gtk_object_get_data
              (GTK_OBJECT(prefs_window), "screenshot_url_entry")));
        
        prefswin.flyer_url_entry = GTK_ENTRY
            (gnome_entry_gtk_entry
             (gtk_object_get_data
              (GTK_OBJECT(prefs_window), "flyer_url_entry")));

        prefswin.cabinet_url_entry = GTK_ENTRY
            (gnome_entry_gtk_entry
             (gtk_object_get_data
              (GTK_OBJECT(prefs_window), "cabinet_url_entry")));
            
        /* Add callbacks to the optionmenus */
        gtk_signal_connect(GTK_OBJECT
                           (GTK_OPTION_MENU(prefswin.joy_option)->menu),
                           "deactivate",
                           GTK_SIGNAL_FUNC(on_prefs_option_selected),
                           NULL);

        gtk_signal_connect(GTK_OBJECT
                           (GTK_OPTION_MENU
                            (prefswin.vectorres_option)->menu),
                           "deactivate",
                           GTK_SIGNAL_FUNC(on_prefs_option_selected),
                           NULL);

        prefs_reset(widget);
        return;
}

void filter_dialog_reset()
{
        GtkWidget *tmpwidget;

        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_driver_status_checkbutton");
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget),
                                     general_prefs.
                                     ListFilter.driver_status_active);
        if (general_prefs.ListFilter.driver_status_good) {
                tmpwidget =
                    gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                        "filter_status_good_radiobutton");
        } else {
                tmpwidget =
                    gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                        "filter_status_preliminary_radiobutton");
        }
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), TRUE);

        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_video_type_checkbutton");
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget),
                                     general_prefs.
                                     ListFilter.video_type_active);
        if (general_prefs.ListFilter.video_type_vector) {
                tmpwidget =
                    gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                        "filter_video_type_vector_radiobutton");
        } else {
                tmpwidget =
                    gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                        "filter_video_type_raster_radiobutton");
        }
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), TRUE);

        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_year_checkbutton");
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget),
                                     general_prefs.ListFilter.year_active);
        switch (general_prefs.ListFilter.year_operator) {
        case 0:
                tmpwidget =
                    gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                        "filter_year_lt_radiobutton");
                break;
        case 1:
                tmpwidget =
                    gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                        "filter_year_gt_radiobutton");
                break;
        case 2:
                tmpwidget =
                    gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                        "filter_year_ne_radiobutton");
                break;
        case 3:
                tmpwidget =
                    gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                        "filter_year_e_radiobutton");
                break;
        }
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), TRUE);
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_year_spinbutton");
        gtk_spin_button_set_value(GTK_SPIN_BUTTON(tmpwidget),
                                  general_prefs.ListFilter.year_value);

        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_players_checkbutton");
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget),
                                     general_prefs.
                                     ListFilter.players_active);
        switch (general_prefs.ListFilter.players_operator) {
        case 0:
                tmpwidget =
                    gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                        "filter_players_lt_radiobutton");
                break;
        case 1:
                tmpwidget =
                    gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                        "filter_players_gt_radiobutton");
                break;
        case 2:
                tmpwidget =
                    gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                        "filter_players_ne_radiobutton");
                break;
        case 3:
                tmpwidget =
                    gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                        "filter_players_e_radiobutton");
                break;
        }
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), TRUE);
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_players_spinbutton");
        gtk_spin_button_set_value(GTK_SPIN_BUTTON(tmpwidget),
                                  general_prefs.ListFilter.players_value);

        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_controls_checkbutton");
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget),
                                     general_prefs.
                                     ListFilter.controls_active);
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_controls_not_checkbutton");
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget),
                                     general_prefs.
                                     ListFilter.not_controls_active);
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_controls_combo_entry");
        gtk_entry_set_text(GTK_ENTRY(tmpwidget),
                           general_prefs.ListFilter.controls_value);

        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_manufacturer_checkbutton");
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget),
                                     general_prefs.
                                     ListFilter.manufacturer_active);
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_manufacturer_not_checkbutton");
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget),
                                     general_prefs.
                                     ListFilter.not_manufacturer_active);
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_manufacturer_combo_entry");
        gtk_entry_set_text(GTK_ENTRY(tmpwidget),
                           general_prefs.ListFilter.manufacturer_value);

        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_driver_checkbutton");
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget),
                                     general_prefs.
                                     ListFilter.driver_active);
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_driver_not_checkbutton");
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget),
                                     general_prefs.
                                     ListFilter.not_driver_active);
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_driver_combo_entry");
        gtk_entry_set_text(GTK_ENTRY(tmpwidget),
                           general_prefs.ListFilter.driver_value);

        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_category_checkbutton");
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget),
                                     general_prefs.
                                     ListFilter.category_active);
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_category_not_checkbutton");
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget),
                                     general_prefs.
                                     ListFilter.not_category_active);
        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_category_combo_entry");
        gtk_entry_set_text(GTK_ENTRY(tmpwidget),
                           general_prefs.ListFilter.category_value);

        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_screenshot_checkbutton");
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget),
                                     general_prefs.
                                     ListFilter.screenshot_active);
        if (general_prefs.ListFilter.not_screenshot_active) {
                tmpwidget =
                    gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                        "filter_screenshot_not_installed_radiobutton");
        } else {
                tmpwidget =
                    gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                        "filter_screenshot_installed_radiobutton");
        }
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), TRUE);

        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_flyer_checkbutton");
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget),
                                     general_prefs.
                                     ListFilter.flyer_active);
        if (general_prefs.ListFilter.not_flyer_active) {
                tmpwidget =
                    gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                        "filter_flyer_not_installed_radiobutton");
        } else {
                tmpwidget =
                    gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                        "filter_flyer_installed_radiobutton");
        }
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), TRUE);

        tmpwidget =
            gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                "filter_cabinet_checkbutton");
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget),
                                     general_prefs.
                                     ListFilter.cabinet_active);
        if (general_prefs.ListFilter.not_cabinet_active) {
                tmpwidget =
                    gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                        "filter_cabinet_not_installed_radiobutton");
        } else {
                tmpwidget =
                    gtk_object_get_data(GTK_OBJECT(filter_dialog),
                                        "filter_cabinet_installed_radiobutton");
        }
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmpwidget), TRUE);

}

void prefs_reset(GtkWidget * widget)
{
        set_prefs_widgets(widget);
        rebuild_prefs_list(widget);
        populate_column_lists(general_prefs.active_columns, prefswin.select_col_list, prefswin.avail_col_list);
        gnome_property_box_set_modified(GNOME_PROPERTY_BOX(widget), FALSE);
        need_to_make_new_gamelist = FALSE;
        need_to_rebuild_gamelist = FALSE;
        need_to_rebuild_gamelist_widget = FALSE;
}

void rom_selected(GtkCList * list, gint row)
{
        gchar *game = NULL;
        gchar *cloneof = NULL;
        RomEntry *romdata;
        gchar *image = NULL;
        gint current_page;

        general_prefs.current_row = row;

        current_page = gtk_notebook_get_current_page(appwin.notebook);

        romdata =
            gtk_clist_get_row_data(appwin.list, general_prefs.current_row);
        if (romdata != NULL) {
                game = romdata->romname;
                cloneof = romdata->cloneof;
                set_game(game);

                if (appwin.favourite_button->active != romdata->favourite) {
                        gtk_toggle_button_set_active
                            (appwin.favourite_button, romdata->favourite);
                }
        }
        if (general_prefs.indicate_notebook_changes) {
                set_notebooktab_status(SCREENSHOT_PAGE,
                                       find_image
                                       (general_prefs.screenshot_dir, game,
                                        cloneof, &image));
                if (current_page == SCREENSHOT_PAGE)
                        draw_image(&screenshot_canvas, game, image);

                set_notebooktab_status(SETTINGS_PAGE,
                                       !game_settings.default_options);
                if (current_page == SETTINGS_PAGE)
                        set_game_settings_widgets();

                set_notebooktab_status(INFO_PAGE, set_game_history(game));
                if (current_page == INFO_PAGE)
                        set_game_info_widgets();

                set_notebooktab_status(FLYER_PAGE,
                                       find_image(general_prefs.flyer_dir,
                                                  game, cloneof, &image));
                if (current_page == FLYER_PAGE)
                        draw_image(&flyer_canvas, game, image);

                set_notebooktab_status(CABINET_PAGE,
                                       find_image
                                       (general_prefs.cabinet_dir, game,
                                        cloneof, &image));
                if (current_page == CABINET_PAGE)
                        draw_image(&cabinet_canvas, game, image);
        } else {
                switch (gtk_notebook_get_current_page(appwin.notebook)) {
                case (SCREENSHOT_PAGE):
                        find_image(general_prefs.screenshot_dir, game,
                                   cloneof, &image);
                        draw_image(&screenshot_canvas, game, image);
                        break;
                case (SETTINGS_PAGE):
                        set_game_settings_widgets();
                        break;
                case (INFO_PAGE):
                        set_game_history(game);
                        set_game_info_widgets();
                        break;
                case (FLYER_PAGE):
                        find_image(general_prefs.flyer_dir, game, cloneof,
                                   &image);
                        draw_image(&flyer_canvas, game, image);
                        break;
                case (CABINET_PAGE):
                        find_image(general_prefs.cabinet_dir, game,
                                   cloneof, &image);
                        draw_image(&cabinet_canvas, game, image);
                        break;
                }
        }
        if (image != NULL)
                g_free(image);
}

void build_new_game_list() {
        gchar *msg = g_malloc(50);
        
        g_snprintf(msg, 50, _("Building gamelist, be patient..."));
        
        
        if (appwin.msgbar){
                lookup_free();
                gnome_appbar_pop(appwin.msgbar);
                gnome_appbar_push(appwin.msgbar, msg);
                gtk_progress_set_show_text(GTK_PROGRESS (gnome_appbar_get_progress(appwin.msgbar)), TRUE);
                gtk_progress_set_format_string(GTK_PROGRESS (gnome_appbar_get_progress (appwin.msgbar)), "%p%%");
                
                while (gtk_events_pending()) gtk_main_iteration();
        
                gtk_clist_clear(appwin.list);
                visible_games = 0;
                
                set_busy_cursor(TRUE);
        }
        else {
                progresswindow=create_gRustibus_progress_window();
                progresswindow_progressbar = GTK_WIDGET(lookup_widget (progresswindow, "progress_window_progressbar"));
                gtk_progress_set_show_text(GTK_PROGRESS (progresswindow_progressbar), TRUE);
                gtk_progress_set_format_string(GTK_PROGRESS (progresswindow_progressbar), "%p%%");
                gtk_widget_show (progresswindow);
                while (gtk_events_pending()) gtk_main_iteration();
        }
        create_list();
        if (appwin.msgbar){
                set_busy_cursor(FALSE);

                gnome_appbar_pop(appwin.msgbar);
                gtk_progress_set_show_text(GTK_PROGRESS(gnome_appbar_get_progress (appwin.msgbar)), FALSE);
                while (gtk_events_pending()) gtk_main_iteration();
                rebuild_game_list();
        }
        else if (progresswindow){
                progresswindow_progressbar=NULL; 
                gtk_widget_destroy(progresswindow);
        }
        g_free(msg);
}

void grustibus_set_progress (gfloat percent) { 
        if (appwin.msgbar) {
                gnome_appbar_set_progress(appwin.msgbar, percent);
                while (gtk_events_pending()) gtk_main_iteration();
        }
        else if (progresswindow){
                gtk_progress_bar_update (GTK_PROGRESS_BAR(progresswindow_progressbar), percent);
                while (gtk_events_pending()) gtk_main_iteration();
        }
}


void rebuild_game_list()
{
        GtkCList *list;
        gint rowbak = general_prefs.current_row;

        supported_games = 0;
        visible_games = 0;

        list = GTK_CLIST(appwin.list);

        lookup_free();

        build_clist(list);

        gtk_clist_set_sort_type(list, general_prefs.sort_order);
        gtk_clist_set_sort_column(list,
                                  lookup_column(general_prefs.sort_by));
        gtk_clist_sort(list);

        /* TODO: Make the selecting of the new row a little smarter.
         * It should keep the current game selected even if the gamelist has
         * changed its length. Check if this is the first build, then remember
         * the romname instead of the row.
         * Use get_row_from_data if possible.
         */
        select_game(rowbak);

        grustibus_appbar_set_default();
}

void set_sort_by_column (GtkCList *clist, gint column) 
{
        gpointer oldrow;
        gint newrownum, col_number;
        GList *tmplist, *column_list;
        CListColumnLabel *tmplabel;

        lookup_free();
        gnome_appbar_pop(appwin.msgbar);

        if (visible_games > 0) {      
                oldrow =
                    gtk_clist_get_row_data(clist,
                                           general_prefs.current_row);
                gtk_clist_freeze(clist);

                column_list =
                    g_list_nth(CListColumnLabels, general_prefs.sort_by);
                tmplabel = column_list->data;
                gtk_widget_hide(GTK_WIDGET(tmplabel->arrow));

                tmplist = g_list_nth(general_prefs.active_columns, column);
                col_number = GPOINTER_TO_INT(tmplist->data);
                column_list = g_list_nth(CListColumnLabels, col_number);
                tmplabel = column_list->data;

                if (general_prefs.sort_by == col_number) {
                        gtk_clist_set_sort_type(clist,
                                                !general_prefs.sort_order);
                        general_prefs.sort_order =
                            !general_prefs.sort_order;

                        if (general_prefs.sort_order) {
                                gtk_arrow_set(tmplabel->arrow,
                                              GTK_ARROW_UP, GTK_SHADOW_IN);

                        } else {
                                gtk_arrow_set(tmplabel->arrow,
                                              GTK_ARROW_DOWN,
                                              GTK_SHADOW_IN);
                        }
                } else {
                        general_prefs.sort_by = col_number;
                        gtk_clist_set_sort_type(clist, GTK_SORT_ASCENDING);
                        general_prefs.sort_order = GTK_SORT_ASCENDING;

                        gtk_arrow_set(tmplabel->arrow,
                                      GTK_ARROW_DOWN, GTK_SHADOW_IN);
                }
                gtk_widget_show(GTK_WIDGET(tmplabel->arrow));

                gtk_clist_set_sort_column(clist, column);
                gtk_clist_sort(clist);

                /* Keep the same rom selected and visible */
                newrownum = gtk_clist_find_row_from_data(clist, oldrow);
                select_game(newrownum);
                general_prefs.current_row = newrownum;
                gtk_clist_thaw(clist);
        }
}

void select_game(gint pos)
{
        GtkCList *list;

        list = GTK_CLIST(appwin.list);

        pos = MAX(0, MIN(pos, visible_games - 1));
        if (visible_games > 0) {
                gtk_clist_select_row(list, pos, general_prefs.sort_order);
                gtk_clist_moveto(list, pos,
                                 lookup_column(general_prefs.sort_by), 0.5,
                                 1.0);
                general_prefs.current_row = pos;
        }
}

void grustibus_appbar_set_default()
{
        gchar *default_message = g_malloc(100);

        gnome_appbar_pop(appwin.msgbar);
        if (xmame_version_ok) {
                g_snprintf(default_message, 100,
                           _
                           ("xmame (%s) version %i.%s:  %i games supported, %i games shown"),
                           general_prefs.xmame_display_target,
                           general_prefs.xmame_major,
                           general_prefs.xmame_minor, supported_games,
                           visible_games);
        } else {
                if (!general_prefs.xmame_minor) {
                        g_snprintf(default_message, 100,
                                   _
                                   ("Wrong xmame version!! %i games shown"),
                                   visible_games);
                } else {
                        g_snprintf(default_message, 100,
                                   _
                                   ("Too old xmame version!! %i games shown"),
                                   visible_games);
                }
        }

        gnome_appbar_set_default(appwin.msgbar, default_message);
        g_free(default_message);

}

void populate_column_lists(GList * selected_columns, GtkCList *sel_list, GtkCList *avail_list)
{
        GList *tmp_select = NULL, *tmp_all = NULL;
        ColumnLabel *tmpdata = NULL;
        gboolean column_used = FALSE; 
        gchar **label = NULL;
        gint row;

        gtk_clist_clear(avail_list);
        gtk_clist_clear(sel_list);

        label = g_new(gchar *, 1);

        /* Populate the selected list */
        tmp_select = g_list_first(selected_columns);
        while (tmp_select != NULL) {
                tmp_all = g_list_first(all_columns);
                while (tmp_all != NULL) {
                        tmpdata = tmp_all->data;
                        if (tmpdata->number ==
                            GPOINTER_TO_INT(tmp_select->data)) {
                                label[0] = tmpdata->label;
                                row = gtk_clist_append(sel_list,
                                                       label);
                                gtk_clist_set_row_data(sel_list,
                                                       row,
                                                       tmp_select->data);
                                tmp_all = g_list_last(tmp_all);
                        }
                        tmp_all = g_list_next(tmp_all);
                }
                tmp_select = g_list_next(tmp_select);
        }

        /* Populate the available list */
        tmp_all = g_list_first(all_columns);
        while (tmp_all != NULL) {
                tmp_select = g_list_first(selected_columns);
                tmpdata = tmp_all->data;
                while (tmp_select != NULL) {
                        if (tmpdata->number == GPOINTER_TO_INT(tmp_select->data)) {
                                column_used = TRUE;
                                tmp_select = g_list_last(tmp_select);
                        }
                        tmp_select = g_list_next(tmp_select);
                }

                if (!column_used) {
                        label[0] = tmpdata->label;
                        row = gtk_clist_append(avail_list,
                                               label);
                        gtk_clist_set_row_data(avail_list,
                                               row,
                                               GINT_TO_POINTER (tmpdata->number));
                        tmp_select = g_list_last(tmp_select);
                }
                else {
                        column_used=FALSE;
                }
                tmp_all = g_list_next(tmp_all);
        }
}

void rebuild_prefs_list(GtkWidget * prefs_window)
{
        GdkColor correctcol, incorrectcol, may_workcol, missingcol,
            favouritecol;
        GdkColormap *map;
        GtkStyle *liststyle;
        char *rom[2];
        gushort r, g, b, a;
        gchar *font_picker_font_name;

        font_picker_font_name =
            gtk_editable_get_chars(GTK_EDITABLE(prefswin.fontentry), 0,
                                   MAX_FONTNAME);
        if (!font_picker_font_name)
                font_picker_font_name = g_strdup("fixed");
        if (!gdk_font_load(font_picker_font_name)) {
                g_free(font_picker_font_name);
                grustibus_warning(_
                                  ("Could not open font. Reverting to last saved value."));
                font_picker_font_name = g_malloc0(MAX_FONTNAME);
                g_snprintf(font_picker_font_name, MAX_FONTNAME, "%s",
                           general_prefs.font);
                gtk_entry_set_text(prefswin.fontentry,
                                   font_picker_font_name);
        }
//      while (liststyle->ref_count > 0) {
//              gtk_style_unref(liststyle);
//        }
        liststyle =
            gtk_style_copy(gtk_widget_get_style
                           (GTK_WIDGET(prefswin.example_list)));

        if (liststyle == NULL) {
                gtk_style_unref(liststyle);
                liststyle = gtk_style_new();
        }
        liststyle->font = gdk_font_load(font_picker_font_name);
        gtk_widget_set_style(GTK_WIDGET(prefswin.example_list), liststyle);

        gnome_color_picker_get_i16(prefswin.correct_colorpicker, &r, &g,
                                   &b, &a);
        correctcol.red = r;
        correctcol.green = g;
        correctcol.blue = b;

        gnome_color_picker_get_i16(prefswin.incorrect_colorpicker, &r, &g,
                                   &b, &a);
        incorrectcol.red = r;
        incorrectcol.green = g;
        incorrectcol.blue = b;

        gnome_color_picker_get_i16(prefswin.missing_colorpicker, &r, &g,
                                   &b, &a);
        may_workcol.red = r;
        may_workcol.green = g;
        may_workcol.blue = b;

        gnome_color_picker_get_i16(prefswin.may_work_colorpicker, &r, &g,
                                   &b, &a);
        missingcol.red = r;
        missingcol.green = g;
        missingcol.blue = b;

        if (prefswin.favourite_overrides_togglebutton->active) {
                gnome_color_picker_get_i16(prefswin.favourite_colorpicker,
                                           &r, &g, &b, &a);
                favouritecol.red = r;
                favouritecol.green = g;
                favouritecol.blue = b;
        } else {
                favouritecol.red = correctcol.red;
                favouritecol.green = correctcol.green;
                favouritecol.blue = correctcol.blue;
        }

        if (main_window->window != NULL)
                map = gdk_window_get_colormap(main_window->window);
        else
                map = gdk_colormap_new (gdk_visual_get_best(), TRUE);

        gdk_colormap_alloc_color(map, &correctcol, TRUE, TRUE);
        gdk_colormap_alloc_color(map, &incorrectcol, TRUE, TRUE);
        gdk_colormap_alloc_color(map, &may_workcol, TRUE, TRUE);
        gdk_colormap_alloc_color(map, &missingcol, TRUE, TRUE);
        gdk_colormap_alloc_color(map, &favouritecol, TRUE, TRUE);

        gtk_clist_freeze(prefswin.example_list);
        gtk_clist_clear(prefswin.example_list);

        rom[0] = g_malloc(MAX_GAMENAME);
        rom[1] = g_malloc(MAX_STATUS);

        /* Just some stuff to put in the list */
        g_snprintf(rom[0], MAX_GAMENAME, "Pac Man");
        g_snprintf(rom[1], MAX_STATUS, _("Correct"));
        gtk_clist_append(GTK_CLIST(prefswin.example_list), rom);
        gtk_clist_set_foreground(prefswin.example_list, 0, &correctcol);

        g_snprintf(rom[0], MAX_GAMENAME, "Super Sprint");
        g_snprintf(rom[1], MAX_STATUS, _("Incorrect"));
        gtk_clist_append(GTK_CLIST(prefswin.example_list), rom);
        gtk_clist_set_foreground(prefswin.example_list, 1, &incorrectcol);

        g_snprintf(rom[0], MAX_GAMENAME, "Gauntlet");
        g_snprintf(rom[1], MAX_STATUS, _("Missing"));
        gtk_clist_append(GTK_CLIST(prefswin.example_list), rom);
        gtk_clist_set_foreground(prefswin.example_list, 2, &missingcol);

        g_snprintf(rom[0], MAX_GAMENAME, "Bomb Jack");
        g_snprintf(rom[1], MAX_STATUS, _("May work"));
        gtk_clist_append(GTK_CLIST(prefswin.example_list), rom);
        gtk_clist_set_foreground(prefswin.example_list, 3, &may_workcol);

        g_snprintf(rom[0], MAX_GAMENAME, "Ghosts n' Goblins");
        g_snprintf(rom[1], MAX_STATUS, _("Favourite"));
        gtk_clist_append(GTK_CLIST(prefswin.example_list), rom);
        gtk_clist_set_foreground(prefswin.example_list, 4, &favouritecol);

        gtk_clist_thaw(prefswin.example_list);

        g_free(rom[0]);
        g_free(rom[1]);
        g_free(font_picker_font_name);
}

/* 
 * Builds a gtkclist *list from the roms GList.
 */
void build_clist(GtkCList * list)
{
        gint sup, pos, n, favcolumn = -1;
        GdkColor correctcol, incorrectcol, may_workcol, missingcol,
            favouritecol;
        GdkColormap *map;
        GtkStyle *liststyle;
        gchar **rom;
        RomEntry *tmprom;
        GList *listpointer;
        GdkBitmap *favorite_mask = NULL;
        GdkPixmap *favorite_pm = NULL;

        pos = 0;

        liststyle = gtk_style_copy(gtk_widget_get_style(GTK_WIDGET(list)));

        if (liststyle == NULL) {
                gtk_style_unref(liststyle);
                liststyle = gtk_style_new();
        }
        liststyle->font = gdk_font_load(general_prefs.font);

        gtk_widget_set_style(GTK_WIDGET(list), liststyle);


        correctcol.red = 256 * general_prefs.correct_color_r;
        correctcol.green = 256 * general_prefs.correct_color_g;
        correctcol.blue = 256 * general_prefs.correct_color_b;

        incorrectcol.red = 256 * general_prefs.incorrect_color_r;
        incorrectcol.green = 256 * general_prefs.incorrect_color_g;
        incorrectcol.blue = 256 * general_prefs.incorrect_color_b;

        may_workcol.red = 256 * general_prefs.may_work_color_r;
        may_workcol.green = 256 * general_prefs.may_work_color_g;
        may_workcol.blue = 256 * general_prefs.may_work_color_b;

        missingcol.red = 256 * general_prefs.missing_color_r;
        missingcol.green = 256 * general_prefs.missing_color_g;
        missingcol.blue = 256 * general_prefs.missing_color_b;

        favouritecol.red = 256 * general_prefs.favourite_color_r;
        favouritecol.green = 256 * general_prefs.favourite_color_g;
        favouritecol.blue = 256 * general_prefs.favourite_color_b;

        if (main_window->window != NULL)
                map = gdk_window_get_colormap(main_window->window);
        else
                map = gdk_colormap_new (gdk_visual_get_best(), TRUE);

        gdk_colormap_alloc_color(map, &correctcol, TRUE, TRUE);
        gdk_colormap_alloc_color(map, &incorrectcol, TRUE, TRUE);
        gdk_colormap_alloc_color(map, &may_workcol, TRUE, TRUE);
        gdk_colormap_alloc_color(map, &missingcol, TRUE, TRUE);
        gdk_colormap_alloc_color(map, &favouritecol, TRUE, TRUE);

        if (main_window->window != NULL ) 
                favorite_pm = gdk_pixmap_create_from_xpm_d(main_window->window,
                                                           &favorite_mask,
                                                           &liststyle->bg
                                                           [GTK_STATE_NORMAL],
                                                           (gchar **)
                                                           favorite_xpm);
        else {
                favorite_pm = gdk_pixmap_colormap_create_from_xpm_d(NULL,
                                                           gdk_colormap_new(gdk_visual_get_best(), FALSE),
                                                           &favorite_mask,
                                                           &liststyle->bg
                                                           [GTK_STATE_NORMAL],
                                                           (gchar **)
                                                           favorite_xpm);

        }

        gtk_clist_freeze(list);
        gtk_clist_clear(list);

        sup = 0;

        listpointer = g_list_first(general_prefs.active_columns);
        n = 0;
        while (listpointer) {
                if (GPOINTER_TO_INT(listpointer->data == FAVORITE))
                        favcolumn = n;
                n++;
                listpointer = g_list_next(listpointer);
        }
        listpointer = g_list_first(roms);

        while ((listpointer != NULL)) {
                tmprom = (RomEntry *) listpointer->data;
                if (filter_rom(tmprom, &general_prefs.ListFilter)) {

                        rom = make_gamelist_entry(tmprom, general_prefs.active_columns);

                        gtk_clist_append(GTK_CLIST(list), (gchar **) rom);

                        gtk_clist_set_row_data(list, pos, tmprom);

                        switch (tmprom->status) {
                        case CORRECT:
                                gtk_clist_set_foreground(list, pos,
                                                         &correctcol);
                                break;
                        case INCORRECT:
                                gtk_clist_set_foreground(list, pos,
                                                         &incorrectcol);
                                break;
                        case MAY_WORK:
                                gtk_clist_set_foreground(list, pos,
                                                         &may_workcol);
                                break;
                        case MISSING:
                                gtk_clist_set_foreground(list, pos,
                                                         &missingcol);
                                break;
                        }

                        if (tmprom->favourite
                            && general_prefs.favourites_overrides)
                                gtk_clist_set_foreground(list, pos,
                                                         &favouritecol);

                        if (favcolumn > -1) {
                                if (tmprom->favourite) {
                                        gtk_clist_set_pixmap(list, pos,
                                                             favcolumn,
                                                             favorite_pm,
                                                             favorite_mask);
                                } else {
                                        /* By inserting a ' ', the list can be
                                         * sorted by this column 
                                         */
                                        gtk_clist_set_text(list, pos,
                                                           favcolumn, " ");
                                }
                        }
                        for (n = 0;
                             n <
                             g_list_length(general_prefs.active_columns);
                             n++) {
                                g_free(rom[n]);
                        }

                        pos++;
                }
                sup++;
                listpointer = g_list_next(listpointer);

        }
        gtk_clist_thaw(list);

        visible_games = pos;
        supported_games = sup;

        grustibus_debug("Supported games: %i Visible games: %i",
                        supported_games, visible_games);

        return;
}

void update_gamelist_row (GtkCList * list, gint row) {
        gint n, favcolumn = -1, timesplayedcolumn = -1;
        GdkColor correctcol, incorrectcol, may_workcol, missingcol,
            favouritecol;
        GdkColormap *map;
        GList *listpointer;
        RomEntry *tmprom;
        GtkStyle *liststyle;
        gchar *tmpstring=NULL;
        GdkBitmap *favorite_mask = NULL;
        GdkPixmap *favorite_pm = NULL;

        liststyle = gtk_style_copy(gtk_widget_get_style(GTK_WIDGET(list)));

        if (liststyle == NULL) {
                gtk_style_unref(liststyle);
                liststyle = gtk_style_new();
        }

        correctcol.red = 256 * general_prefs.correct_color_r;
        correctcol.green = 256 * general_prefs.correct_color_g;
        correctcol.blue = 256 * general_prefs.correct_color_b;

        incorrectcol.red = 256 * general_prefs.incorrect_color_r;
        incorrectcol.green = 256 * general_prefs.incorrect_color_g;
        incorrectcol.blue = 256 * general_prefs.incorrect_color_b;

        may_workcol.red = 256 * general_prefs.may_work_color_r;
        may_workcol.green = 256 * general_prefs.may_work_color_g;
        may_workcol.blue = 256 * general_prefs.may_work_color_b;

        missingcol.red = 256 * general_prefs.missing_color_r;
        missingcol.green = 256 * general_prefs.missing_color_g;
        missingcol.blue = 256 * general_prefs.missing_color_b;

        favouritecol.red = 256 * general_prefs.favourite_color_r;
        favouritecol.green = 256 * general_prefs.favourite_color_g;
        favouritecol.blue = 256 * general_prefs.favourite_color_b;

        if (main_window->window != NULL)
                map = gdk_window_get_colormap(main_window->window);
        else
                map = gdk_colormap_new (gdk_visual_get_best(), TRUE);

        gdk_colormap_alloc_color(map, &correctcol, TRUE, TRUE);
        gdk_colormap_alloc_color(map, &incorrectcol, TRUE, TRUE);
        gdk_colormap_alloc_color(map, &may_workcol, TRUE, TRUE);
        gdk_colormap_alloc_color(map, &missingcol, TRUE, TRUE);
        gdk_colormap_alloc_color(map, &favouritecol, TRUE, TRUE);

        favorite_pm = gdk_pixmap_create_from_xpm_d(main_window->window,
                                                   &favorite_mask,
                                                   &liststyle->bg
                                                   [GTK_STATE_NORMAL],
                                                   (gchar **)
                                                   favorite_xpm);


        listpointer = g_list_first(general_prefs.active_columns);
        n = 0;
        while (listpointer) {
                if (GPOINTER_TO_INT(listpointer->data) == FAVORITE)
                        favcolumn = n;
                if (GPOINTER_TO_INT(listpointer->data) == TIMESPLAYED)
                        timesplayedcolumn = n;
                n++;
                listpointer = g_list_next(listpointer);
        }

        tmprom = (RomEntry *) gtk_clist_get_row_data(list, row);

        switch (tmprom->status) {
        case CORRECT:
                gtk_clist_set_foreground(list, row, &correctcol);
                break;
        case INCORRECT:
                gtk_clist_set_foreground(list, row, &incorrectcol);
                break;
        case MAY_WORK:
                gtk_clist_set_foreground(list, row, &may_workcol);
                break;
        case MISSING:
                gtk_clist_set_foreground(list, row, &missingcol);
                break;
        }

        if (tmprom->favourite && general_prefs.favourites_overrides)
                gtk_clist_set_foreground(list, row, &favouritecol);

        if (favcolumn > -1) {
                if (tmprom->favourite) {
                        gtk_clist_set_pixmap(list, row, favcolumn,
                                             favorite_pm, favorite_mask);
                } else {
                        /* By inserting a ' ', the list can be
                         * sorted by this column 
                         */
                        gtk_clist_set_text(list, row, favcolumn, " ");
                }
        }
        if (timesplayedcolumn > -1) {
                tmpstring=g_malloc(MAX_TIMESPLAYED);
                g_snprintf(tmpstring, MAX_TIMESPLAYED, "%i", tmprom->timesplayed);
                gtk_clist_set_text(list, row, timesplayedcolumn, tmpstring);
                g_free (tmpstring);
        }
        
        return;
}
/* 
 * Prepares the widgets for the columnheaders in the clist (All of them, not just the ones shown)
 */
void create_column_widgets()
{
        GList *tmplist;
        CListColumnLabel *tmpcolumnlabel;
        ColumnLabel *tmplabel;
        gchar **column_sizes;
        gint i;
        gboolean size_list_end_reached = FALSE;
        GdkBitmap *favorite_mask = NULL;
        GdkPixmap *favorite_pm = NULL;

        if (main_window->window != NULL)
                favorite_pm = gdk_pixmap_create_from_xpm_d(main_window->window,
                                                           &favorite_mask,
                                                           NULL, (gchar **)
                                                           favorite_xpm);
        else {
                favorite_pm = gdk_pixmap_colormap_create_from_xpm_d(NULL,
                                                           gdk_colormap_new(gdk_visual_get_best(), FALSE),
                                                           &favorite_mask,
                                                           NULL, (gchar **)
                                                           favorite_xpm);

        }

        tmplist = g_list_first(all_columns);
        column_sizes =
            g_strsplit(general_prefs.column_size_string, ",",
                       g_list_length(tmplist));

        i = 0;
        while (tmplist != NULL) {
                tmplabel = tmplist->data;
                tmpcolumnlabel = malloc(sizeof(CListColumnLabel));

                tmpcolumnlabel->number = tmplabel->number;
                if (column_sizes[i] && !size_list_end_reached) {
                        tmpcolumnlabel->size = atoi(column_sizes[i]);
                } else {
                        tmpcolumnlabel->size = 50;
                        size_list_end_reached = TRUE;
                }

                tmpcolumnlabel->label_hbox =
                    GTK_HBOX(gtk_hbox_new(FALSE, 0));
                gtk_widget_ref(GTK_WIDGET(tmpcolumnlabel->label_hbox));

                if (i == FAVORITE) {
                        tmpcolumnlabel->gtk_label =
                            gtk_pixmap_new(favorite_pm, favorite_mask);
                } else {
                        tmpcolumnlabel->gtk_label =
                            gtk_label_new((gchar *) tmplabel->label);
                }

                gtk_widget_ref(tmpcolumnlabel->gtk_label);
                gtk_box_pack_start(GTK_BOX(tmpcolumnlabel->label_hbox),
                                   GTK_WIDGET(tmpcolumnlabel->gtk_label),
                                   FALSE, FALSE, 0);
                gtk_widget_show(tmpcolumnlabel->gtk_label);

                tmpcolumnlabel->arrow =
                    GTK_ARROW(gtk_arrow_new
                              (GTK_ARROW_DOWN, GTK_SHADOW_NONE));
                gtk_widget_ref(GTK_WIDGET(tmpcolumnlabel->arrow));
                gtk_box_pack_end(GTK_BOX(tmpcolumnlabel->label_hbox),
                                 GTK_WIDGET(tmpcolumnlabel->arrow), FALSE,
                                 FALSE, 0);

                CListColumnLabels =
                    g_list_prepend(CListColumnLabels, tmpcolumnlabel);
                tmplist = g_list_next(tmplist);
                i++;
        }

        CListColumnLabels = g_list_reverse(CListColumnLabels);
        g_strfreev(column_sizes);
}

/* Creates a list widget according to wich columns are currently selected to
 * be shown 
 */
void create_gamelist_widget(GtkCList ** list)
{
        GList *tmp_active_list;
        GList *tmp_all_list;
        CListColumnLabel *tmplabel;
        gboolean found_sort_by_column=FALSE;
        gint i;

        tmp_all_list = g_list_first(CListColumnLabels);
        tmp_active_list = g_list_first(general_prefs.active_columns);

        i = 0;
        if (*list != NULL) {
                gtk_clist_clear(*list);
                while (gtk_clist_get_column_widget(GTK_CLIST(*list), i)) {
                        gtk_clist_set_column_widget(GTK_CLIST(*list), i,
                                                    NULL);
                        i++;
                }
                gtk_container_remove(GTK_CONTAINER(appwin.scrolled),
                                     GTK_WIDGET(*list));
                gtk_widget_destroy(GTK_WIDGET(*list));
        }

        i = 0;

        *list = GTK_CLIST(gtk_clist_new(g_list_length(tmp_active_list)));
        gtk_widget_ref(GTK_WIDGET(*list));
        gtk_object_set_data_full(GTK_OBJECT(main_window), "game_clist",
                                 *list,
                                 (GtkDestroyNotify) gtk_widget_unref);

        gtk_container_add(GTK_CONTAINER(appwin.scrolled),
                          GTK_WIDGET(*list));
        gtk_container_set_border_width(GTK_CONTAINER(*list), 5);
        gtk_clist_set_selection_mode(GTK_CLIST(*list),
                                     GTK_SELECTION_BROWSE);
        gtk_clist_column_titles_show(GTK_CLIST(*list));

        /* Traverse the list and pick out the selected columns */
        i = 0;
        while (tmp_active_list != NULL) {
                tmp_all_list = g_list_first(CListColumnLabels);
                while (tmp_all_list != NULL) {
                        tmplabel = tmp_all_list->data;
                        if (tmplabel->number ==
                            GPOINTER_TO_INT(tmp_active_list->data)) {
                                gtk_clist_set_column_widget(GTK_CLIST
                                                            (*list), i,
                                                            GTK_WIDGET
                                                            (tmplabel->
                                                             label_hbox));
                                gtk_widget_show(GTK_WIDGET
                                                (tmplabel->label_hbox));
                                gtk_clist_set_column_width(GTK_CLIST
                                                           (*list), i,
                                                           tmplabel->size);
                                if (tmplabel->number ==
                                    general_prefs.sort_by) {
                                        found_sort_by_column=TRUE;
                                        if (general_prefs.sort_order) {
                                                gtk_arrow_set(tmplabel->
                                                              arrow,
                                                              GTK_ARROW_UP,
                                                              GTK_SHADOW_IN);
                                        } else {
                                                gtk_arrow_set(tmplabel->
                                                              arrow,
                                                              GTK_ARROW_DOWN,
                                                              GTK_SHADOW_IN);
                                        }
                                        gtk_widget_show(GTK_WIDGET
                                                        (tmplabel->arrow));
                                }
                                tmp_all_list = g_list_last(tmp_all_list);
                                i++;
                        }
                        tmp_all_list = g_list_next(tmp_all_list);
                }
                tmp_active_list = g_list_next(tmp_active_list);
        }

        gtk_signal_connect(GTK_OBJECT(*list), "select_row",
                           GTK_SIGNAL_FUNC(on_game_clist_select_row),
                           NULL);
        gtk_signal_connect(GTK_OBJECT(*list), "resize_column",
                           GTK_SIGNAL_FUNC(on_game_clist_resize_column),
                           NULL);
        gtk_signal_connect(GTK_OBJECT(*list), "click_column",
                           GTK_SIGNAL_FUNC(on_game_clist_click_column),
                           NULL);
        gtk_signal_connect(GTK_OBJECT(*list), "key_release_event",
                           GTK_SIGNAL_FUNC
                           (on_game_clist_key_release_event), NULL);
        gtk_signal_connect(GTK_OBJECT(*list), "focus_out_event",
                           GTK_SIGNAL_FUNC(on_game_clist_focus_out_event),
                           NULL);
        gtk_signal_connect(GTK_OBJECT(*list), "focus_in_event",
                           GTK_SIGNAL_FUNC(on_game_clist_focus_in_event),
                           NULL);

        gtk_widget_grab_focus(GTK_WIDGET(*list));

        if (!found_sort_by_column) set_sort_by_column (*list, 0);
        gtk_widget_show(GTK_WIDGET(*list));
}


/* This function is placed among the gui functions since it searches through
 * the contents of the actual clist widget 
 */
void lookup_game(const gchar * search_string)
{
        gint row;
        gint endrow;
        gint len;
        gint column;
        gchar *listentry;
        gchar *listentry2;
        gchar *new_search_string = NULL;
        GameLookUp *tmp;
        gboolean hit = FALSE;

        column = lookup_column(general_prefs.sort_by);
        len = strlen(search_string);
        /* New searches starts from the beginning of the list */
        if (len == 1)
                row = 0;
        else
                row = general_prefs.current_row;

        do {
                gtk_clist_get_text(appwin.list, row, column, &listentry);
                row++;
        } while ((g_strncasecmp(search_string, listentry, len))
                 && row < visible_games);

        /* Reset the row variable if the row is not a match (the list has been traversed without a hit */
        if (g_strncasecmp(search_string, listentry, len))
                row = general_prefs.current_row + 1;
        else {
                hit = TRUE;
                endrow = row - 1;
                /* Find the span of the match */
                do {
                        endrow++;
                        gtk_clist_get_text(appwin.list,
                                           endrow, column, &listentry2);
                } while ((!g_strncasecmp(search_string, listentry2, len))
                         && endrow < visible_games);

                /* Get back to the last good match */
                gtk_clist_get_text(appwin.list,
                                   endrow - 1, column, &listentry2);

                /* Find the length of the match */
                while ((!g_strncasecmp(listentry, listentry2, len))
                       && (len <= strlen(listentry))
                       && (len <= strlen(listentry2))) {
                        len++;
                }
                new_search_string = g_strndup(listentry, len - 1);
        }
        if (hit) {
                /* Save a copy in the history list */
                tmp = g_malloc(sizeof(GameLookUp));
                g_snprintf(tmp->search_string, MAX_SEARCH_LENGTH, "%s",
                           romlookup.search_string);
                tmp->pos = romlookup.pos;
                lookup_filo = g_list_prepend(lookup_filo, (gpointer) tmp);

                /* Set the new info in the romlookup struct */
                g_snprintf(romlookup.search_string, MAX_SEARCH_LENGTH,
                           "%s", new_search_string);
                romlookup.pos = row - 1;
                g_free(new_search_string);

        }
}

void set_prefs_widgets(GtkWidget * prefs_window)
{
        gtk_toggle_button_set_active
            (prefswin.playback_no_highscore_togglebutton,
             general_prefs.playback_no_highscore);
        gtk_toggle_button_set_active
            (prefswin.iconify_while_mame_togglebutton,
             general_prefs.iconify_while_xmame);
        gtk_toggle_button_set_active
            (prefswin.loadsa_image_formats_togglebutton,
             general_prefs.loadsa_image_formats);
        gtk_toggle_button_set_active
            (prefswin.look_for_clone_images_togglebutton,
             general_prefs.look_for_clone_images);
        gtk_toggle_button_set_active
            (prefswin.indicate_notebook_changes_togglebutton,
             general_prefs.indicate_notebook_changes);

        gtk_toggle_button_set_active(prefswin.fullscreen_togglebutton,
                                     global_settings.fullscreen);
        gtk_toggle_button_set_active(prefswin.scanlines_togglebutton,
                                     global_settings.scanlines);
        gtk_toggle_button_set_active(prefswin.x_artwork_togglebutton,
                                     global_settings.extra_artwork);
        gtk_toggle_button_set_active(prefswin.autoframeskip_togglebutton,
                                     global_settings.autoframeskip);
        gtk_toggle_button_set_active(prefswin.auto_colordepth_togglebutton,
                                     global_settings.auto_colordepth);
        gtk_toggle_button_set_active(prefswin.rotatel_togglebutton,
                                     global_settings.rot_left);
        gtk_toggle_button_set_active(prefswin.rotater_togglebutton,
                                     global_settings.rot_right);
        gtk_toggle_button_set_active(prefswin.flipx_togglebutton,
                                     global_settings.flipx);
        gtk_toggle_button_set_active(prefswin.flipy_togglebutton,
                                     global_settings.flipy);
        gtk_adjustment_set_value(gtk_range_get_adjustment
                                 (GTK_RANGE(prefswin.scale_hscale)),
                                 global_settings.scale);

        gtk_toggle_button_set_active(prefswin.antialias_togglebutton,
                                     global_settings.antialias);
        gtk_toggle_button_set_active(prefswin.translucency_togglebutton,
                                     global_settings.translucency);
        gtk_adjustment_set_value(gtk_range_get_adjustment
                                 (GTK_RANGE(prefswin.beam_hscale)),
                                 global_settings.beam);
        gtk_adjustment_set_value(gtk_range_get_adjustment
                                 (GTK_RANGE(prefswin.flicker_hscale)),
                                 global_settings.flicker);
        gtk_option_menu_set_history(prefswin.vectorres_option,
                                    global_settings.vectorres);

        gtk_toggle_button_set_active(prefswin.analog_joy_togglebutton,
                                     global_settings.analog_joy);
        gtk_toggle_button_set_active(prefswin.mouse_togglebutton,
                                     global_settings.mouse);
        gtk_toggle_button_set_active(prefswin.winkeys_togglebutton,
                                     global_settings.winkeys);
        gtk_toggle_button_set_active(prefswin.grab_mouse_togglebutton,
                                     global_settings.grab_mouse);
        gtk_option_menu_set_history(prefswin.joy_option,
                                    global_settings.joytype);



        gtk_toggle_button_set_active(prefswin.sound_togglebutton,
                                     global_settings.sound);
        gtk_toggle_button_set_active(prefswin.samples_togglebutton,
                                     global_settings.samples);
        gtk_toggle_button_set_active(prefswin.fakesound_togglebutton,
                                     global_settings.fake_sound);
        gtk_adjustment_set_value(gtk_range_get_adjustment
                                 (GTK_RANGE(prefswin.volume_hscale)),
                                 global_settings.volume);

        gtk_toggle_button_set_active(prefswin.cheat_togglebutton,
                                     global_settings.cheat);
        if (global_settings.extra_options) {
                gtk_entry_set_text(prefswin.extra_options,
                                   global_settings.extra_options);
        }

        gtk_entry_set_text(prefswin.xmame_exe_entry,
                           general_prefs.xmame_exe);
        gtk_entry_set_text(prefswin.rom_path_entry, general_prefs.rom_dir);
        gtk_entry_set_text(prefswin.screenshots_dir_entry,
                           general_prefs.screenshot_dir);
        gtk_entry_set_text(prefswin.highscore_dir_entry,
                           general_prefs.highscore_dir);
        gtk_entry_set_text(prefswin.flyers_dir_entry,
                           general_prefs.flyer_dir);
        gtk_entry_set_text(prefswin.cabinets_dir_entry,
                           general_prefs.cabinet_dir);
        gtk_entry_set_text(prefswin.gamehistory_path_entry,
                           general_prefs.game_history_file);
        gtk_entry_set_text(prefswin.cheatfile_path_entry,
                           general_prefs.cheat_file);
        gtk_entry_set_text(prefswin.playback_dir_entry,
                           general_prefs.playback_dir);
        gnome_color_picker_set_i8(prefswin.favourite_colorpicker,
                                  general_prefs.favourite_color_r,
                                  general_prefs.favourite_color_g,
                                  general_prefs.favourite_color_b, 255);
        gnome_color_picker_set_i8(prefswin.correct_colorpicker,
                                  general_prefs.correct_color_r,
                                  general_prefs.correct_color_g,
                                  general_prefs.correct_color_b, 255);
        gnome_color_picker_set_i8(prefswin.incorrect_colorpicker,
                                  general_prefs.incorrect_color_r,
                                  general_prefs.incorrect_color_g,
                                  general_prefs.incorrect_color_b, 255);
        gnome_color_picker_set_i8(prefswin.missing_colorpicker,
                                  general_prefs.missing_color_r,
                                  general_prefs.missing_color_g,
                                  general_prefs.missing_color_b, 255);
        gnome_color_picker_set_i8(prefswin.may_work_colorpicker,
                                  general_prefs.may_work_color_r,
                                  general_prefs.may_work_color_g,
                                  general_prefs.may_work_color_b, 255);
        gnome_font_picker_set_font_name(prefswin.fontpicker,
                                        general_prefs.font);
        gtk_entry_set_text(prefswin.fontentry, general_prefs.font);

        gtk_toggle_button_set_active
            (prefswin.favourite_overrides_togglebutton,
             general_prefs.favourites_overrides);


        gtk_entry_set_text(prefswin.rom_url_entry,
                           general_prefs.rom_url);
        
        gtk_entry_set_text(prefswin.screenshot_url_entry,
                           general_prefs.screenshot_url);

        gtk_entry_set_text(prefswin.flyer_url_entry,
                           general_prefs.flyer_url);

        gtk_entry_set_text(prefswin.cabinet_url_entry,
                           general_prefs.cabinet_url);
        return;
}

void set_notebooktab_status(gint tab_num, gboolean status)
{
        if (status) {
                gtk_pixmap_set(appwin.notebook_leds[tab_num],
                               appwin.active_led_pixmap,
                               appwin.active_led_mask);
        } else {
                gtk_pixmap_set(appwin.notebook_leds[tab_num],
                               appwin.inactive_led_pixmap,
                               appwin.inactive_led_mask);
        }
}

void notebooktab_status_show_all_but(gint tab_num)
{
        gint i;

        for (i = 0; i < LAST_PAGE; i++) {
                gtk_widget_show_all(GTK_WIDGET(appwin.notebook_leds[i]));
        }

        gtk_widget_hide_all(GTK_WIDGET(appwin.notebook_leds[tab_num]));

}

void notebooktab_status_hide_all()
{
        gint i;

        for (i = 0; i < LAST_PAGE; i++) {
                gtk_widget_hide_all(GTK_WIDGET(appwin.notebook_leds[i]));
        }
}

/* Sets the game setting widgets, except the favourite game and default options togglebuttons */
void set_game_settings_widgets()
{
        gchar *tooltip = NULL;
        gchar *gamename = NULL;
        GtkWidget *save_button;
        GtkTooltips *tooltips;
        RomEntry *romdata;

        tooltip = g_malloc0(50);

        romdata =
            gtk_clist_get_row_data(appwin.list, general_prefs.current_row);
        if (romdata) {
                gamename = romdata->gamename;
        }

        if (romdata->vector) {
                gtk_widget_show(appwin.vector_frame);
                /* Set the vector widgets */
                gtk_toggle_button_set_active(gtk_object_get_data
                                             (GTK_OBJECT(main_window),
                                              "vector_antialias_checkbutton"),
                                             game_settings.antialias);

                gtk_toggle_button_set_active(gtk_object_get_data
                                             (GTK_OBJECT(main_window),
                                              "vector_translucency_checkbutton"),
                                             game_settings.translucency);

                gtk_option_menu_set_history(GTK_OPTION_MENU
                                            (appwin.vectorres_option),
                                            game_settings.vectorres);

                gtk_adjustment_set_value(gtk_range_get_adjustment
                                         (gtk_object_get_data(GTK_OBJECT
                                                              (main_window),
                                                              "vector_beam_hscale")),
                                         game_settings.beam);

                gtk_adjustment_set_value(gtk_range_get_adjustment
                                         (gtk_object_get_data(GTK_OBJECT
                                                              (main_window),
                                                              "vector_flicker_hscale")),
                                         game_settings.flicker);

        } else {
                gtk_widget_hide(appwin.vector_frame);
                gtk_widget_queue_resize(appwin.display_vbox);
        }
        gtk_toggle_button_set_active(gtk_object_get_data
                                     (GTK_OBJECT(main_window),
                                      "default_settings_checkbutton"),
                                     game_settings.default_options);

        gtk_toggle_button_set_active(gtk_object_get_data
                                     (GTK_OBJECT(main_window),
                                      "dga_checkbutton"),
                                     game_settings.fullscreen);
        gtk_toggle_button_set_active(gtk_object_get_data
                                     (GTK_OBJECT(main_window),
                                      "scanlines_checkbutton"),
                                     game_settings.scanlines);
        gtk_toggle_button_set_active(gtk_object_get_data
                                     (GTK_OBJECT(main_window),
                                      "artwork_checkbutton"),
                                     game_settings.extra_artwork);
        gtk_toggle_button_set_active(gtk_object_get_data
                                     (GTK_OBJECT(main_window),
                                      "autoframeskip_checkbutton"),
                                     game_settings.autoframeskip);
        gtk_toggle_button_set_active(gtk_object_get_data
                                     (GTK_OBJECT(main_window),
                                      "colordepth_checkbutton"),
                                     game_settings.auto_colordepth);
        gtk_toggle_button_set_active(gtk_object_get_data
                                     (GTK_OBJECT(main_window),
                                      "rotl_checkbutton"),
                                     game_settings.rot_left);
        gtk_toggle_button_set_active(gtk_object_get_data
                                     (GTK_OBJECT(main_window),
                                      "rotr_checkbutton"),
                                     game_settings.rot_right);
        gtk_toggle_button_set_active(gtk_object_get_data
                                     (GTK_OBJECT(main_window),
                                      "xflip_checkbutton"),
                                     game_settings.flipx);
        gtk_toggle_button_set_active(gtk_object_get_data
                                     (GTK_OBJECT(main_window),
                                      "yflip_checkbutton"),
                                     game_settings.flipy);
        gtk_adjustment_set_value(gtk_range_get_adjustment
                                 (gtk_object_get_data(GTK_OBJECT
                                                      (main_window),
                                                      "scale_hscale")),
                                 game_settings.scale);

        gtk_toggle_button_set_active(gtk_object_get_data
                                     (GTK_OBJECT(main_window),
                                      "analog_joy_checkbutton"),
                                     game_settings.analog_joy);
        gtk_toggle_button_set_active(gtk_object_get_data
                                     (GTK_OBJECT(main_window),
                                      "mouse_checkbutton"),
                                     game_settings.mouse);
        gtk_toggle_button_set_active(gtk_object_get_data
                                     (GTK_OBJECT(main_window),
                                      "winkeys_checkbutton"),
                                     game_settings.winkeys);
        gtk_toggle_button_set_active(gtk_object_get_data
                                     (GTK_OBJECT(main_window),
                                      "grab_mouse_checkbutton"),
                                     game_settings.grab_mouse);
        gtk_option_menu_set_history(GTK_OPTION_MENU(appwin.joy_option),
                                    game_settings.joytype);


        gtk_toggle_button_set_active(gtk_object_get_data
                                     (GTK_OBJECT(main_window),
                                      "sound_checkbutton"),
                                     game_settings.sound);
        gtk_toggle_button_set_active(gtk_object_get_data
                                     (GTK_OBJECT(main_window),
                                      "samples_checkbutton"),
                                     game_settings.samples);
        gtk_toggle_button_set_active(gtk_object_get_data
                                     (GTK_OBJECT(main_window),
                                      "fakesound_checkbutton"),
                                     game_settings.fake_sound);
        gtk_adjustment_set_value(gtk_range_get_adjustment
                                 (gtk_object_get_data(GTK_OBJECT
                                                      (main_window),
                                                      "volume_hscale")),
                                 game_settings.volume);

        gtk_toggle_button_set_active(gtk_object_get_data
                                     (GTK_OBJECT(main_window),
                                      "cheat_checkbutton"),
                                     game_settings.cheat);
        if (game_settings.extra_options) {
                gtk_entry_set_text(GTK_ENTRY
                                   (gnome_entry_gtk_entry
                                    (gtk_object_get_data
                                     (GTK_OBJECT(main_window),
                                      "extra_entry"))),
                                   game_settings.extra_options);
        }

        gtk_widget_set_sensitive(lookup_widget
                                 (GTK_WIDGET(main_window), "vbox17"),
                                 !game_settings.default_options);

        g_snprintf(tooltip, 50, _("Save settings for %s"), gamename);
        tooltips =
            gtk_object_get_data(GTK_OBJECT(main_window), "tooltips");
        save_button =
            gtk_object_get_data(GTK_OBJECT(main_window),
                                "save_settings_button");

        gtk_tooltips_set_tip(tooltips, save_button, tooltip, NULL);
}

gboolean set_game_history(const gchar * game)
{
        FILE *history_file;
        GtkText *history_txt;
        gchar line[2000];
        gint i, n;
        gchar *tmp, *p;
        gchar **games;
        gboolean found_game = FALSE;
        gboolean pointer_in_info = FALSE;

        history_txt = GTK_TEXT(gtk_object_get_data
                               (GTK_OBJECT(main_window),
                                "game_history_text"));

        gtk_text_freeze(history_txt);
        gtk_text_set_point(history_txt, gtk_text_get_length(history_txt));
        gtk_text_backward_delete(history_txt,
                                 gtk_text_get_length(history_txt));

        history_file = fopen(general_prefs.game_history_file, "r");

        if (!history_file) {
                gtk_text_insert(history_txt,
                                NULL,
                                NULL,
                                NULL,
                                _
                                ("\n\n           History.dat file not found"),
                                -1);
                gtk_text_thaw(history_txt);
                return (FALSE);
        }

        /* TODO: history.dat should be read in a function in fileio.c that should return
         * the read text or NULL if no text was found
         */

        /* Scan through the file. First look for '$',then $info=game, $bio and $end */
        while (fgets(line, 500, history_file)) {
                p = line;
                tmp = p;
                if (*tmp == '$' && !found_game) {
                        /* Found a line */
                        i = 0;
                        while (*tmp && (*tmp++ != '=') && (*tmp != '\n'))
                                i++;
                        p[i] = 0;
                        if (!strncmp(p, "$info", 5)) {
                                p = tmp;
                                /* It is an info line */
                                i = 0;
                                while (*tmp && (*tmp++ != '\n'))
                                        i++;
                                /* Sometimes the list is continued on the next line */
                                i--;
                                do {
                                        p[i] = fgetc(history_file);
                                        if (p[i] == '\n')
                                                i--;
                                        if (p[i] == '\r')
                                                i--;
                                        i++;
                                } while ((p[i - 1] != '$'));
                                p[i - 1] = 0;
                                games = g_strsplit(p, ",", 20);
                                n = 0;
                                while ((games[n] != NULL)) {
                                        games[n] = g_strchomp(games[n]);

                                        if (!strcmp(games[n], game)) {
                                                /* It is the info for the wanted game */
                                                found_game = TRUE;
                                        }
                                        n++;
                                }
                                g_strfreev(games);
                        }
                } else if (found_game && (*tmp == '$')) {
                        i = 0;
                        while (*tmp && (*tmp++ != '=') && (*tmp != '\n'))
                                i++;
                        p[i] = 0;
                        if (!strncmp(p, "$bio", 4))
                                pointer_in_info = TRUE;
                        if (!strncmp(p, "$end", 4)) {
                                pointer_in_info = FALSE;
                                break;
                        }
                } else if (found_game && pointer_in_info) {
                        i = 0;
                        while (*tmp && (*tmp++ != '=') && (*tmp != '\n'))
                                i++;
                        if (i == 0)
                                p[i] = '\n';
                        else
                                p[i] = ' ';

                        p[i + 1] = 0;
                        gtk_text_insert(history_txt,
                                        NULL, NULL, NULL, p, -1);
                        grustibus_debug("Added history txt: %s", p);
                }
        }

        if (!found_game) {
                tmp = g_malloc(100);
                g_snprintf(tmp, 100,
                           _
                           ("\n\n           No history information found for %s"),
                           game);
                gtk_text_insert(history_txt, NULL, NULL, NULL, tmp, -1);
                g_free(tmp);
        }

        fclose(history_file);
        gtk_text_thaw(history_txt);

        return (found_game);
}

void set_game_info_widgets()
{
        RomEntry *romdata;
        romdata =
            gtk_clist_get_row_data(appwin.list, general_prefs.current_row);

        if (strcmp(romdata->cpu1, "-"))
                gtk_entry_set_text(GTK_ENTRY
                                   (gtk_object_get_data
                                    (GTK_OBJECT(main_window),
                                     "cpu1_entry")), romdata->cpu1);
        else
                gtk_entry_set_text(GTK_ENTRY
                                   (gtk_object_get_data
                                    (GTK_OBJECT(main_window),
                                     "cpu1_entry")), _("None"));


        if (strcmp(romdata->cpu2, "-"))
                gtk_entry_set_text(GTK_ENTRY
                                   (gtk_object_get_data
                                    (GTK_OBJECT(main_window),
                                     "cpu2_entry")), romdata->cpu2);
        else
                gtk_entry_set_text(GTK_ENTRY
                                   (gtk_object_get_data
                                    (GTK_OBJECT(main_window),
                                     "cpu2_entry")), _("None"));

        if (strcmp(romdata->cpu3, "-"))
                gtk_entry_set_text(GTK_ENTRY
                                   (gtk_object_get_data
                                    (GTK_OBJECT(main_window),
                                     "cpu3_entry")), romdata->cpu3);
        else
                gtk_entry_set_text(GTK_ENTRY
                                   (gtk_object_get_data
                                    (GTK_OBJECT(main_window),
                                     "cpu3_entry")), _("None"));

        if (strcmp(romdata->cpu4, "-"))
                gtk_entry_set_text(GTK_ENTRY
                                   (gtk_object_get_data
                                    (GTK_OBJECT(main_window),
                                     "cpu4_entry")), romdata->cpu4);
        else
                gtk_entry_set_text(GTK_ENTRY
                                   (gtk_object_get_data
                                    (GTK_OBJECT(main_window),
                                     "cpu4_entry")), _("None"));


        if (strcmp(romdata->sound1, "-"))
                gtk_entry_set_text(GTK_ENTRY
                                   (gtk_object_get_data
                                    (GTK_OBJECT(main_window),
                                     "sound1_entry")), romdata->sound1);
        else
                gtk_entry_set_text(GTK_ENTRY
                                   (gtk_object_get_data
                                    (GTK_OBJECT(main_window),
                                     "sound1_entry")), _("None"));

        if (strcmp(romdata->sound2, "-"))
                gtk_entry_set_text(GTK_ENTRY
                                   (gtk_object_get_data
                                    (GTK_OBJECT(main_window),
                                     "sound2_entry")), romdata->sound2);
        else
                gtk_entry_set_text(GTK_ENTRY
                                   (gtk_object_get_data
                                    (GTK_OBJECT(main_window),
                                     "sound2_entry")), _("None"));

        if (strcmp(romdata->sound3, "-"))
                gtk_entry_set_text(GTK_ENTRY
                                   (gtk_object_get_data
                                    (GTK_OBJECT(main_window),
                                     "sound3_entry")), romdata->sound3);
        else
                gtk_entry_set_text(GTK_ENTRY
                                   (gtk_object_get_data
                                    (GTK_OBJECT(main_window),
                                     "sound3_entry")), _("None"));

        if (strcmp(romdata->sound4, "-"))
                gtk_entry_set_text(GTK_ENTRY
                                   (gtk_object_get_data
                                    (GTK_OBJECT(main_window),
                                     "sound4_entry")), romdata->sound4);
        else
                gtk_entry_set_text(GTK_ENTRY
                                   (gtk_object_get_data
                                    (GTK_OBJECT(main_window),
                                     "sound4_entry")), _("None"));


        gtk_entry_set_text(GTK_ENTRY
                           (gtk_object_get_data
                            (GTK_OBJECT(main_window),
                             "status_entry")),
                           romdata->working ? _("Good") :
                           _("Preliminary"));

        gtk_entry_set_text(GTK_ENTRY
                           (gtk_object_get_data
                            (GTK_OBJECT(main_window),
                             "driver_entry")), romdata->driver);
}

/* Function to render the screenshot/flyer/cabinet images */
void draw_image(ImageCanvas * imagestruct, const gchar * gamename,
                const gchar * imagefile)
{
        gdouble height = 0, width =
            0, src_height, src_width, dest_height, dest_width;
        GdkPixbuf *newpixbuf = NULL;

        /* No need to render images that are too small to be seen anyway */
        if (imagestruct->canvas->parent->allocation.width < 10 ||
            imagestruct->canvas->parent->allocation.height < 10) {
                return;
        }

        /* It is not necessary to load the same image several times) */
        if (strcmp(gamename, imagestruct->name)) {
                grustibus_debug
                    ("New image: game name: %s, previous image: %s",
                     gamename, imagestruct->name);
                if (imagefile) {
                        newpixbuf = gdk_pixbuf_new_from_file(imagefile);
                }

                if (!newpixbuf) {
                        newpixbuf = imagestruct->imagestruct.default_image;
                        gdk_pixbuf_ref(imagestruct->
                                       imagestruct.default_image);
                }

                g_free(imagestruct->name);
                imagestruct->name = g_strdup(gamename);
        } else {
                grustibus_debug
                    ("Same image: game name: %s, previous image: %s",
                     gamename, imagestruct->name);
                newpixbuf = imagestruct->imagestruct.image;
                gdk_pixbuf_ref(imagestruct->imagestruct.image);
        }

        gdk_pixbuf_unref(imagestruct->imagestruct.image);

        src_width = (gfloat) gdk_pixbuf_get_width(newpixbuf);
        src_height = (gfloat) gdk_pixbuf_get_height(newpixbuf);

        gtk_aspect_frame_set(GTK_ASPECT_FRAME(imagestruct->canvas->parent),
                             0.5, 0.5, src_width / src_height, FALSE);

        /*Just reading the size of the aspect_frame doesn't work, because it is not
         * always updated soon enough.
         */
        dest_width =
            imagestruct->canvas->parent->parent->allocation.width - 27;
        dest_height =
            imagestruct->canvas->parent->parent->allocation.height - 27;

        width = dest_width;
        height = ((src_height * width) / src_width);

        if (height > dest_height) {
                height = dest_height;
                width = ((src_width * height) / src_height);
        }

        grustibus_debug("pixbuf height: %g , pixbuf width: %g", src_height,
                        src_width);
        grustibus_debug("Aspect: %g", src_width / src_height);
        grustibus_debug("render height: %g , render width: %g", height,
                        width);

        gnome_canvas_set_scroll_region(GNOME_CANVAS
                                       (imagestruct->canvas), 0, 0,
                                       width, height);
        imagestruct->imagestruct.image =
            gdk_pixbuf_scale_simple(newpixbuf, (gint) width, (gint) height,
                                    1);

        gnome_canvas_item_set(imagestruct->image_item,
                              "pixbuf", imagestruct->imagestruct.image,
                              "width", width, "height", height, NULL);

        gdk_pixbuf_unref(newpixbuf);
}



void start_game(gint row)
{
        gchar *game;
        RomEntry *romdata;

        if (xmame_version_ok) {
                romdata =
                    gtk_clist_get_row_data(appwin.list,
                                           general_prefs.current_row);
                if (romdata) {
                        game = romdata->romname;

                        iconify_window();
                        real_start_game(game, romdata);
                        game_played_once_more(romdata);
                        update_gamelist_row (appwin.list, general_prefs.current_row); 
                        deiconify_window();
                } else {
                        grustibus_warning(_
                                          ("Your xmame version is not supported by gRustibus.\nPlease upgrade."));
                }
        }
}

void iconify_window()
{
        GdkWindow *window;
        Status iconify_status;

        window = main_window->window;

        if (general_prefs.iconify_while_xmame) {
                iconify_status = XIconifyWindow(GDK_DISPLAY(),
                                                GDK_WINDOW_XWINDOW(window),
                                                DefaultScreen(GDK_DISPLAY
                                                              ()));

                if (!iconify_status)
                        g_warning(_("The window could not be iconified."));
                gdk_flush();
        }

}

void deiconify_window()
{
        GdkWindow *window;

        window = main_window->window;

        if (general_prefs.iconify_while_xmame)
                XMapWindow(GDK_DISPLAY(), GDK_WINDOW_XWINDOW(window));

}

void grustibus_warning(const gchar * format, ...)
{
        va_list args;
        gchar *message;

        g_return_if_fail(format != NULL);

        va_start(args, format);
        message = g_strdup_vprintf(format, args);
        va_end(args);

        gnome_app_warning(GNOME_APP(main_window), message);

        g_free(message);
        key_pressed = FALSE;
}

gboolean grustibus_question(gchar **button_array, const gchar *format, ...)
{
        va_list args;
        gchar *message;
        GtkWidget *dialog;
        
        gint answer;

        g_return_val_if_fail(format != NULL, FALSE);

        va_start(args, format);
        message = g_strdup_vprintf(format, args);
        va_end(args);

        dialog = gnome_message_box_newv(message, GNOME_MESSAGE_BOX_QUESTION, (const gchar **)button_array);
        answer = gnome_dialog_run_and_close(GNOME_DIALOG(dialog));

        g_free(message);
        key_pressed = FALSE;

        return (answer);
}

void set_busy_cursor(gboolean state)
{
        if (state) {
                gdk_window_set_cursor(main_window->window, busy_cursor);
        } else {
                gdk_window_set_cursor(main_window->window, NULL);
        }
}

/* A clist compare function that compares alphabetically but ignores case */
gint grustibus_clist_compare_func(GtkCList * clist,
                                  gconstpointer ptr1, gconstpointer ptr2)
{
        const GtkCListRow *row1 = (const GtkCListRow *) ptr1;
        const GtkCListRow *row2 = (const GtkCListRow *) ptr2;
        gchar *text1 = NULL;
        gchar *text2 = NULL;

        text1 = GTK_CELL_TEXT(row1->cell[clist->sort_column])->text;
        text2 = GTK_CELL_TEXT(row2->cell[clist->sort_column])->text;

        return g_strcasecmp(text1, text2);
}

// vim: ai sw=8 ts=8
