/*
 * gui.h -- Function prototypes for gui.c
 *
 * gRustibus (c) 2000-2001 Kjetil Thuen <kjetilt@users.sourceforge.net>
 * MAME frontend for the GNOME desktop environment.
 *
 * gRustibus is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * This code is indented with GNU indent using k&r style with 8 character
 * indents as suggested by the GNOME project. Please keep it that way.
 * ($ indent -kr -ts8 -i8 *.[ch])
 *
 * Last modified: 22 April 2001 <kjetilt@users.sourceforge.net>
 *
 */

#include <gnome.h>
#include <X11/Xlib.h>
#include <gdk/gdkx.h>
#include "support.h"


typedef enum _NoteBookTabs {
        SCREENSHOT_PAGE,
        SETTINGS_PAGE,
        INFO_PAGE,
        FLYER_PAGE,
        CABINET_PAGE,
        LAST_PAGE               // This is not really a page. Just something to count up to
} NoteBookTabs;

typedef struct _CListColumnLabel {
        gint size;
        Columns number;
        GtkArrow *arrow;
        GtkWidget *gtk_label;
        GtkHBox *label_hbox;
} CListColumnLabel;

struct MainWindowWidgets {
        GtkCList *list;
        GtkPaned *pane;
        GtkNotebook *notebook;
        GnomeAppBar *msgbar;
        GtkWidget *scrolled;
        GtkOptionMenu *joy_option;
        GtkOptionMenu *vectorres_option;
        GtkToggleButton *favourite_button;
        GdkPixmap *active_led_pixmap;
        GdkBitmap *active_led_mask;
        GdkPixmap *inactive_led_pixmap;
        GdkBitmap *inactive_led_mask;
        GtkPixmap *notebook_leds[LAST_PAGE];
        GtkWidget *vector_frame;
        GtkWidget *display_vbox;
};

struct PrefsWindowWidgets {
        GtkToggleButton *playback_no_highscore_togglebutton;
        GtkToggleButton *iconify_while_mame_togglebutton;
        GtkToggleButton *loadsa_image_formats_togglebutton;
        GtkToggleButton *look_for_clone_images_togglebutton;
        GtkToggleButton *indicate_notebook_changes_togglebutton;

        GtkToggleButton *fullscreen_togglebutton;
        GtkToggleButton *scanlines_togglebutton;
        GtkToggleButton *x_artwork_togglebutton;
        GtkToggleButton *autoframeskip_togglebutton;
        GtkToggleButton *auto_colordepth_togglebutton;
        GtkToggleButton *rotatel_togglebutton;
        GtkToggleButton *rotater_togglebutton;
        GtkToggleButton *flipx_togglebutton;
        GtkToggleButton *flipy_togglebutton;
        GtkHScale *scale_hscale;

        GtkToggleButton *translucency_togglebutton;
        GtkToggleButton *antialias_togglebutton;
        GtkHScale *flicker_hscale;
        GtkHScale *beam_hscale;
        GtkOptionMenu *vectorres_option;

        GtkToggleButton *sound_togglebutton;
        GtkToggleButton *samples_togglebutton;
        GtkToggleButton *fakesound_togglebutton;
        GtkHScale *volume_hscale;

        GtkOptionMenu *joy_option;
        GtkToggleButton *analog_joy_togglebutton;
        GtkToggleButton *mouse_togglebutton;
        GtkToggleButton *winkeys_togglebutton;
        GtkToggleButton *grab_mouse_togglebutton;

        GtkToggleButton *cheat_togglebutton;
        GtkEntry *extra_options;

        GnomeColorPicker *correct_colorpicker;
        GnomeColorPicker *incorrect_colorpicker;
        GnomeColorPicker *missing_colorpicker;
        GnomeColorPicker *may_work_colorpicker;
        GnomeColorPicker *favourite_colorpicker;
        GtkToggleButton *favourite_overrides_togglebutton;
        GnomeFontPicker *fontpicker;
        GtkCList *example_list;
        GtkEntry *fontentry;

        GtkCList *avail_col_list;
        GtkCList *select_col_list;

        GtkEntry *xmame_exe_entry;
        GtkEntry *rom_path_entry;
        GtkEntry *screenshots_dir_entry;
        GtkEntry *flyers_dir_entry;
        GtkEntry *cabinets_dir_entry;
        GtkEntry *highscore_dir_entry;
        GtkEntry *gamehistory_path_entry;
        GtkEntry *cheatfile_path_entry;
        GtkEntry *playback_dir_entry;

        GtkEntry *rom_url_entry;
        GtkEntry *screenshot_url_entry;
        GtkEntry *flyer_url_entry;
        GtkEntry *cabinet_url_entry;
};

struct Image {
        GdkPixbuf *default_image;
        GdkPixbuf *image;
};

typedef struct _ImageCanvas {
        GtkWidget *canvas;
        GnomeCanvasItem *image_item;
        gchar *name;
        struct Image imagestruct;
} ImageCanvas;

struct MainWindowWidgets appwin;
struct PrefsWindowWidgets prefswin;

GList *CListColumnLabels;

ImageCanvas screenshot_canvas;
ImageCanvas flyer_canvas;
ImageCanvas cabinet_canvas;

GtkWidget *main_window;
GtkWidget *prefs_window;
GtkWidget *filter_dialog;
GtkWidget *fileselection;
GtkWidget *reportdruid;


GtkWidget *progresswindow;
GtkWidget *progresswindow_progressbar;

gboolean key_pressed;


void startup_gui(GtkWidget * widget);
void grustibus_appbar_set_default(void);
void set_icon(GtkWidget * widget, const gchar * name);
void prefs_startup(GtkWidget * widget);
void filter_startup(GtkWidget * widget);
void prefs_reset(GtkWidget * widget);
void filter_dialog_reset(void);
void rom_selected(GtkCList * list, gint row);
void build_new_game_list(void);
void grustibus_set_progress (gfloat percent);  
void rebuild_game_list(void);
void set_sort_by_column (GtkCList *clist, gint column); 
void populate_column_lists(GList * selected_columns, GtkCList *sel_list, GtkCList *avail_list);
void rebuild_prefs_list(GtkWidget * prefs_window);
void build_clist(GtkCList * list);
void update_gamelist_row (GtkCList * list, gint row);
void create_gamelist_widget(GtkCList ** list);
void create_column_widgets(void);
void set_prefs_widgets(GtkWidget * prefs_window);
void set_game_settings_widgets(void);
void set_game_info_widgets(void);
gboolean set_game_history(const gchar * game);
void set_notebooktab_status(gint tab_num, gboolean status);
void notebooktab_status_show_all_but(gint tab_num);
void notebooktab_status_hide_all(void);
void draw_image(ImageCanvas * imagestruct, const gchar * game,
                const gchar * imagefile);
void lookup_game(const gchar * search_string);
void start_game(gint row);
void iconify_window(void);
void deiconify_window(void);
void grustibus_warning(const gchar * format, ...);
gboolean grustibus_question(gchar **button_array, const gchar *format, ...);
void set_busy_cursor(gboolean state);
void select_game(gint pos);
gint grustibus_clist_compare_func(GtkCList * clist, gconstpointer ptr1,
                                  gconstpointer ptr2);
// vim: ai sw=8 ts=8
