/*
 * mameio.c -- Takes care of all the interaction with mame.
 *
 * gRustibus (c) 2000-2001 Kjetil Thuen <kjetilt@users.sourceforge.net>
 * MAME frontend for the GNOME desktop environment.
 *
 * gRustibus is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * This code is indented with GNU indent using k&r style with 8 character
 * indents as suggested by the GNOME project. Please keep it that way.
 * ($ indent -kr -ts8 -i8 *.[ch])
 *
 * Last modified: 28 May 2001 <kjetilt@users.sourceforge.net>
 *
 */

#include "grustibus.h"
#include "mameio.h"
#include "fileio.h"
#include "gui.h"

/* create list parses the output of xmame -listinfo and xmame -verifyroms to
 * create the list of games. Most of the code in this function is taken from
 * gmame, whos author I can't seem to find at the moment (come forward :) )
*/
void create_list()
{
        FILE *xmame_info;
        FILE *xmame_vrfy;
        FILE *xmame_drv;
        gchar line[500];
        gchar *infocmd = NULL;
        gchar *vrfycmd = NULL;
        gchar *drvcmd = NULL;

        gchar *romname = NULL;
        gchar *gamename = NULL;
        gchar *year = NULL;
        gchar *manu = NULL;
        gchar *cloneof = NULL;
        gchar *romof = NULL;
        gchar *input = NULL;
        gchar *video = NULL;
        gchar *chip[16];
        gchar *driver_status = NULL;

        gchar *driver = NULL;
        gchar *status = NULL;

        gchar *p = NULL;
        gchar *keyword = NULL;
        gchar *value = NULL;
        gchar *verifyname = NULL;
        gchar *drivername = NULL;

        gchar **tmp_array;

        RomEntry *rom;

        gint i = 0;
        gint tmp_counter = 0;
        gint done_roms = 0;
        gfloat done;

        romname = g_malloc0(MAX_ROMNAME);
        gamename = g_malloc0(MAX_GAMENAME);
        year = g_malloc0(MAX_YEAR);
        manu = g_malloc0(MAX_MANU);
        cloneof = g_malloc0(MAX_ROMNAME);
        romof = g_malloc0(MAX_ROMNAME);
        input = g_malloc0(200);
        status = g_malloc0(200);
        driver = g_malloc0(MAX_ROMNAME);
        for (tmp_counter = 0; tmp_counter < 16; tmp_counter++) {
                chip[tmp_counter] = NULL;
        }
        driver_status = g_malloc0(200);
        video = g_malloc0(200);

        /* Clean the existing list */
        g_list_foreach(roms, (GFunc) g_free, NULL);
        g_list_free(roms);
        roms = NULL;
        
        check_xmame_exe();
        if (!xmame_version_ok) {
                g_free(romname);
                g_free(gamename);
                g_free(year);
                g_free(manu);
                g_free(cloneof);
                g_free(romof);
                g_free(input);
                g_free(video);
                tmp_counter = 0;
                while ((chip[tmp_counter] != NULL)) {
                        g_free(chip[tmp_counter]);
                        tmp_counter++;
                }
                g_free(driver_status);
                g_free(status);
                supported_games = 0;
                rebuild_game_list();
                return;
        }


        /* Get number of supported games */
        makecmd_line("-list 2>/dev/null", &vrfycmd, NULL);
        xmame_info = popen(vrfycmd, "r");

        while (fgets(line, 500, xmame_info)) {
                p = line + 1;
                while (*p && (*p++ != ':'));

                value = p;
                i = 0;

                while (*p && (*p++ != '\n'))
                        i++;
                value[i] = 0;
        }
        supported_games = atoi(value);

        pclose(xmame_info);
        g_free(vrfycmd);
        /* Generate the list */
        makecmd_line("-listinfo 2>/dev/null", &infocmd, NULL);
        xmame_info = popen(infocmd, "r");

        makecmd_line("-verifyroms 2>/dev/null", &vrfycmd, NULL);
        xmame_vrfy = popen(vrfycmd, "r");

        makecmd_line("-listsourcefile 2>/dev/null", &drvcmd, NULL);
        xmame_drv = popen(drvcmd, "r");

        while (fgets(line, 500, xmame_info)) {
                line[strlen(line) - 1] = 0;
                if (!strcmp(line, "game (")) {
                        g_snprintf(romname, MAX_ROMNAME, _("Unknown"));
                        g_snprintf(gamename, MAX_GAMENAME, _("Unknown"));
                        g_snprintf(year, MAX_YEAR, "-");
                        g_snprintf(manu, MAX_MANU, _("Unknown"));
                        g_snprintf(cloneof, MAX_ROMNAME, "-");
                        g_snprintf(romof, MAX_ROMNAME, "-");
                        while (fgets(line, 500, xmame_info)) {
                                if (line[0] == ')')
                                        break;
                                p = line + 1;
                                keyword = p;
                                i = 0;

                                while (*p && (*p++ != ' '))
                                        i++;
                                keyword[i] = 0;


                                if (p[0] == '\"')
                                        *p = *p++;
                                value = p;

                                i = 0;

                                while (*p && (*p++ != '\n'))
                                        i++;

                                if (value[i - 1] == '\"')
                                        i--;
                                value[i] = '\0';

                                if (!strcmp(keyword, "name"))
                                        g_snprintf(romname, MAX_ROMNAME,
                                                   value);
                                if (!strcmp(keyword, "description"))
                                        g_snprintf(gamename, MAX_GAMENAME,
                                                   value);
                                if (!strcmp(keyword, "year"))
                                        g_snprintf(year, MAX_YEAR, value);
                                if (!strcmp(keyword, "manufacturer"))
                                        g_snprintf(manu, MAX_MANU, value);
                                if (!strcmp(keyword, "cloneof"))
                                        g_snprintf(cloneof, MAX_ROMNAME,
                                                   value);
                                if (!strcmp(keyword, "romof"))
                                        g_snprintf(romof, MAX_ROMNAME,
                                                   value);
                                if (!strcmp(keyword, "chip")) {
                                        tmp_counter = 0;
                                        /* Put it in the next free chip[] slot */
                                        while ((chip[tmp_counter] != NULL)
                                               && tmp_counter < 16)
                                                tmp_counter++;
                                        if ((chip[tmp_counter] == NULL))
                                                chip[tmp_counter] =
                                                    g_strdup(value);
                                }
                                if (!strcmp(keyword, "video"))
                                        g_snprintf(video, 200, value);
                                if (!strcmp(keyword, "input"))
                                        g_snprintf(input, 200, value);
                                if (!strcmp(keyword, "driver"))
                                        g_snprintf(driver_status, 200,
                                                   value);

                        }
                        /* Get romstatus */
                        while (TRUE) {
                                if (!fgets(line, 500, xmame_vrfy))
                                        break;
                                if (strstr(line, "romset")) {
                                        verifyname = line + 7;
                                        p = verifyname;
                                        i = 0;

                                        while (*p && (*p++ != ' '))
                                                i++;

                                        verifyname[i] = 0;
                                        i++;

                                        value = line + 7 + i;
                                        i++;

                                        while (*p && (*p++ != '\n'))
                                                i++;

                                        value[i] = '\0';
                                        g_snprintf(status, 200, value);
                                        break;
                                }
                        }
                        /* Get drivername */
                        while (TRUE) {
                                if (!fgets(line, 500, xmame_drv))
                                        break;

                                drivername = line + 1;
                                p = drivername;
                                i = 0;

                                while (*p && (*p++ != '/') && (*p != '\n'))
                                        i++;
                                /* If no '/' was found, this line is not of the
                                 * wanted kind, skip to next */
                                if (*p != '\n') {
                                        i++;
                                        while (*p && (*p++ != '/'))
                                                i++;
                                        i++;

                                        value = line + i + 1;
                                        i = 0;

                                        while (*p && (*p++ != '.')
                                               && (*p != '\n'))
                                                i++;

                                        value[i] = '\0';

                                        if (i > 0) {
                                                g_snprintf(driver,
                                                           MAX_ROMNAME,
                                                           value);
                                                break;
                                        }
                                }
                        }
                        rom = g_malloc(sizeof(RomEntry));
                        if (rom == NULL)
                                g_warning
                                    ("Out of memory while generating gamelist");

                        /* Setting som default values */
                        g_snprintf(rom->cpu1, MAX_ROMNAME, "-");
                        g_snprintf(rom->cpu2, MAX_ROMNAME, "-");
                        g_snprintf(rom->cpu3, MAX_ROMNAME, "-");
                        g_snprintf(rom->cpu4, MAX_ROMNAME, "-");
                        g_snprintf(rom->sound1, MAX_ROMNAME, "-");
                        g_snprintf(rom->sound2, MAX_ROMNAME, "-");
                        g_snprintf(rom->sound3, MAX_ROMNAME, "-");
                        g_snprintf(rom->sound4, MAX_ROMNAME, "-");
                        g_snprintf(rom->control, MAX_CONTROL, "-");
                        rom->num_buttons = 0;
                        rom->num_players = 0;


                        g_snprintf(rom->romname, MAX_ROMNAME,
                                   "%s", romname);
                        g_snprintf(rom->gamename, MAX_GAMENAME,
                                   "%s", gamename);
                        g_snprintf(rom->year, MAX_YEAR, "%s", year);
                        g_snprintf(rom->manu, MAX_MANU, "%s", manu);
                        g_snprintf(rom->cloneof, MAX_ROMNAME,
                                   "%s", cloneof);
                        g_snprintf(rom->romof, MAX_ROMNAME, "%s", romof);
                        g_snprintf(rom->driver, MAX_ROMNAME, "%s", driver);


                        tmp_counter = 0;
                        while ((chip[tmp_counter] != NULL)
                               && (tmp_counter < 16)) {
                                tmp_array =
                                    g_strsplit(chip[tmp_counter], " ",
                                               MAX_CPU);
                                if (!strcmp(tmp_array[3], "name")) {
                                        if (!strcmp(tmp_array[2], "cpu")) {
                                                if (!strcmp
                                                    (rom->cpu1, "-"))
                                                        g_snprintf(rom->
                                                                   cpu1,
                                                                   MAX_CPU,
                                                                   "%s",
                                                                   tmp_array
                                                                   [4]);
                                                else if (!strcmp
                                                         (rom->cpu2, "-"))
                                                        g_snprintf(rom->
                                                                   cpu2,
                                                                   MAX_CPU,
                                                                   "%s",
                                                                   tmp_array
                                                                   [4]);
                                                else if (!strcmp
                                                         (rom->cpu3, "-"))
                                                        g_snprintf(rom->
                                                                   cpu3,
                                                                   MAX_CPU,
                                                                   "%s",
                                                                   tmp_array
                                                                   [4]);
                                                else if (!strcmp
                                                         (rom->cpu4, "-"))
                                                        g_snprintf(rom->
                                                                   cpu4,
                                                                   MAX_CPU,
                                                                   "%s",
                                                                   tmp_array
                                                                   [4]);
                                        }
                                        if (!strcmp(tmp_array[2], "audio")) {
                                                if (!strcmp
                                                    (rom->sound1, "-"))
                                                        g_snprintf(rom->
                                                                   sound1,
                                                                   MAX_CPU,
                                                                   "%s",
                                                                   tmp_array
                                                                   [4]);
                                                else if (!strcmp
                                                         (rom->sound2,
                                                          "-"))
                                                        g_snprintf(rom->
                                                                   sound2,
                                                                   MAX_CPU,
                                                                   "%s",
                                                                   tmp_array
                                                                   [4]);
                                                else if (!strcmp
                                                         (rom->sound3,
                                                          "-"))
                                                        g_snprintf(rom->
                                                                   sound3,
                                                                   MAX_CPU,
                                                                   "%s",
                                                                   tmp_array
                                                                   [4]);
                                                else if (!strcmp
                                                         (rom->sound4,
                                                          "-"))
                                                        g_snprintf(rom->
                                                                   sound4,
                                                                   MAX_CPU,
                                                                   "%s",
                                                                   tmp_array
                                                                   [4]);
                                        }
                                }
                                g_strfreev(tmp_array);
                                tmp_counter++;
                        }

                        tmp_counter = 0;
                        while ((chip[tmp_counter] != NULL)) {
                                g_free(chip[tmp_counter]);
                                chip[tmp_counter] = NULL;
                                tmp_counter++;
                        }

                        tmp_array = g_strsplit(video, " ", MAX_CPU);
                        if (!strcmp(tmp_array[2], "vector")) {
                                rom->vector = TRUE;
                        } else {
                                rom->vector = FALSE;
                        }

                        g_strfreev(tmp_array);


                        tmp_array = g_strsplit(input, " ", MAX_CPU);
                        tmp_counter = 0;
                        while ((tmp_array[tmp_counter + 1] != NULL)) {
                                if (!strcmp
                                    (tmp_array[tmp_counter], "players"))
                                        rom->num_players =
                                            atoi(tmp_array
                                                 [tmp_counter + 1]);
                                if (!strcmp
                                    (tmp_array[tmp_counter], "control"))
                                        g_snprintf(rom->control,
                                                   MAX_CONTROL,
                                                   tmp_array[tmp_counter +
                                                             1]);
                                if (!strcmp
                                    (tmp_array[tmp_counter], "buttons"))
                                        rom->num_buttons =
                                            atoi(tmp_array
                                                 [tmp_counter + 1]);
                                tmp_counter++;
                        }
                        g_strfreev(tmp_array);

                        tmp_array =
                            g_strsplit(driver_status, " ", MAX_CPU);
                        if ((tmp_array[2])
                            && !strcmp(tmp_array[2], "good")) {
                                rom->working = TRUE;
                        } else {
                                rom->working = FALSE;
                        }
                        g_strfreev(tmp_array);

                        if (!strcmp(status, "correct\n"))
                                rom->status = CORRECT;
                        else if (!strcmp(status, "incorrect\n"))
                                rom->status = INCORRECT;
                        else if (!strcmp(status, "not found\n"))
                                rom->status = MISSING;
                        else
                                rom->status = MAY_WORK;

                        grustibus_debug
                            ("Added game to list:\nRomname: %s\nGamename: %s\nYear: %s\nManufacurer: %s\nCloneof: %s\nRomOf: %s\nStatus: %sCPU1: %s\nCPU2: %s\nCPU3: %s\nCPU4: %s\nSound1: %s\nSound2: %s\nSound3: %s\nSound4: %s\nControls: %s\nNumber of players: %i\nNumber of buttons %i\nVector: %s\nDriver: %s\nGood Driver: %s",
                             romname, gamename, year, manu, cloneof, romof,
                             status, rom->cpu1, rom->cpu2, rom->cpu3,
                             rom->cpu4, rom->sound1, rom->sound2,
                             rom->sound3, rom->sound4, rom->control,
                             rom->num_players, rom->num_buttons,
                             rom->vector ? "Yes" : "No", rom->driver,
                             rom->working ? "Yes" : "No");
                        roms = g_list_prepend(roms, (gpointer) rom);
                        done_roms++;
                        done = (gfloat) ((gfloat) (done_roms) /
                                         (gfloat) (supported_games));
                        
                        grustibus_set_progress(done);
                }

        }

        pclose(xmame_info);
        pclose(xmame_vrfy);
        pclose(xmame_drv);
        g_free(vrfycmd);
        g_free(infocmd);

        g_free(romname);
        g_free(gamename);
        g_free(year);
        g_free(manu);
        g_free(cloneof);
        g_free(romof);
        g_free(input);
        g_free(video);
        g_free(status);
        tmp_counter = 0;
        while ((chip[tmp_counter] != NULL)) {
                g_free(chip[tmp_counter]);
                tmp_counter++;
        }
        g_free(driver_status);

        if (done_roms > 1)
                roms = g_list_sort(roms, (GCompareFunc)
                                   grustibus_g_list_compare_func);

        load_catver();
        if (!load_favourites()) {
                save_favourites();
                load_favourites();
        }
        load_timesplayed();
        save_list();
        save_xmame_version();

        grustibus_set_progress(0.0);
        
        set_timestamp ();
        get_entries_for_multiple_choice_filters ();
}

void real_start_game(const gchar * game, RomEntry * romdata)
{
        FILE *command;
        gchar *exec;

        makecmd_line(game, &exec, romdata);
        command = popen(exec, "w");
        /* Send a newline to *command in case xmame wants the user to "press any key" */
        fprintf(command, "\n");
        pclose(command);
        g_free(exec);
}

void playback_game(const gchar * file, const gchar * game, RomEntry * romdata)
{
        FILE *command;
        gchar *exec;
        gchar *extra_cmd_line = g_malloc(300);
        gchar *hiscoredirbackup = NULL;

        if (fopen(file, "r")) {
                if (general_prefs.playback_no_highscore) {
                        hiscoredirbackup =
                            g_strdup(general_prefs.highscore_dir);
                        g_snprintf(general_prefs.highscore_dir, MAX_DIR,
                                   "/dev/null/ ");
                }
                g_snprintf(extra_cmd_line, 300, " -playback \"%s\" %s",
                           file, game);
		makecmd_line(extra_cmd_line, &exec, romdata);

                iconify_window();
                command = popen(exec, "w");
                fprintf(command, "\n");
                pclose(command);
                deiconify_window();

                if (general_prefs.playback_no_highscore) {
                        g_free(general_prefs.highscore_dir);
                        general_prefs.highscore_dir =
                            g_strdup(hiscoredirbackup);
                        g_free(hiscoredirbackup);
                }
                g_free(exec);
        } else
                grustibus_warning(_("Could not open playbackfile %s"),
                                  file);
        g_free(extra_cmd_line);
}


void record_game(const gchar * file, const gchar * game, RomEntry * romdata)
{
        FILE *command;
        gchar *exec;
        gchar *hiscorebackup = NULL;
        gchar *extra_cmd_line = g_malloc(300);

        if (fopen(file, "w")) {
                if (general_prefs.playback_no_highscore) {
                        hiscorebackup =
                            g_strdup(general_prefs.highscore_dir);
                        g_snprintf(general_prefs.highscore_dir, MAX_DIR,
                                   "/dev/null/");
                }
                g_snprintf(extra_cmd_line, 300, " -record \"%s\" %s", file,
                           game);

                iconify_window();
		makecmd_line(extra_cmd_line, &exec, romdata);
                command = popen(exec, "w");
                fprintf(command, "\n");
                pclose(command);
                deiconify_window();

                if (general_prefs.playback_no_highscore) {
                        g_free(general_prefs.highscore_dir);
                        general_prefs.highscore_dir =
                            g_strdup(hiscorebackup);
                        g_free(hiscorebackup);
                }
                g_free(exec);
        } else
                grustibus_warning(_("Could not write to file %s."), file);

        g_free(extra_cmd_line);
}

gboolean check_xmame_exe()
{
        FILE *command;
        gboolean changed = FALSE;       // This is the return value
        gchar *exec = NULL;
        gchar line[255];
        gchar *xmame_version_string = NULL;
        gchar *tmp = NULL;
        gint i, major, release, versionchk;
        gchar *minor, *versionchk_string = NULL;
        gchar **version_array = NULL;

        xmame_version_ok = FALSE;

        /* Check if the xmame executable has changed */
        /* TODO: Expand mame executable to full path if the user has simply entered xmame */
        /* just check if it has an absolute path, get the PATH variable, split it and go through each part */
        /* Maybe set the prefs to the absolute path after */
        if (!fopen(general_prefs.xmame_exe, "r")) {
                grustibus_warning(_
                                  ("Could not find the xmame executable.\nPlease check your preferences."));

                /* Remove the version currently defined in prefs */
                if (general_prefs.xmame_major ||
                    general_prefs.xmame_minor ||
                    general_prefs.xmame_release ||
                    general_prefs.xmame_display_target) {
                        changed = TRUE;
                }
                general_prefs.xmame_major = 0;
                if (general_prefs.xmame_minor)
                        g_free(general_prefs.xmame_minor);
                general_prefs.xmame_minor=NULL;
                general_prefs.xmame_release = 0;
                if (general_prefs.xmame_display_target)
                        g_free(general_prefs.xmame_display_target);
                general_prefs.xmame_display_target=NULL;
        } else {
                /* Get output of xmame -version */
                exec =
                    g_strconcat(general_prefs.xmame_exe,
                                " -version 2>/dev/null", NULL);
                grustibus_debug("Running \"%s\" to get version info.",
                                exec);
                command = popen(exec, "r");

                /* Chech xmame display target */
                while (fgets(line, 254, command) && !xmame_version_ok) {
                        tmp = line + 1;
                        while ((*tmp && (*tmp++ != '('))
                               && (*tmp != '\n'));
                        if (*tmp != '\n') {
                                xmame_version_string = tmp;
                                i = 0;

                                while (*tmp && (*tmp++ != ')'))
                                        i++;
                                xmame_version_string[i] = 0;
                        }

                        /* Set the xmame display_target in the general_prefs struct */
                        grustibus_debug("xmame display target: %s",
                                        xmame_version_string);
                        if (xmame_version_string) {
                                g_free(general_prefs.xmame_display_target);
                                general_prefs.xmame_display_target =
                                    g_strdup(xmame_version_string);
                                /* Warn the user that the ggi display target need to be root to go fullscreen */
                                if (global_settings.fullscreen &&
                                    (int) getuid() &&
                                    !strcmp
                                    (general_prefs.xmame_display_target,
                                     "ggi"))
                                        grustibus_warning(_
                                                          ("Please note that you have to be root to use\nxmame.ggi in fullscreen mode.\n Playing games as root is strongly discouraged."));
                        }

                        /* Check xmame version */
                        xmame_version_string = NULL;
                        i = 0;
                        if ((*tmp != '\n')) {
                                /* skip " version " */
                                tmp = tmp + 9;
                                xmame_version_string = tmp;
                                while (*tmp && (*tmp++ != '\n'))
                                        i++;
                                xmame_version_string[i] = 0;

                                version_array =
                                    g_strsplit(xmame_version_string, ".",
                                               254);

                                major = atoi(version_array[0]);
                                if (version_array[1])
                                        minor = g_strdup(version_array[1]);
                                else
                                        minor = g_strdup("00");
                                if (version_array[2])
                                        release = atoi(version_array[2]);
                                else
                                        release = 0;

                                grustibus_debug("xmame major: %i", major);
                                grustibus_debug("xmame minor: %s", minor);
                                grustibus_debug("xmame release: %i",
                                                release);

                                if (!general_prefs.xmame_minor ||
                                    general_prefs.xmame_major != major ||
                                    strcmp(general_prefs.xmame_minor,
                                           minor)
                                    || general_prefs.xmame_release !=
                                    release) {
                                        changed = TRUE;

                                        /* This is just for debuging */
                                        if (general_prefs.xmame_major !=
                                            major)
                                                grustibus_debug
                                                    ("xmame major version changed (%i -> %i)",
                                                     general_prefs.
                                                     xmame_major, major);
                                        if (!general_prefs.xmame_minor
                                            || strcmp(general_prefs.xmame_minor, minor))
                                                grustibus_debug
                                                    ("xmame minor version changed(%s -> %s)",
                                                     general_prefs.
                                                     xmame_minor, minor);
                                        if (general_prefs.xmame_release !=
                                            release)
                                                grustibus_debug
                                                    ("xmame version release changed(%i -> %i)",
                                                     general_prefs.
                                                     xmame_release,
                                                     release);

                                }
                                if (general_prefs.xmame_minor) g_free(general_prefs.xmame_minor);
                                general_prefs.xmame_major = major;
                                general_prefs.xmame_minor = minor;
                                general_prefs.xmame_release = release;

                                /* See if xmame is new enough. */
                                versionchk_string = g_malloc(3);
                                g_snprintf(versionchk_string, 3, "%s",
                                           minor);
                                versionchk = atoi(versionchk_string);
                                /*0.37 and up is ok */
                                if (versionchk >= 37)
                                        xmame_version_ok = TRUE;
                                else {
                                        /* The following four versions are ok */
                                        if (!strcmp(minor, "36b16") ||
                                            !strcmp(minor, "36rc1") ||
                                            !strcmp(minor, "36"))
                                                xmame_version_ok = TRUE;
                                }
                                g_free(versionchk_string);

                        }
                        g_strfreev(version_array);
                }
                pclose(command);
                g_free(exec);

                /* There is a problem with the installed xmame */
                if (!xmame_version_ok) {
                        if (xmame_version_string) {
                                grustibus_warning(_
                                                  ("You need a newer xmame version."));
                        } else {
                                /* If the version was not defined before, something has changed */
                                if (general_prefs.xmame_major ||
                                    general_prefs.xmame_minor ||
                                    general_prefs.xmame_release ||
                                    general_prefs.xmame_display_target) {
                                        changed = TRUE;
                                        grustibus_debug
                                            ("too old xmame executable is now installed (the previous check reported not installed or pre 0.36b15)");
                                }
                                general_prefs.xmame_major = 0;
                                if (general_prefs.xmame_minor)
                                        g_free(general_prefs.xmame_minor);
                                general_prefs.xmame_minor = NULL;
                                general_prefs.xmame_release = 0;
                                if (general_prefs.xmame_display_target)
                                        g_free
                                            (general_prefs.
                                             xmame_display_target);
                                general_prefs.xmame_display_target = NULL;
                                grustibus_warning(_
                                                  ("Could not check xmame version. This means either\nthat your xmame is way too old, way to new (check for\na new gRustibus version) or that it could not be run."));
                        }
                }
        }
        grustibus_debug("xmame (%s) version %i.%s.%i",
                        general_prefs.xmame_display_target,
                        general_prefs.xmame_major,
                        general_prefs.xmame_minor,
                        general_prefs.xmame_release);
        return (changed);
}


// vim: ai sw=8 ts=8
